#!/bin/sh
# 

trap "on_error; exit 1" INT QUIT TERM

innovaphone_tmp=""
innovaphone_app_tmp=""
app=""
xml_tmp=""
installed_packages=""
custom_declared=0

source ./functions.sh

function on_exit()
{
    if [ -n "${innovaphone_tmp}" ]; then
        rm -R -f ${innovaphone_tmp}
    fi
    if [ -n "${innovaphone_app_tmp}" ]; then
        rm -R -f ${innovaphone_app_tmp}
    fi
    if [ -n "${xml_tmp}" ]; then
        rm -f ${xml_tmp}
    fi
    on_exit_app
    if [ ${custom_declared} -eq 1 ]; then
        on_exit_custom
    fi
}

function on_error()
{
    if [ -n "${installed_packages}" ]; then
        apt-get -y --force-yes purge ${installed_packages}
	    apt-get autoremove
    fi
    if [ -n "${innovaphone_tmp}" ]; then
        /etc/init.d/lighttpd stop
        cp -p -R ${innovaphone_tmp}/* /var/www/innovaphone
        /etc/init.d/lighttpd start
    fi
    if [ -n "${innovaphone_app_tmp}" ]; then
        cp -p -R ${innovaphone_app_tmp}/* /var/www/innovaphone/apps/${app}
    fi
    if [ -n "${xml_tmp}" ]; then
        cp -p ${xml_tmp} /var/www/innovaphone/apps/${app}.xml
    fi
    on_error_app
    if [ ${custom_declared} -eq 1 ]; then
        on_error_custom
    fi
    on_exit
}


process_tmp=""
uninstall_tmp=""
install_tmp=""
functions_tmp=""
config_tmp=""
check_tmp=""
boot_tmp=""
boot_end_tmp=""
inno_logs_tmp=""
inno_rsyslog_tmp=""
inno_rsyslog_conf_tmp=""
lighty_tmp=""
rpcapd_tmp=""
innoseq_tmp=""
main_template_tmp=""
process=0
lighttpd=0
rsyslog_logs_tmp=""
pg_analyze_tmp=""
psql1_tmp=""
psql2_tmp=""
psql3_tmp=""
softenc_tmp=""

function on_exit_app()
{
    if [ -n "${process_tmp}" ]; then
        rm -f ${process_tmp}
    fi
    if [ -n "${uninstall_tmp}" ]; then
        rm -f ${uninstall_tmp}
    fi
    if [ -n "${functions_tmp}" ]; then
        rm -f ${functions_tmp}
    fi
    if [ -n "${config_tmp}" ]; then
        rm -f ${config_tmp}
    fi
    if [ -n "${check_tmp}" ]; then
        rm -f ${check_tmp}
    fi
    if [ -n "${inno_logs_tmp}" ]; then
        rm -f ${inno_logs_tmp}
    fi
    if [ -n "${rsyslog_logs_tmp}" ]; then
        rm -f ${rsyslog_logs_tmp}
    fi
    if [ -n "${inno_rsyslog_tmp}" ]; then
        rm -f ${inno_rsyslog_tmp}
    fi
    if [ -n "${inno_rsyslog_conf_tmp}" ]; then
        rm -f ${inno_rsyslog_conf_tmp}
    fi
    if [ -n "${boot_tmp}" ]; then
        rm -f ${boot_tmp}
    fi
    if [ -n "${boot_end_tmp}" ]; then
        rm -f ${boot_end_tmp}
    fi
    if [ -n "${lighty_tmp}" ]; then
        rm -f ${lighty_tmp}
    fi
    if [ -n "${rpcapd_tmp}" ]; then
        rm -f ${rpcapd_tmp}
    fi
    if [ -n "${install_tmp}" ]; then
        rm -f ${install_tmp}
    fi
    if [ -n "${innoseq_tmp}" ]; then
        rm -f ${innoseq_tmp}
    fi
    if [ -n "${main_template_tmp}" ]; then
        rm -f ${main_template_tmp}
    fi
    if [ -n "${pg_analyze_tmp}" ]; then
        rm -f ${pg_analyze_tmp}
    fi
    if [ -n "${psql1_tmp}" ]; then
        rm -f ${psql1_tmp}
    fi
    if [ -n "${psql2_tmp}" ]; then
        rm -f ${psql2_tmp}
    fi
    if [ -n "${psql3_tmp}" ]; then
        rm -f ${psql3_tmp}
    fi
    if [ -n "${softenc_tmp}" ]; then
        rm -f ${softenc_tmp}
    fi
}

function on_error_app()
{
    if [ -n "${process_tmp}" ]; then
        cp -p ${process_tmp} /var/www/innovaphone/process.fcgi
    fi
    if [ -n "${uninstall_tmp}" ]; then
        cp -p ${uninstall_tmp} /usr/local/bin/uninstall_script.sh
    fi
    if [ -n "${functions_tmp}" ]; then
        cp -p ${functions_tmp} /usr/local/bin/functions.sh
    fi
    if [ -n "${config_tmp}" ]; then
        cp -p ${config_tmp} /usr/local/bin/config.sh
    fi
    if [ -n "${check_tmp}" ]; then
        cp -p ${check_tmp} /etc/cron.d/innovaphone_check
    fi
    if [ -n "${inno_logs_tmp}" ]; then
        cp -p ${inno_logs_tmp} /etc/logrotate.d/innovaphone-logs
    fi
    if [ -n "${rsyslog_logs_tmp}" ]; then
        cp -p ${rsyslog_logs_tmp} /etc/logrotate.d/rsyslog
    fi
    if [ -n "${inno_rsyslog_tmp}" ]; then
        cp -p ${inno_rsyslog_tmp} /etc/rsyslog.d/innovaphone-syslog.conf
        /etc/init.d/rsyslog restart
    fi
    if [ -n "${inno_rsyslog_conf_tmp}" ]; then
        cp -p ${inno_rsyslog_conf_tmp} /etc/rsyslog.conf
        /etc/init.d/rsyslog restart
    fi
    if [ -n "${boot_tmp}" ]; then
        cp -p ${boot_tmp} /etc/init.d/bootscript
    fi
    if [ -n "${boot_end_tmp}" ]; then
        cp -p ${boot_end_tmp} /etc/init.d/zbootscript_end
    fi
    if [ -n "${lighty_tmp}" ]; then
        cp -p ${lighty_tmp} /etc/init.d/lighttpd
    fi
    if [ -n "${innoseq_tmp}" ]; then
        cp -p ${innoseq_tmp} /home/postgres/restore/
    fi
    if [ -n "${psql1_tmp}" ]; then
        cp -p ${psql1_tmp} /home/postgres/restore/
    fi
    if [ -n "${psql2_tmp}" ]; then
        cp -p ${psql2_tmp} /home/postgres/restore/
    fi
    if [ -n "${psql3_tmp}" ]; then
        cp -p ${psql3_tmp} /home/postgres/restore/
    fi
    if [ -n "${main_template_tmp}" ]; then
        cp -p ${main_template_tmp} /home/lighttpd/server_config/main_template
    fi
    if [ -n "${pg_analyze_tmp}" ]; then
        cp -p ${pg_analyze_tmp} /etc/cron.d/pg_analyze
    fi
    if [ -n "${rpcapd_tmp}" ]; then
        running=`ps aux | grep -v grep | grep rpcapd | wc -l`
        if [ ${running} -gt 0 ]; then
            ps aux | grep -v grep | grep rpcapd | awk '{ print $2 }' | xargs kill -9
        fi
        cp -p ${rpcapd_tmp} /usr/local/bin
        if [ ${running} -gt 0 ]; then
            /usr/local/bin/rpcapd -p 2002 -4 -n -d > /dev/null 2>&1
        fi
    fi
    if [ -n "${install_tmp}" ]; then
        cp -p ${install_tmp} /usr/local/bin/install_script.sh
    fi
    if [ -n "${softenc_tmp}" ]; then
        cp -p ${softenc_tmp} /usr/local/bin/softenc.o
    fi
}
# if your're using this script, you have to provide an on_error function
# and you must have included functions.sh

# $1 either install or upgrade
# $2 empty to install all packages inside 'packages' or a string with package names, e.g. "php5-pgsql php5-gd"
function install_packages() 
{
    sources_list="/etc/apt/sources.list"
    if [ -d packages ]; then
        if [ -n "$(dpkg -l '*' | grep '^i[^iU]')" -o -n "$(dpkg -C)" ]; then
            dpkgl=`dpkg -l '*' | grep '^i[^iU]'`
            dpkgc=`dpkg -C`
            log_error "Debian packages can't be installed: ${dpkgl} ${dpkgc}" 
            on_error
            exit 1
        fi
        cd packages
        if [ "$2" ]; then
            installed_packages="$2"
        else
            installed_packages=`ls -1 *.deb | cut -d"_" -f1 | xargs`
        fi
        cd ..
        if [ ! -n "${installed_packages}" ]; then
            log_msg "No debian packages to install"
        else
            log_msg "Install debian packages ${installed_packages}"
            apt-ftparchive packages packages/ > packages/Packages
            sed -e 's/packages\///' -i packages/Packages
            gzip -9c packages/Packages > packages/Packages.gz
            rm packages/Packages
            mv packages/Packages.gz /var/cache/apt/archives
            mv packages/*.deb /var/cache/apt/archives
            mv ${sources_list} ${sources_list}.tmp
            echo "deb file:///var/cache/apt/archives /" > ${sources_list}
            apt-get update
            tmp=""
            if [ "$1" = "install" ]; then
                installed_tmp=""
                for d in $installed_packages; do
                    p_installed=`dpkg -l | egrep -oe "[a-zA-Z]{2}[ ]*$d"`
                    if [ -z "$p_installed" ]; then
                        installed_tmp="$installed_tmp $d"
                    else
                        log_msg "Package $d already installed"
                    fi
                done
                apt-get -y --force-yes install ${installed_packages}
                apt_res=$?
                installed_packages=$installed_tmp
            elif [ "$1" = "upgrade" ]; then
                installed_packages=""
                tmp=`apt-get -y --force-yes upgrade`
                apt_res=$?
                echo $tmp
            fi
            apt-get clean
            mv ${sources_list}.tmp ${sources_list}
            if [ ! ${apt_res} -eq 0 ]; then
                log_error "Installation of debian packages failed"
                on_error
                exit 1
            fi
            if echo $tmp | grep "The following packages have been kept back" >/dev/null
            then
                if [ "$1" = "upgrade" ]; then
                    missing=`echo $tmp | sed 's/.*have been kept back: \+\(.*\) \+The \+following \+packages \+will \+be \+upgraded.*/\1/'`
                    log_msg "Some package dependencies are missing!"
                    tmp_file="/var/www/innovaphone/log/missing_packages.log"			 
                    echo ${missing} > $tmp_file
                    chmod 755 $tmp_file
                    chown www-data:www-data $tmp_file
                fi
            fi
        fi
    else
        log_msg "No debian packages to install"
    fi
}

app=image
chmod 755 *.sh
./check_platform.sh platform
if [ $? -ne 0 ]; then
    exit 1
fi
./check_version.sh 120014 /var/www/innovaphone/version 1 0 image 12r1/linux/image
if [ ! $? -eq 0 ]; then
    exit 1
fi
 
 
innovaphone_tmp=`mktemp -d /tmp/innovaphone_XXXXXXXX`
./php_template.sh www-data www-data web/innovaphone /var/www/innovaphone ${innovaphone_tmp}
 
 
tmp_dir=""

# will be called in on_error, automatically added by make.sh
function on_error_custom() 
{
    echo -n ""
}

# will be called in on_exit, automatically added by make.sh
function on_exit_custom()
{
    echo -n ""
}

# only for vm
vm_runs=0
if [ -n "$(uname -m | egrep -oe 'i[0-9]{1}86')" ]; then
    vm_runs=1
fi

# call package install at first
custom_declared=1
install_packages install

config_tmp=`mktemp /tmp/cfg_XXXXXXXX`
./cp_template.sh 700 root root scripts/runtime/config.sh /usr/local/bin/config.sh ${config_tmp}
inno_logs_tmp=`mktemp /tmp/inno_logs_XXXXXXXX`
./cp_template.sh 644 root root config/rsyslog/innovaphone-logs /etc/logrotate.d/innovaphone-logs ${inno_logs_tmp}

rpcapd_tmp=`mktemp /tmp/rpcapd_XXXXXXXX`
running=`ps aux | grep -v grep | grep rpcapd | wc -l`
if [ ${running} -gt 0 ]; then
    ps aux | grep -v grep | grep rpcapd | awk '{ print $2 }' | xargs kill -9
fi
./cp_template.sh 755 root root rpcapd /usr/local/bin/rpcapd ${rpcapd_tmp}
if [ ${running} -gt 0 ]; then
    /usr/local/bin/rpcapd -p 2002 -4 -n -d > /dev/null 2>&1
fi
softenc_tmp=`mktemp /tmp/softenc_XXXXXXXX`
./cp_template.sh 755 www-data www-data softenc.o /usr/local/bin/softenc.o ${softenc_tmp}

process=1
process_tmp=`mktemp /tmp/process_XXXXXXXX`
cp -p /var/www/innovaphone/process.fcgi ${process_tmp}
chmod 755 process.fcgi
chown www-data:www-data process.fcgi
mv process.fcgi /var/www/innovaphone/process.fcgi

lighttpd=1
mkdir lighttpd
mv lighttpd.tar.xz lighttpd
cd lighttpd
tar xf lighttpd.tar.xz
cd lighttpd*
make install
if [ $? -eq 0 ]; then
    log_msg "lighttpd updated"
else
    log_error "Update of lighttpd failed"
    on_error
    exit 1
fi
cd ../..

echo -n "dvl" > /var/www/innovaphone/dvl
 
 
echo -n 120268 > /var/www/innovaphone/version
echo -n 1 > /var/www/innovaphone/hotfix

# finish
if [ ${process} -eq 1 ] || [ ${lighttpd} -eq 1 ]; then
    if [ $1 ]; then
        echo 1 > /var/www/innovaphone/tmp/hotfix_lighttpd.restart
    else 
        log_msg "The web server will be restarted a few seconds after the installation!"
        log_msg "Installation was successful"
        sleep 10s
        php5-cgi /var/www/innovaphone/update_webserver_cfg.php
    fi
elif [ ! $1 ]; then
    log_msg "Installation was successful"
fi

on_exit
