#!/usr/bin/env bash
#
#

source /home/init/scripts/functions.sh
source /home/init/scripts/postgresql.sh
source /home/init/scripts/user.sh

ready_file=/var/www/innovaphone/ready.htm

function on_error()
{
    echo "{ \"error\":1 }" > $ready_file
}

function init_error()
{
    on_error
    throw_error "$1"
}

sda3_exists=`fdisk -l /dev/sda | grep '/dev/sda3'`

if [ "$sda3_exists" = "" ]; then
    init_error "partitions not yet initialized!"
fi

# let's wait for NTP time
ntp_received=0
for i in {1..30}
do
    date=`date`
    if [[ $date == *1970* ]]
    then
        sleep 1s
    else
        ntp_received=1
        break
    fi
done
if [ $ntp_received -eq 1 ] 
then
    log_msg "NTP time received after $i seconds"
else
    log_msg "No NTP time received"
fi

# at first remove init script from startup
update-rc.d init remove
rm -f /etc/init.d/init

install_file=/var/www/innovaphone/install_running

cron_table="/var/spool/cron/crontabs/root"
cron_table_innovaphone="/etc/cron.d/innovaphone_backup"
cron_table_innovaphone_check="/etc/cron.d/innovaphone_check"
cron_table_lighttpd_cleanup="/etc/cron.d/lighttpd_upload_cleanup"
cron_tmp_table="cron_tmp.table"
log_file="/var/log/init_install.log"
vm_runs=0

rm -f /dev/fd0

echo "set nocompatible" >> /etc/vim/vimrc
ln -f -s /bin/bash /bin/sh
ln -f -s /bin/dash /bin/sh.distrib
if [ -d "/var/cache/man" ]; then
    chown -R man:root "/var/cache/man"
fi
mandb -pq

# disable apt install of recommended packages
echo "APT::Install-Recommends \"0\";" > /etc/apt/apt.conf
echo "APT::Install-Suggests \"0\";" >> /etc/apt/apt.conf

# disable IPV6
echo net.ipv6.conf.all.disable_ipv6=1 > /etc/sysctl.d/disableipv6.conf

log_msg "Start installation script"

if [ -n "$(uname -m | egrep -oe 'i[0-9]{1}86')" ]; then
    vm_runs=1
    log_msg "script runs on a virtual machine"
else
    log_msg "script runs on an innovaphone device"
fi

if [ $vm_runs -eq 0 ]; then
    # Copy syslog and other files to fat partition in case something goes wrong...
    fat_part=/mnt/sda1
    cp /var/log/syslog $fat_part 2>/dev/null
    cp /var/log/syslog.* $fat_part 2>/dev/null
    cp /var/log/daemon.log* $fat_part 2>/dev/null
    cp /var/lib/dhcp/dhclient.eth0.leases $fat_part 2>/dev/null
    tmp_file=`mktemp $fat_part/boot.XXXXX`
    dmesg > $tmp_file
    echo "" >> $tmp_file
    echo "LOG:ifconfig: " >> $tmp_file
    ifconfig >> $tmp_file
    echo "" >> $tmp_file
    echo "LOG:running processes: " >> $tmp_file
    ps axwww >> $tmp_file
fi

version=`tr -d '\n\r ' < /var/www/innovaphone/version`
hotfix=`tr -d '\n\r ' < /var/www/innovaphone/hotfix`
log_msg "installing build ${version} SR${hotfix}"

# Change default language to english
if [ -f "/etc/profile" ]; then
    sed -e "s/de_DE/en_GB/g" < /etc/profile > /tmp/profile
    mv /tmp/profile /etc/profile
fi

# generate Packages.gz file
apt-ftparchive packages /var/cache/apt/archives > /tmp/Packages
sed -i -e 's/\/var\/cache\/apt\/archives\///' /tmp/Packages
gzip -9c /tmp/Packages > /var/cache/apt/archives/Packages.gz
rm /tmp/Packages
apt-get update

if [ -z "$(dpkg -l | grep sudo)" ]; then
    log_msg "Warning: sudo not installed"
fi
dos2unix_av=`type dos2unix | grep 'type:'`
if [ -n "$dos2unix_av" ]; then
    log_msg "Warning: dos2unix not available"
fi

if [ -z "$(dpkg -l | egrep -e 'ntp[ \t]+')" ]; then
    log_msg "Warning: ntpd daemon not running"
fi
# disable removable of manual configured NTP servers
if [ -f /etc/dhcp/dhclient-exit-hooks.d/ntp ]; then
    sed -i "s/sed -r -e/# sed -r -e/g" /etc/dhcp/dhclient-exit-hooks.d/ntp
    sed -i "s/) >>\$tmp/  cat \$NTP_CONF\n        ) >>\$tmp/g" /etc/dhcp/dhclient-exit-hooks.d/ntp
fi
if [ -f /etc/ntp.conf ]; then
    #delete default servers
    sed -i -e "/^server / d" /etc/ntp.conf
fi

if [ -f "/etc/dhcp/dhclient.conf" ]; then
    if ! grep "ntp-servers" < /etc/dhcp/dhclient.conf >/dev/null
    then
        log_msg "Warning: DHCP maybe not requesting ntp-servers"
        sed -e "s/^[ \t]*request[ \t]*/request ntp-servers, /" < /etc/dhcp/dhclient.conf > /etc/dhcp/dhclient.conf
    fi
else
    log_msg "Warning: DHCP conf file not found"
fi

chmod 775 /usr/local/bin
chmod +s /usr/local/bin
cp /home/init/scripts/functions.sh /usr/local/bin
# permission for root scripts, sudoers will allow www-data to execute them
chmod 700 /usr/local/bin/config.sh
chmod 700 /usr/local/bin/install_script.sh
chmod 700 /usr/local/bin/uninstall_script.sh
chmod 700 /usr/local/bin/create_devices.sh
chmod 755 /usr/local/bin/functions.sh

#change rights of process.fcgi
chmod 755 /var/www/innovaphone/process.fcgi
chown www-data:www-data /var/www/innovaphone/process.fcgi

# install lighttpd
mkdir /tmp/lighttpd
if [ $? -ne 0 ]; then
    init_error "lighttpd temp dir could not be created..."
fi
cd /tmp/lighttpd
tar xf /home/init/lighttpd.tar.xz
cd lighttpd*
make install
if [ $? -ne 0 ]; then
    init_error "lighttpd could not be installed..."
fi
cd ../..
rm -R -f lighttpd

# install php
if [ ! -d "/etc/php5" ]; then
    init_error "php was not installed..."
fi
cp /home/init/php/php.ini /etc/php5/cgi/php.ini
cp /home/init/php/php.ini /etc/php5/cli/php.ini
cp /home/init/php/xcache.ini /etc/php5/conf.d/xcache.ini
if [ ! -d "/etc/lighttpd" ]; then
    mkdir /etc/lighttpd
fi
lighttpd_conf="/etc/lighttpd/lighttpd.conf"
cp /home/init/lighttpd/lighttpd.conf $lighttpd_conf
cp /home/init/lighttpd/lighttpd.initd /etc/init.d/lighttpd
chmod +x /etc/init.d/lighttpd
chmod 755 $lighttpd_conf

if [ ! -d "/usr/share/lighttpd" ]; then
    mkdir /usr/share/lighttpd
fi
cp /home/init/lighttpd/perl/* /usr/share/lighttpd

groupname=`cat $lighttpd_conf | egrep -oe "^[ \t]*server.groupname[ \t]*=[ \t]*"".*$""" | cut -d"=" -f2 | sed -e 's/^[^"]"//' -e 's/"*$//'`
username=`cat $lighttpd_conf | egrep -oe "^[ \t]*server.username[ \t]*=[ \t]*"".*$""" | cut -d"=" -f2 | sed -e 's/^[^"]"//' -e 's/"*$//'`

if test ! -d /var/log/lighttpd; then
    mkdir /var/log/lighttpd    
fi
chmod -R 755 /usr/share/lighttpd
chown -R $username:$groupname /usr/share/lighttpd
chown -R $username:$groupname /var/log/lighttpd

echo "1" > $install_file
chown $username:$groupname $install_file

if [ ! -d "/var/run/lighttpd" ]; then
    mkdir "/var/run/lighttpd"
fi
cp /home/init/lighttpd/webdav_props.db /var/run/lighttpd
chmod 644 /var/run/lighttpd/webdav_props.db

if [ -z "$(dpkg -l | grep libpam0g-dev)" ]; then
    init_error "dev package libpam not installed"
else
    if [ -f /usr/bin/pam-change-pw ]; then
        log_msg "pam-change-pw compiled, adding executable flag"
        chmod +x /usr/bin/pam-change-pw
    else
        init_error "pam-change-pw not successfully compiled"    
    fi
fi

if [ -z "$(dpkg -l | grep libfcgi)" ]; then
    init_error "install of fcgi packages not successful"    
fi
    
if [ ! -d "/var/cache/lighttpd/compress" ]; then
    mkdir -p "/var/cache/lighttpd/compress"
fi
if [ ! -d "/var/cache/upload" ]; then
    mkdir "/var/cache/upload"
fi
# Directory for uploads in php.ini
chown -R $username:$groupname /var/cache/upload
# Other lighttpd directories
chown -R $username:$groupname /var/cache/lighttpd
chown -R $username:$groupname /var/run/lighttpd

if [ -z "$dos2unix_av" ]; then
    find /var/www/innovaphone -type f -iname "*.php" | xargs dos2unix -f
fi
chown -R $username:$groupname /var/www

# Copy softenc
chmod +x /usr/local/bin/softenc.o
chown www-data:www-data /usr/local/bin/softenc.o

# install libharu
if [ ! -f "/home/init/libharu.tar.xz" ]; then
    init_error "install of fcgi packages not successful"    
else
    mkdir /tmp/libharu
    cd /tmp/libharu
    tar xf /home/init/libharu.tar.xz
    cd libharu*
    make install
    cd ../..
    rm -R -f libharu
fi

# Lighttpd files
if [ ! -d "/home/lighttpd" ]; then
    mkdir -p /home/lighttpd
fi
cp /home/init/lighttpd/lighttpd_htdigest.user /home/lighttpd
cp /home/init/lighttpd/webdav_htdigest.user /home/lighttpd
chmod 644 /home/lighttpd/*
cp -r /home/init/lighttpd/server_config /home/lighttpd
chown -R $username:$groupname /home/lighttpd

# Certificate
mkdir -p /home/root/ssl_cert/
mv /home/init/lighttpd/server.pem /home/root/ssl_cert
chown root:root /home/root/ssl_cert/server.pem
# make server certificate readable, that www-data can use curl to send client certificate
chmod 444 /home/root/ssl_cert/server.pem
chmod +x /home/root/ssl_cert/

mv /home/init/lighttpd/dhparams.pem /home/root/ssl_cert
chown root:root /home/root/ssl_cert/dhparams.pem
# make dh certificate readable, that www-data can use curl to send client certificate
chmod 444 /home/root/ssl_cert/dhparams.pem

mv /home/init/lighttpd/innovaphone-ca.pem /home/root/ssl_cert
chmod 400 /home/root/ssl_cert/innovaphone-ca.pem
chown root:root /home/root/ssl_cert/innovaphone-ca.pem

update-rc.d lighttpd defaults
/etc/init.d/lighttpd stop
if [ -f "/var/run/lighttpd.pid" ]; then
    rm -f /var/run/lighttpd.pid
fi
usermod -d /home/lighttpd www-data
/etc/init.d/lighttpd start
if [ -z "$(ps axwww | grep lighttpd | grep -v grep)" ] ; then
    init_error "Lighttpd not running"
fi

# Install boot scripts
if [ ! -f "/home/init/scripts/bootscript.sh" ]; then
    init_error "Boot script not available"
fi
chmod +x /home/init/scripts/bootscript.sh
cp /home/init/scripts/bootscript.sh /etc/init.d/bootscript
defaultstart=`cat /etc/inittab | grep "initdefault" | cut -d":" -f2 | tr -d "\r\n"`
lighttpdlevel=`ls -l /etc/rc${defaultstart}.d/ | grep lighttpd | awk '{print $9}' | egrep -oe "[0-9]+"`
if [ -f "/etc/rc${defaultstart}.d/S${lighttpdlevel}bootscript" ]; then
    rm -f /etc/rc${defaultstart}.d/S${lighttpdlevel}bootscript
fi
update-rc.d bootscript defaults

if [ ! -f "/home/init/scripts/bootscript_end.sh" ]; then
    init_error "End boot script not available"
fi
chmod +x /home/init/scripts/bootscript_end.sh
cp /home/init/scripts/bootscript_end.sh /etc/init.d/zbootscript_end
sshlevel=`ls -l /etc/rc${defaultstart}.d/ | grep ssh | awk '{print $9}' | egrep -oe "[0-9]+"`
if [ -f "/etc/rc${defaultstart}.d/S${sshlevel}zbootscript_end" ]; then
    rm -f /etc/rc${defaultstart}.d/S${sshlevel}zbootscript_end
fi
update-rc.d zbootscript_end defaults 02

deb_timezone=`date | egrep -oe 'CET[ \t]+1970' | wc -l`
if [ "$deb_timezone" -eq "1" ]; then
    echo "Europe/Berlin" > /etc/timezone
    dpkg-reconfigure --frontend noninteractive tzdata
    log_msg "Timezone is Europe/Berlin. You may change it later"
fi

log_msg "installing postgresql 9.1"
postgresql_install=`apt-get -y --force-yes install postgresql-9.1 lsb-release`
echo $postgresql_install
apt-get -y --force-yes install postgresql-common php5-pgsql

failed_inst=`echo $postgresql_install | egrep -oe "failed!"`
if [ -n "$failed_inst" ]; then
    init_error "Postgresql could not be installed/started. Check your log files."
fi

cluster=main
port_number=`echo $postgresql_install | egrep -o "Configuring[ ]+[^ ]+[ ]+to[ ]+use[ ]+port[ ]+[0-9]{4,5}" | cut -d" " -f6`
log_msg "Configure POSTGRESQL 9.1 $cluster port: $port_number"

pg_dropcluster --stop 9.1 $cluster
postgresql_install=`pg_createcluster -u postgres --start 9.1 $cluster -e utf8 --locale=en_GB.UTF-8 --lc-collate=C`

#
# Get here parameters from postgresql_install	
#
config_pg_dir=`echo $postgresql_install | tr "\011" " " | egrep -io "configuration[ ]*:[ ]*(/[^/, ]+)*" | cut -d" " -f2`
data_pg_dir=`echo $postgresql_install | tr "\011" " " | egrep -io "data[ ]*:[ ]*(/[^/)]+)*" | cut -d" " -f2`
config_pg_file=`echo $postgresql_install | egrep -o "Configuring[ ]+[^ ]+[ ]+to[ ]+use" | cut -d" " -f2`
installed_packages=$installed_packages" postgresql-9.1 postgresql-common" 

log_msg "CONFIG DIR: $config_pg_dir"
log_msg "CONFIGURATION: $config_pg_file"
log_msg "DATA DIR: $data_pg_dir"
log_msg "PORT NUMBER: $port_number"
log_msg "POSTGRESQL CLUSTER: $cluster"

cfg_tmp_dir=/tmp
pgsql_cfg="${config_pg_dir}/${config_pg_file}"
cp $pgsql_cfg $cfg_tmp_dir
cp "${config_pg_dir}/pg_hba.conf" $cfg_tmp_dir

postgresql_tmp=`mktemp $cfg_tmp_dir/pg_conf.$$.XXXX`
if [ $? -ne 0 ]; then
    init_error "Can't create temp config file, exiting..."
fi

# Listen to all addresses and filter them through pg_hba.conf
listen="listen_addresses='*'"

psql_is_listening=`cat $pgsql_cfg | tr "\011" " " | sed -n "/\(^[ ]*[;#]\+[ ]*\)listen_addresses[ ]*=[ ]*'/p" | wc -l`
if [ "$psql_is_listening" -eq "1" ]; then
    cat $pgsql_cfg | tr "\011" " " | sed "s/\(^[ ]*[;#]\+[ ]*\)listen_addresses[ ]*=[ ]*'[^']*'/$listen/" > $postgresql_tmp
    cp $postgresql_tmp $pgsql_cfg
else
    psql_is_listening=`cat /etc/postgresql/9.1/main/postgresql.conf | tr "\011" " " | sed -n "/\(^[ ]*\)listen_addresses[ ]*=[ ]*'/p" | wc -l`
    if [ "$psql_is_listening" -eq "0" ]; then
        log_msg "Warning: listen_addresses address variable does not exit in config file"
        echo "$listen " >> $pgsql_cfg
    else
        if [ "$psql_is_listening" -gt "1" ]; then
            log_msg "Warning: listen_addresses set more than once...Delete all and add a new one"
            cat $pgsql_cfg | tr "\011" " " | sed "/\(^[ ]*\)listen_addresses[ ]*=[ ]*'/ d" > $postgresql_tmp
            echo "$listen " >> $postgresql_tmp
            cp $postgresql_tmp $pgsql_cfg
        else
            cat $pgsql_cfg | tr "\011" " " | sed "s/\(^[ ]*\)listen_addresses[ ]*=[ ]*'[^']*'/$listen/" > $postgresql_tmp
            cp $postgresql_tmp $pgsql_cfg
        fi
    fi
fi

# Change effective cache size to 32 MB, default is 128 MB
cat $pgsql_cfg | tr "\011" " " | sed "s/\(^[ ]*[;#]*[ ]*\)effective_cache_size[ ]*=[ ]*[0-9]\+MB/effective_cache_size=32MB/" > $postgresql_tmp
cp $postgresql_tmp $pgsql_cfg

# Change standard_conforming_strings to on, default is off
cat $pgsql_cfg | tr "\011" " " | sed "s/\(^[ ]*[;#]*[ ]*\)standard_conforming_strings[ ]*=[ ]*[^;]*/standard_conforming_strings = on/" > $postgresql_tmp
cp $postgresql_tmp $pgsql_cfg

# Change autovacuum and track_counts to on
cat $pgsql_cfg | tr "\011" " " | sed "s/\(^[ ]*[;#]*[ ]*\)autovacuum[ ]*=[ ]*[^;]*/autovacuum = on/" > $postgresql_tmp
cp $postgresql_tmp $pgsql_cfg
cat $pgsql_cfg | tr "\011" " " | sed "s/\(^[ ]*[;#]*[ ]*\)track_counts[ ]*=[ ]*[^;]*/track_counts = on/" > $postgresql_tmp
cp $postgresql_tmp $pgsql_cfg

# Switch SSL support on
cat $pgsql_cfg | tr "\011" " " | sed "s/\(^[ ]*[;#]*[ ]*\)ssl[ ]*=[ ]*[^;]*/ssl = on/" > $postgresql_tmp
cp $postgresql_tmp $pgsql_cfg

config_port=`cat $pgsql_cfg | tr "\011" " " | egrep -o "^[ ]*port[ ]*=[ ]*[0-9]{4,5}" | cut -d"=" -f2 | cut -d"=" -f2 | sed -e 's/^[ ]*//' -e 's/[ ]*$//'`
if [ -z "$config_port" ]; then
    config_port=`cat $pgsql_cfg | tr "\011" " " | egrep -o "^[ ]*[;#]+[ ]*port[ ]*=[ ]*[0-9]{4,5}" | cut -d"=" -f2 | cut -d"=" -f2 | sed -e 's/^[ ]*//' -e 's/[ ]*$//'`
    log_msg "Warning: port variable exists but it is commented out! Add default port 5432" 
    cat $pgsql_cfg | tr "\011" " " | sed "s/\(^[ ]*[;#]\+[ ]*\)port[ ]*=[ ]*'[^']*'/port=5432/" > $postgresql_tmp
    cp $postgresql_tmp $pgsql_cfg
else
    if [ "$config_port" != "$port_number" ]; then
        log_msg "Port has been modified: New: $config_port Old: $port_number"
        port_number=$config_port
    fi
fi

cat $config_pg_dir/pg_hba.conf | tr "\011" " " | sed -e '/^[ ]*host.*127.0.0.1\/32/ d' > $postgresql_tmp
# Do this to check actual password through web interface
echo "host    postgres     postgres         127.0.0.1/32             md5" >> $postgresql_tmp
cp $postgresql_tmp $config_pg_dir/pg_hba.conf

# Add web user to db user and the other way around
if [ -z "$(type gpasswd | tr "\011" " " | grep 'type:[ ]*gpasswd')" ]; then
    gpasswd -a $username postgres
    gpasswd -a postgres $groupname
else
    if [ -z "$(type usermod | tr "\011" " " | grep 'type:[ ]*usermod')" ]; then
        usermod -aG postgres $username
        usermod -aG $groupname postgres
    else
        if [ -z "$(type adduser | tr "\011" " " | grep 'type:[ ]*adduser')" ]; then
            adduser $username postgres
            adduser postgres $groupname
        else
            log_msg "$username could not be added to postgres group"
        fi
    fi
fi

# User in postgres group can also modify this file
chmod 660 /etc/postgresql/9.1/main/pg_hba.conf

# md5 for postgres password
chk_psql=`type psql | tr "\011" " " | egrep -oe "^psql[ ]*is"`
if [ -z "$chk_psql" ]; then
    log_msg "Warning: Default postgres password could not be set. Please set a password through the web interface"
else
    change_password=`sudo -u postgres psql -U postgres -c "ALTER user postgres WITH PASSWORD 'postgres';"`
    if [ -n "$(echo $change_password | grep 'ALTER ROLE')" ]; then
        log_msg "Set default postgresql password"
    else
        log_msg "Warning: Default postgres password could not be set. Please set a password up through the web interface"
    fi
fi
if [ ! -f "/home/postgres/restore" ]; then
    mkdir -p /home/postgres/restore
fi    

if [ -d "/var/log/postgresql" ]; then
    chmod -R +r /var/log/postgresql
fi

# SSL certificate, use the Linux Application platform certificate
web_pem="/home/root/ssl_cert/server.pem"
web_crt="/home/root/ssl_cert/server.crt"
web_key="/home/root/ssl_cert/server.key"
server_crt="/var/lib/postgresql/9.1/main/server.crt"
server_key="/var/lib/postgresql/9.1/main/server.key"

if [ -f $web_crt -a -f $web_key ]; then
    rm -f $server_crt $server_key
    ln -s $web_crt server.crt
    ln -s $web_key server.key
    mv server.crt /var/lib/postgresql/9.1/main
    mv server.key /var/lib/postgresql/9.1/main
    chown postgres:postgres $web_crt $web_key
    chmod 400 $web_crt $web_key
    if [ ! -L "/var/lib/postgresql/9.1/main/server.crt" -o ! -L "/var/lib/postgresql/9.1/main/server.key" ]; then
        log_msg "Warning: SSL may not work"
    fi
else
    if [ -f $web_pem ]; then
        rm -f $server_crt $server_key
        rm -f $web_crt $web_key
        touch $web_crt 2>/dev/null
        touch $web_key 2>/dev/null

        if head -n 1 $web_pem | grep "CERTIFICATE" > /dev/null
        then
            ind_crt=1
        else
            ind_crt=0
        fi
        while read LINE
        do
            if [ $ind_crt -eq 1 ]; then
                echo $LINE >> $web_crt
                if echo $LINE | grep "END CERTIFICATE" > /dev/null
                then
                    ind_crt=0
                fi
            else
                echo $LINE >> $web_key
                if echo $LINE | grep "END RSA PRIVATE KEY" > /dev/null
                then
                    ind_crt=1
                fi
            fi
        done < $web_pem

        ln -s $web_crt server.crt
        ln -s $web_key server.key
        mv server.crt /var/lib/postgresql/9.1/main
        mv server.key /var/lib/postgresql/9.1/main
        chown postgres:postgres $web_crt $web_key
        chmod 400 $web_crt $web_key
        if [ ! -L "/var/lib/postgresql/9.1/main/server.crt" -o ! -L "/var/lib/postgresql/9.1/main/server.key" ]; then
            log_msg "Warning: SSL may not work"
        fi
    else
        init_error "Application platform certificate does not exist"
    fi
fi
log_msg "SSL certificate for postgresql installed"

/etc/init.d/postgresql restart
if [ -z "$(ps axwww | grep postgresql | grep -v 'grep')" ]; then
    log_msg "Warning: Postgresql service not running"
fi

# call create_user function of user.sh
create_user innovaphone

# call postgresql function of postgresql.sh
cd /home/init/database/
postgresql innovaphone $cfg_tmp_dir innovaphone_database.psql innovaphone
cd /

rm -f $cfg_tmp_dir/pg_*
rm -f $cfg_tmp_dir/$pgsql_cfg

mv /home/init/rootfs_packages /home/innovaphone/
chmod 755 /home/innovaphone/rootfs_packages

postfix_dir=/etc/postfix
postfix_1=main.cf
postfix_2=sender_relayhost.cf
postfix_3=sasl_password.cf
chown root:root $postfix_dir/$postfix_1
chown root:root $postfix_dir/$postfix_2
chown root:root $postfix_dir/$postfix_3
chmod 644 $postfix_dir/$postfix_1
chmod 644 $postfix_dir/$postfix_2
chmod 644 $postfix_dir/$postfix_3

postfix -c /etc/postfix set-permissions 
chown -R postfix:postfix /var/lib/postfix
update-rc.d postfix defaults
chown root:postdrop /usr/sbin/postdrop /usr/sbin/postqueue
chmod g+s /usr/sbin/postdrop /usr/sbin/postqueue

# install pear packages
if [ -z "$(pear list | grep 'Net_Socket')" ] ; then
    log_msg "Install pear Net_Socket"
    pear install /home/init/pear/Net_Socket-1.0.14.tgz
fi
if [ -z "$(pear list | grep 'Net_SMTP')" ] ; then
    log_msg "Install pear Net_SMTP"
    pear install /home/init/pear/Net_SMTP-1.6.2.tgz
fi
if [ -z "$(pear list | grep 'Mail ')" ] ; then
    log_msg "Install pear Mail"
    pear install /home/init/pear/Mail-1.2.0.tgz
fi
if [ -z "$(pear list | grep 'Mail_Mime')" ] ; then
    log_msg "Install pear Mail_Mime"
    pear install /home/init/pear/Mail_Mime-1.8.9.tgz
fi

apt-get clean

# set log permissions
chmod 644 /etc/logrotate.d/innovaphone-logs
chmod 644 /etc/logrotate.d/rsyslog

# apt-get update sometimes hangs too long, so we set a timeout
if [ -e "/etc/apt/apt.conf.d/70debconf" ]; then
    echo "" >> "/etc/apt/apt.conf.d/70debconf"
    echo "Acquire" >> "/etc/apt/apt.conf.d/70debconf"
    echo "{" >> "/etc/apt/apt.conf.d/70debconf"
    echo "	http" >> "/etc/apt/apt.conf.d/70debconf"
    echo "	{" >> "/etc/apt/apt.conf.d/70debconf"
    echo '		Timeout "120";' >> "/etc/apt/apt.conf.d/70debconf"
    echo "	};" >> "/etc/apt/apt.conf.d/70debconf"
    echo "};" >> "/etc/apt/apt.conf.d/70debconf"
fi
# Install from internet
if [ -e "/etc/apt/sources.list" ]; then
    echo "deb http://ftp.de.debian.org/debian wheezy main" > /etc/apt/sources.list
    echo "deb-src http://ftp.de.debian.org/debian wheezy main" >> /etc/apt/sources.list
    echo "deb http://security.debian.org/ wheezy/updates main" >> /etc/apt/sources.list
    echo "deb-src http://security.debian.org/ wheezy/updates main" >> /etc/apt/sources.list
fi

# Delete init directory
cd /home
rm -R -f /home/init

if [ ! -d "/tmp" ]; then
    mkdir /tmp
fi
chmod 777 /tmp
chmod a+t /tmp
if [ ! -d "/var/log" ]; then
    mkdir -p /var/log
fi

echo "# /etc/cron.d/innovaphone_backup" > ${cron_table_innovaphone}
chown root:root ${cron_table_innovaphone}
chmod 644 ${cron_table_innovaphone}
chown root:root ${cron_table_innovaphone_check}
chmod 644 ${cron_table_innovaphone_check}
chown root:root ${cron_table_lighttpd_cleanup}
chmod 644 ${cron_table_lighttpd_cleanup}

# increase memory limit on VM
if [ $vm_runs -eq 1 ]; then
    php_tmp=`mktemp /tmp/php.$$.XXXX`
    php_ini=/etc/php5/cgi/php.ini
    cat $php_ini | tr "\011" " " | sed "s/\(^[ ]*[;#]*[ ]*\)memory_limit[ ]*=[ ]*[0-9]\+M/memory_limit = 256M/" > $php_tmp
    cp $php_tmp $php_ini
    php_ini=/etc/php5/cli/php.ini
    cat $php_ini | tr "\011" " " | sed "s/\(^[ ]*[;#]*[ ]*\)memory_limit[ ]*=[ ]*[0-9]\+M/memory_limit = 256M/" > $php_tmp
    cp $php_tmp $php_ini
    rm $php_tmp
fi

# Zend stuff only for vm, as Zend is not available for the ARM platform
if [ $vm_runs -eq 1 ]; then
    php_ini=/etc/php5/cgi/php.ini
    log_msg "Installing Zend Loader"
    echo "" >>  ${php_ini}
    echo "zend_extension=/usr/lib/php5/ZendGuardLoader.so" >> ${php_ini}
    echo "" >> ${php_ini}
    echo "[Zend]" >> ${php_ini}
    echo "zend_loader.enable=1" >> ${php_ini}
    echo "zend_loader.license_path=/home/root/zend-licenses" >> ${php_ini}
    mkdir /home/root/zend-licenses
    chmod 644 /usr/lib/php5/ZendGuardLoader.so
    log_msg "Directory for Zend licenses: /home/root/zend-licenses"
    cp $php_ini /etc/php5/cli/php.ini
fi

# Add install script to crontab in new partition and disable sending mails on errors
echo 'MAILTO=""' >> ${cron_table}
# echo "*   *   *   *   *    /usr/local/bin/install_script.sh /var/www/innovaphone/log/appl_install.log >> /var/www/innovaphone/log/appl_install.log 2>&1" >> ${cron_table}

# It is important to rotate lighttpd logs since they can become really big...
echo "# /etc/cron.d/logrotate" > /etc/cron.d/logrotate
echo "*/15 * * * * root [ -x /usr/sbin/logrotate ] && /usr/sbin/logrotate /etc/logrotate.conf" >> /etc/cron.d/logrotate
chmod +x /etc/cron.d/logrotate

rm -f /var/run/lighttpd/php-fastcgi.socket-*
rm -f /var/run/lighttpd/fcgi-fastcgi.socket-*
if [ ! -d "/var/cache/upload" ]; then
    mkdir "/var/cache/upload"
fi

# Directory for uploads in php.ini
chown -R $username:$groupname "/var/cache/upload"
# Other lighttpd directories
chown -R $username:$groupname "/var/cache/lighttpd"
chown -R $username:$groupname "/var/run/lighttpd"

chmod 755 /usr/local/bin/rpcapd
dos2unix -f /usr/local/bin/config.sh
dos2unix -f /etc/init.d/bootscript

chmod -R 750 /var/www/

echo "<?xml version=\"1.0\" encoding=\"utf-8\"?>" > "/home/lighttpd/server_config/innovaphone.conf"
echo "<config/>" >> "/home/lighttpd/server_config/innovaphone.conf"
chown $username:$groupname "/home/lighttpd/server_config/innovaphone.conf"
chmod 600 "/home/lighttpd/server_config/innovaphone.conf"

# create quit file for openssl s_client tool
echo "quit\n" > "/home/lighttpd/quit.in"
/etc/init.d/lighttpd restart

# Avoid cron jobs to log into syslog file
sed -e "s/*.*;auth,authpriv.none/*.*;auth,authpriv,cron.none/" < "/etc/rsyslog.conf" > tmp.conf
if [ -f tmp.conf ]; then
    sed -e "s/$FileCreateMode\([ \t\]*\)0640/$FileCreateMode 0644/" < tmp.conf > "/etc/rsyslog.conf"
    rm -f tmp.conf
else
    log_msg "Warning: rsyslog changes not done"
fi

# Syslog will be needed to find out DHCP server address
if [ -e /var/log/syslog ]; then
    cp /var/log/syslog /tmp
    chown www-data:www-data /tmp/syslog
fi

if [ -f "/var/log/kern.log" ]; then
    chmod 644 /var/log/kern.log*
fi
if [ -f "/var/log/syslog" ]; then
    chmod 644 /var/log/syslog*
fi
if [ -f "/var/log/mail.log" ]; then
    chmod 644 /var/log/mail*
fi

if [ -z "$(cat /etc/vim/vimrc 2>/dev/null | egrep -e '(expandtab|tabstop=4|shiftwidth=4)')" ]; then    
    echo "set expandtab" >> "/etc/vim/vimrc"
    echo "set shiftwidth=4" >> "/etc/vim/vimrc"
    echo "set tabstop=4" >> "/etc/vim/vimrc"
fi
if [ -z "$(cat /etc/vim/vimrc 2>/dev/null | egrep -e 'nobackup')" ]; then    
    echo "set nobackup" >> "/etc/vim/vimrc"
fi

# restart some services
/etc/init.d/cron restart
/etc/init.d/rsyslog restart
/etc/init.d/postfix restart
/etc/init.d/ssh restart
/etc/init.d/ntp restart

rm -R -f /tmp/*

log_msg "Installation was successful"
echo "{ \"error\":0 }" > $ready_file
rm -f $install_file
exit 0
