<?php
/*
 * the home page
 */
require_once("include_main.php");
require_once("classes/mail/innovaphone.mailbox.class.php");

InputPage::$method = "post";

class ButtonAnnouncement extends InputPageAction {
    
    public function ButtonAnnouncement($name, $label) {
        parent::__construct($name, $label);
        $this->autorender = false;
        $this->anyway = true;
    }
    
    function action(InputPage &$page, $phase) {
        $file = $page->getField("file");
        $chk = $page->getField("zip");
        if(!$file->valid()) {
            $page->render();
            return;
        }
        $tmpDir = tempdir("/tmp", "announce");
        $wav = $tmpDir."/".$file->status->name;
        move_uploaded_file($file->status->path, $wav);
        $fileParts = explode(".", $file->status->name);
        $prefix = $fileParts[0];
        $coders = array("g711u", "g711a", "g729", "g723");
        foreach($coders as $coder) {
            $cmd = $this->getCmd($wav, $tmpDir, $prefix, $coder);
            $ret = 0;
            $out = main_system($cmd, $ret);
            if($ret != 0) {
                $file->msg = tl::tl("Conversion failed: ").$out;
                rrmdir($tmpDir);
                return $page->render();
            }
        }
        if($chk->value["on"]) { // create ZIP file
            $zipFileName = "$prefix.zip";
            $zip = "{$tmpDir}/$zipFileName";
            $zipFile = new ZipArchive();
            $zipFile->open($zip, ZIPARCHIVE::CREATE);
            foreach($coders as $coder) {
                $zipFile->addFile($tmpDir."/$prefix.$coder", "$prefix.$coder");
            }
            $zipFile->close();
            header('Pragma: private');
            header('Cache-control: private, must-revalidate');
            header("Content-Disposition: attachment; filename=\"$zipFileName\"");
            header('Content-type: application/zip');
            header('Content-len: '.filesize($zip));
            readfile($zip);
        }
        else {
            if(!file_exists(Config::announceDir)) {
                mkdir(Config::announceDir);
            }
            $out = main_system("mv $tmpDir/$prefix.g* ".Config::announceDir."/");
        }
        rrmdir($tmpDir);
        if(!$chk->value["on"]) {
            $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneApplicationConsts::pageApplication."&".$page->getArgs()."&announcements=".  urlencode($prefix));
        }
    }
    
    function getCmd($wav, $tmpDir, $prefix, $coder) {
        $bitRate = "2";
        if($coder == "g729") $bitRate = "1";
        $cmd = "/usr/local/bin/softenc.o \"$wav\" \"$tmpDir/$prefix.$coder\" $bitRate -$coder";
        return $cmd;
    }
}

class ButtonAskDeleteRelayHost extends InputPageButtonOnlyAction {
    public function ButtonAskDeleteRelayHost($name, $value) {
        parent::__construct($name, $value);
        $this->anyway = true;
    }
    
    function action(InputPage &$page, $phase) {
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs()."&mode=".InnovaphoneAdminConsts::modeDeleteRelay);
    }
}

class ButtonDeleteRelayHost extends MainInputPageOkCancelButton {
    public function ButtonDeleteRelayHost($name, $pageNo, InputPage &$page, $nextMode, $prevMode) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $mainSession MainSession*/global $mainSession;
        if($mainSession->relayHost->delete() === false) {
            return $this->reRenderPage($page, $mainSession->relayHost->error);
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&tab=".InnovaphoneAdminConsts::tabRelayHosts."&relay_id=0&deletedrelay=".urlencode($page->relayName));
    }
    
    function reRenderPage(&$page, $error) {
        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
        return false;
    }
}

class ButtonRelayHost extends MainInputPageOkCancelButton {
    public function ButtonRelayHost($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode, null, true);
        $this->anyway = false;
    }

    function action(InputPage &$page, $phase) {
        if(self::setValues($page) === false) {
            return $this->error($page, "");
        }
        if($this->prevButton != null) {
            /* @var $mainSession MainSession*/global $mainSession;
            if($mainSession->relayHost->server == null || $mainSession->relayHost->server == "") {
                home_die(tl::tl("Session exceeded!"));
            }
            $mode = ($mainSession->relayHost->id == 0 ? "createdrelay" : "updatedrelay");
            if($mainSession->relayHost->write() === false) {
                return $this->error($page, $mainSession->relayHost->error);
            }
            $arg = "";
            if($page->relayId === 0) {
                $arg = "&createdrelay=".urlencode($mainSession->relayHost->email);
            }
            else {
                $arg = "&editedrelay=".urlencode($mainSession->relayHost->email);
            }
            $relayId = $page->getField("relay_id");
            $relayId->value = $page->relayId = 0;
            $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs().$arg);
        }
        else {
            $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs()."&mode={$this->prevMode}");
        }
    }

    static function setValues(InputPage &$page) {
        $server = $page->getField("server");
        $port = $page->getField("port");
        $pw = $page->getField("password");
        $user = $page->getField("user");
        $email = $page->getField("email");
        
        if(!$server->valid() || !$email->valid()) {
            return false;
        }
        
        /* @var $mainSession MainSession*/global $mainSession;
        $mainSession->relayHost->server = $server->value;
        $mainSession->relayHost->port = $port->value;
        if($pw->value != MainMethods::dummyPw) {
            $mainSession->relayHost->password = $pw->value;
        }
        $mainSession->relayHost->user = $user->value;
        $mainSession->relayHost->email = $email->value;
        $mainSession->updateRelayHost($mainSession->relayHost);
        return true;
    }
    
    function error(InputPage &$page, $error) {
        $this->msg = $error;
        $page->render();
        return false;
    }
}

class ButtonSave extends ButtonHidden {
    /**
     *
     * @var BackupCmd
     */
    var $cmd;
    
    public function ButtonSave($name, $label, $cmd) {
        parent::__construct($name, $label);
        $this->cmd = $cmd;
    }

    function action(InputPage &$page, $phase) {
        $semaphore = new SemaphoreCommand;

        if($semaphore->getSemaphore() === false) {
            return $this->error($page, tl::tl("This action is currently locked."));
        }

        $backup = new MainBackup();
        global $mainSession;
        if($mainSession->webCfg->error != "") {
            return $this->error($page, $mainSession->webCfg->error, $semaphore);
        }
        $result = false;
        switch($this->cmd->cmd) {
            case MainBackup::saveInnovaphoneCfgs:
                $result = $backup->saveCfgs("innovaphone_cfg_#i.tar.gz", "echo", $error);
                break;
            case MainBackup::saveInnovaphoneLogs:
                $result = $backup->saveLogs("innovaphone_logs_#i.tar.gz", "echo", $error);
                break;
            default:
                $result = $backup->saveDefaultCmd("", $this->cmd->cmd, "echo", $error, $this->cmd->script);
                break;
        }
        if($result === false) {
            return $this->error($page, $error, $semaphore);
        }
        $semaphore->releaseSemaphore();
    }
    
    function error(InputPage &$page, $error, $semaphore = false) {
        if($semaphore !== false) $semaphore->releaseSemaphore();
        $fld = $page->getField("error");
        $fld->value = "<div style=\"color:red\">$error</div><br>";
        $fld->autorender = true;
        $page->render();
        return false;
    }
}

class ButtonRestore extends InputPageAction {
    /**
     *
     * @var RestoreCmd
     */
    var $cmd;
    
    public function ButtonRestore($name, $label, $cmd) {
        parent::__construct($name, $label);
        $this->autorender = false;
        $this->anyway = false;
        $this->cmd = $cmd;
    }

    function action(InputPage &$page, $phase) {
        global $mainSession;
        $file = $page->getField($this->cmd->cmd);
        if(!$file->valid()) {
           $page->render();
           return;
        }
        $semaphore = new SemaphoreCommand;

        if($semaphore->getSemaphore() === false) {
            return $this->error($page, $file, tl::tl("This action is currently locked."));
        }

        $backup = new MainBackup();
        if($mainSession->webCfg->error != "") {
            return $this->error($page, $file, $mainSession->webCfg->error, "", $semaphore);
        }
        switch($this->cmd->cmd) {
            case MainBackup::restoreInnovaphoneCfgs:
                $tmpDir = tempdir("/tmp", "restore");
                $tar = "/bin/tar xzvf \"{$file->status->path}\" -C \"$tmpDir\" 2>&1";
                $t = main_system($tar, $ret);
                if($ret != 0) {
                    return $this->error($page, $file, $t, $tmpDir, $semaphore);
                }
                $cmd = "/usr/bin/sudo /usr/local/bin/config.sh innovaphone cfg-restore '/var/www/innovaphone/config.php' $tmpDir 2>&1";
                $error = MainMethods::callConfigSh($cmd, $retval);
                if($error != "") {
                    return $this->error($page, $file, $error, $tmpDir, $semaphore);
                }
                $this->cleanup($tmpDir);
                $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs()."&restoredconfig=innovaphone");
                $mainSession->webCfg->reloadLighttpdConfig($ret);
                break;
            case MainBackup::restoreInnovaphoneDb:
            case MainBackup::restoreExchangeDb:
            case MainBackup::restoreFaxserverDb:
            case MainBackup::restoreReportingDb:
                $dbDump = "";
                $db = "";
                switch($this->cmd->cmd) {
                    case MainBackup::restoreExchangeDb:
                        $dbDump = Config::exchangeDbDump;
                        $db = "innovaphone-exchange";
                        break;
                    case MainBackup::restoreFaxserverDb:
                        $dbDump = Config::faxserverDbDump;
                        $db = "innovaphone-faxserver";
                        break;
                    case MainBackup::restoreReportingDb:
                        $dbDump = Config::reportingDbDump;
                        $db = "innovaphone-reporting";
                        break;
                    case MainBackup::restoreInnovaphoneDb:
                        $dbDump = Config::innovaphoneDbDump;
                        $db = "innovaphone";
                        break;
                }
                $tmpDir = tempdir("/tmp", "restore");
                exec("/bin/chmod 777 $tmpDir");
                if(ends_with($file->status->name,".tar.gz")) {
                    $tmpFile = $tmpDir."/pg_dump.tar.gz";
                    $format = "tar";
                } elseif(ends_with($file->status->name,".gz")) {
                    $tmpFile = $tmpDir."/pg_dump.gz";
                    $format = "gz";
                } else {
                    return $this->error($page, $file, tl::tl("Unknown format"), $tmpDir, $semaphore);
                }
                move_uploaded_file($file->status->path, $tmpFile);
                // extract uploaded archive
                if(!strcmp($format,"tar"))
                    $tar = "/bin/tar xzvf $tmpFile -C $tmpDir 2>&1";
                else
                    $tar = "/bin/gzip -cd $tmpFile --uncompress > $tmpDir/$dbDump 2>&1";
                $t = main_system($tar, $ret);
                if($ret != 0) {
                    return $this->error($page, $file, tl::tl("Extraction failed: ")."($ret) $t", $tmpDir, $semaphore);
                }
                $tmpSchema = $tmpDir."/pg_schema.dump";
                $cmd = "/usr/bin/sudo /usr/local/bin/config.sh postgres db-restore $tmpSchema \"$tmpDir/$dbDump\" $db 2>&1";
                $error = MainMethods::callConfigSh($cmd, $retval);
                if($error != "") {
                    return $this->error($page, $file, $error, $tmpDir, $semaphore);
                }
                $this->cleanup($tmpDir);
                $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs()."&restoreddb=".urlencode($db));
                break;
            default:
                if($this->cmd->script != "") {
                    $cmd = "/usr/bin/sudo /usr/local/bin/config.sh application script {$this->cmd->appName} {$this->cmd->script} ".$file->status->path;
                    $error = MainMethods::callConfigSh($cmd, $retval);
                    if($error != "") {
                        return $this->error($page, $file, $error, "", $semaphore);
                    }
                } else {
                    preg_match('/restore(.*)cfgs/', $this->cmd->cmd, $matches);
                    if($matches[1]) {
                        $tmpDir = tempdir("/tmp", "restore");
                        $tar = "/bin/tar xzvf \"{$file->status->path}\" -C \"$tmpDir\" 2>&1";
                        $t = main_system($tar, $ret);
                        if($ret != 0) {
                            return $this->error($page, $file, $t, $tmpDir, $semaphore);
                        }
                        $cmd = "/usr/bin/sudo /usr/local/bin/config.sh application cfg-restore {$this->cmd->appName} {$matches[1]}.xml ".$tmpDir;
                        foreach($this->cmd->actions as $action) {
                            $cmd .= " \"$action\"";
                        }
                        $cmd .= " 2>&1";
                        $error = MainMethods::callConfigSh($cmd, $retval);
                        if($error != "") {
                            return $this->error($page, $file, $error, $tmpDir, $semaphore);
                        }
                        $this->cleanup($tmpDir);
                        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs()."&restoredconfig=".$matches[1]);
                        $mainSession->webCfg->reloadLighttpdConfig($ret);
                    } else {
                        return $this->error($page, $file, tl::tlx('No application was found with this name {$arg[1]}', $matches[1]), $semaphore);
                    }
                }
                break;
        }
        $semaphore->releaseSemaphore();
    }

    function cleanup($tmpDir) {
        if($tmpDir != "" && file_exists($tmpDir)) {
            exec("/bin/rm -R \"$tmpDir\"");
        }
    }

    function error($page, $file, $error, $tmpDir = "", $semaphore = false) {
        if($semaphore !== false) $semaphore->releaseSemaphore();
        $this->cleanup($tmpDir);
        $file->msg = tl::tlx('Restoring files failed with: {$arg[1]}', $error);
        $page->render();
    }
}

class ButtonCerts extends InputPageButtonOnlyAction {
    const typeDelete=1, typeTrust=2, typeDeleteReject=3, typeDeleteRequest=4, typeInstallCert = 5;
    var $listName;
    var $buttonType;
    
    public function ButtonCerts($name, $label, $listName, $buttonType) {
        parent::__construct($name, $label);
        $this->anyway = true;
        $this->buttonType = $buttonType;
        $this->listName = $listName;
    }

    function action(InputPage &$page, $phase) {
        global $mainSession;
        $list = $page->getField($this->listName);
        $files = array();
        foreach($list->value as $value) {
            if($value[$list->name."chk"]["on"]) {
                $files[] = $value[$list->name."file"];
            }
        }
        if(!count($files)) {
            return $this->reRender($page, "");
        }
        switch($this->buttonType) {
            case self::typeDeleteReject:
            case self::typeDelete:
                foreach($files as $file) {
                    unlink($file);
                }
                break;
            case self::typeTrust:
                if(!file_exists(Config::certDirTrusted)) {
                    exec("/bin/mkdir -p \"".Config::certDirTrusted."\"");
                }
                foreach($files as $file) {
                    $name = strrchr($file, "/");
                    rename($file, Config::certDirTrusted.$name);
                }
                break;
            case self::typeDeleteRequest:
                foreach($files as $file) {
                    if(strstr($file, "ss_certs"))
                        $delCsr = str_replace(".pem",".crt",$files[0]);
                    else
                        $delCsr = str_replace(".pem",".csr",$files[0]);
                    $cmd="/usr/bin/sudo /usr/local/bin/config.sh ssl del_csr ".$delCsr;
                    MainMethods::callConfigSh($cmd, $retval);
                }
                break;
            case self::typeInstallCert:
                if(count($files) == 1) {
                    $newCert = str_replace(".crt",".pem",$files[0]);
                    $newCertTemp = tempnam("/tmp", "cert");
                    if (!copy($newCert, $newCertTemp)) {
                        return $this->reRender($page, tl::tl('Couldn\'t copy certificate'));
                    }
                    $cmd = "/usr/bin/sudo /usr/local/bin/config.sh ssl write ".Config::certServer." ".Config::certServer.".bck ".$newCertTemp;
                    $cmd .= " ".Config::certServerCert." ".Config::certServerKey;
                    $error = MainMethods::callConfigSh($cmd, $retval);
                    if($error != "") {
                        return $this->reRender($page, tl::tlx('Couldn\'t install new certificate: ', $error));
                    }
                    $mainSession->webCfg->reloadLighttpdConfig($ret);
                }
                break;
        }
        if($this->buttonType != self::typeDeleteReject) {
            $rm = "/bin/rm -f ".Config::certDirTrusted."/*.0 2>&1";
            $t = main_system($rm, $ret);
            /*if($ret != 0) {
                return $this->reRender($page, tl::tlx('Couldn\'t delete symbolic links of trusted certificates: ', $t));
            }*/
            $reHash = "/usr/bin/c_rehash ".Config::certDirTrusted."/ 2>&1";
            $t = main_system($reHash, $ret);
            if($ret != 0) {
                return $this->reRender($page, tl::tlx('Couldn\'t create symbolic links for trusted certificates: {$arg[1]}', $t));
            }
        }
        $this->reRender($page, "");
    }
    
    function reRender($page, $error) {
        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
    }
}

class ButtonCfgBackup extends InputPageButtonOnlyAction {
    public function ButtonCfgBackup($name, $label) {
        parent::__construct($name, $label);
        $this->anyway = false;
    }

    function action(InputPage &$page, $phase) {
        global $mainSession;
        if($mainSession->webCfg->error != "") {
            return $this->reRenderPage($page, $mainSession->webCfg->error);
        }
        /*$alarmServerIp = $page->getField("alarmserverip");
        $mainSession->webCfg->alarmServerIp = $alarmServerIp->value;*/
        $url = $page->getField("backupurl");
        $url->value = trim($url->value, " ");
        $poll = $page->getField("poll");
        $mainSession->webCfg->backupUrl = $url->value;
        $mainSession->webCfg->pollTimeout = $poll->value;
        if($mainSession->webCfg->write() === false) {
            return $this->reRenderPage($page, $mainSession->webCfg->error);
        }
        $cron = new CronJobs(Config::cronInnovaphoneBackup);
        if($url->value == "" || (int)$poll->value == 0) {
            $cron->remove(Config::backupScript);
        }
        else {
            $backupJob = new CronJob();
            if((int)$poll->value == 60) {
                $backupJob->hour = "*/1";
                $backupJob->min = "0";
            }
            else {
                $backupJob->min = "*/{$poll->value}";
            }
            $backupJob->cmd = "root [ -x /usr/bin/sudo ] && /usr/bin/sudo -u www-data nice -1 php5 ".Config::backupScript." >> ".Config::backupLog." 2>&1";
            $cron->add(Config::backupScript, $backupJob);
        }
        $cron->submit(Config::cronInnovaphoneBackup);
        if($cron->error != "") {
            return $this->reRenderPage($page, $cron->error);
        }
        if(file_exists(Config::backupLog)) {
            unlink(Config::backupLog);
        }
        $page->render();
    }
    
    function reRenderPage(InputPage &$page, $error) {
        $poll = $page->getField("poll");
        $poll->msg = $error;
        $page->render();
    }
}

class ButtonUploadSSL extends InputPageAction {

    public function ButtonUploadSSL($name, $label) {
        parent::__construct($name, $label);
        $this->autorender = false;
        $this->anyway = true;
    }

    function action(InputPage &$page, $phase) {
        global $installFile;
        global $mainSession;

        $file = $page->getField("sslupload");
        if(!$file->valid()) {
            return $this->reRender($page, $file, "");
        }
        $cert = file_get_contents($file->status->path);
        if(($x509 = openssl_x509_parse($cert)) === false) {
            return $this->reRender($page, $file, tl::tl("Couldn't parse uploaded file."));
        }
        if(openssl_pkey_get_public("file://".$file->status->path) === false) {
            return $this->reRender($page, $file, tl::tl("Couldn't parse public key."));
        }
        if(openssl_pkey_get_private("file://".$file->status->path) === false) {
            $notFound = true;
            $dir = Config::certRequests;
            $pemFile = "";
            if(is_dir($dir)) {
                if ($dh = opendir($dir)) {
                    while (($fileT = readdir($dh)) !== false) {
                        if(strstr($fileT, ".key")) {
                            $cmd = "/usr/bin/sudo /usr/local/bin/config.sh ssl check_pp ".$dir."/".$fileT." ".$file->status->path;
                            $error = MainMethods::callConfigSh($cmd, $retval);
                            if($error == "") {
                                $pemFile = tempnam("/tmp", "pem");
                                if(copy($file->status->path, $pemFile)) {
                                    $key = file_get_contents($dir."/".$fileT);
                                    file_put_contents($pemFile, $key, FILE_APPEND | LOCK_EX);
                                    $notFound = false;
                                }
                                break;
                            }
                        }
                    }
                }
                else
                    return $this->reRender($page, $file, tl::tl("Couldn't parse private key."));
            }
            if($notFound)
                return $this->reRender($page, $file, tl::tl("Couldn't parse private key."));
            // create file with keys!
            $cmd = "/usr/bin/sudo /usr/local/bin/config.sh ssl write ".Config::certServer." ".Config::certServer.".bck ".$pemFile;
        }
        else
            $cmd = "/usr/bin/sudo /usr/local/bin/config.sh ssl write ".Config::certServer." ".Config::certServer.".bck ".$file->status->path;
        $cmd .= " ".Config::certServerCert." ".Config::certServerKey;
        $error = MainMethods::callConfigSh($cmd, $retval);
        if($error != "") {
            return $this->reRender($page, $file, $error." ".$retval);
        }
        /*if(file_exists(Config::certServer)) {
            rename(Config::certServer, Config::certServer.".bck");
        }
        move_uploaded_file($file->status->path, Config::certServer);
        exec("/bin/chmod u=r,o=,g= ".Config::certServer);*/
        if($mainSession->webCfg->writeLighttpdConfig() === false) {
            return $this->reRender($page, $file, $mainSession->webCfg->error);
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs()."&newcert=1");
        $mainSession->webCfg->reloadLighttpdConfig($ret);
    }
    
    function reRender($page, $file, $error) {
        $file->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
    }
}

class ButtonChangeTz extends InputPageAction {
    public function ButtonChangeTz($name, $value) {
        parent::__construct($name, $value);
        $this->autorender = false;
    }

    function action(InputPage &$page, $phase) {
        global $mainSession;
        $tz = $page->getField("tz");
        $timezone = str_replace(" ", "", array_search(true, $tz->value)); 
        $cmd="/usr/bin/sudo /usr/local/bin/config.sh timezone check ".$timezone;
        $error = MainMethods::callConfigSh($cmd, $retval);
        if($error != "") {
            return $this->error($page, $error);
        }
        $cmd="/usr/bin/sudo /usr/local/bin/config.sh timezone change ".$timezone;
        $error = MainMethods::callConfigSh($cmd, $retval);
        if($error != "") {
            return $this->error($page, $error);
        }
        $mainSession->webCfg->timezone = $timezone;
        if($mainSession->webCfg->write() === false) {
            return $this->error($page, $tl::tlx('Timezone could not be saved in config file: {$arg[1]}', $mainSession->webCfg->error));
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs()."&tzchanged=1&newtimezone=".$timezone);
        $mainSession->webCfg->reloadLighttpdConfig($ret);
    }

    function error(InputPage &$page, $error) {
        $tz = $page->getField("tz");
        $tz->msg = $error;
        $page->render();
    }
}

class ButtonAddNtpServer extends InputPageAction {
    public function ButtonAddNtpServer($name, $value) {
        parent::__construct($name, $value);
        $this->autorender = false;
    }

    function action(InputPage &$page, $phase) {
        global $mainSession;
        $ntp = $page->getField("ntp");
        if(!$ntp->valid()) {
            $page->render();
            return;
        }
        $cmd="/usr/bin/sudo /usr/local/bin/config.sh ntp add ".$ntp->value;
        $error = MainMethods::callConfigSh($cmd, $retval);
        if($error != "") {
            return $this->error($page, $error);
        }
        $ntps = read_ntps_server($error);
        $mainSession->webCfg->ntpServer = $ntps;
        if($mainSession->webCfg->write() === false) {
            return $this->error($page, $tl::tlx('NTP server could not be saved in config file: {$arg[1]}', $mainSession->webCfg->error));
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs()."&ntpadded=1");
    }
    
    function error(InputPage &$page, $error) {
        $ntp = $page->getField("ntp");
        $ntp->msg = $error;
        $page->render();
    }
}

class ButtonDeleteNtpServer extends InputPageButtonOnlyAction {
    public function ButtonDeleteNtpServer($name, $value) {
        parent::__construct($name, $value);
    }

    function action(InputPage &$page, $phase) {
        global $mainSession;
        $list = $page->getField("ntplist");
        $ntpsToDelete = array();
        foreach($list->value as $value) {
            if($value[$list->name."chkdelete"]["on"]) {
                $ntpsToDelete[count($ntpsToDelete)] = $value[$list->name."ntp"];
            }
        }
        if(count($ntpsToDelete) == 0) {
            return $this->error($page, "");
        } else {
            $error = "";
            foreach($ntpsToDelete as $ntpDel) {
                $cmd="/usr/bin/sudo /usr/local/bin/config.sh ntp delete ".$ntpDel;
                $error = MainMethods::callConfigSh($cmd, $retval);
                if($error != "") {
                    return $this->error($page, $error);
                }
            }
        }
        $ntps = read_ntps_server($error);
        $mainSession->webCfg->ntpServer = $ntps;
        if($mainSession->webCfg->write() === false) {
            return $this->error($page, $tl::tlx('NTP server could not be saved in config file: {$arg[1]}', $mainSession->webCfg->error));
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs()."&ntpdeleted=1");
    }

    function error(InputPage &$page, $error) {
        $this->msg = $error;
        $page->render();
    }
}

class ButtonIPConfig extends MainInputPageOkCancelButton {
    public function ButtonIPConfig($name, $pageNo, InputPage &$page) {
        parent::__construct($name, $pageNo, $page);
    }

    function action(InputPage &$page, $phase) {
        global $mainSession;
        if(!$mainSession->isVm) {
            die("only available on a VM!");
        }
        $dhcp = $page->getField("dhcp");
        $ip = $page->getField("ip");
        $gw = $page->getField("gw");
        $mask = $page->getField("mask");
        $dns1 = $page->getField("dns1");
        $dns2 = $page->getField("dns2");
        $dhcpVal = intval(array_search(true, $dhcp->value));
        
        $cmd = "sudo /usr/local/bin/config.sh ip-config";
        if($dhcpVal === 1) {
            $cmd .= " -d 1";
        }
        else {
            if(!$ip->valid() || !$mask->valid() ||
                ($gw->value != "" && !$gw->valid()) ||
                ($dns1->value != "" && !$dns1->valid()) ||
                ($dns2->value != "" && !$dns2->valid())) {
                return $page->render();
            }
            $cmd .= " -a {$ip->value}";
            $cmd .= " -m {$mask->value}";
            if($gw->value != "") {
                $cmd .= " -g {$gw->value}";
            }
            if($dns1->value != "") {
                $cmd .= " -d1 {$dns1->value}";
            }
            if($dns2->value != "") {
                $cmd .= " -d2 {$dns2->value}";
            }
        }
        $cmd .= " &"; // do not wait for return
        $page->pages[$page->pageNo]->hideSection(InnovaphoneAdminConsts::modeIp);
        $txt = $page->getField("hinttext");
        $txt->autorender = true;
        $txt->value = "<br><i>".tl::tl("The network interface is restarting. Please use your new IP address to reconnect...")."</i><br><br>";
        $page->render();
        $error = MainMethods::callConfigSh($cmd, $retval);
        $mainSession->destroy();
        $mainSession = null;
    }
}

class ButtonNTPserver extends MainInputPageOkCancelButton {
	public function ButtonNTPserver($name, $pageNo, InputPage &$page) {
        parent::__construct($name, $pageNo, $page);
    }

    function action(InputPage &$page, $phase) {
        global $mainSession;
        if($mainSession->webCfg->error != "") {
            return $this->reRenderPage($page, $mainSession->webCfg->error);
        }
        $ntpServer = $page->getField("ntpserverip");
        $log = $page->getField("ntpserverlog");
        $cmd = "sudo /usr/local/bin/config.sh ntp $ntpServer->value 2>&1";
        $error = MainMethods::callConfigSh($cmd, $retval);
        if($error != "") {
            $log->value = $error;
            $page->render();
            return;
        }
        $mainSession->webCfg->ntpServer = $ntpServer->value;
        if($mainSession->webCfg->write() === false) {
            $log->value = tl::tlx('NTP server could not be saved in config file: {$arg[1]}', $mainSession->webCfg->error);
            $page->render();
            return;
        }
        $page->render();
    }
}

class ButtonChangePassword extends MainInputPageOkCancelButton {
    var $appName;
    
    public function ButtonChangePassword($name, $pageNo, InputPage &$page, $appName) {
        parent::__construct($name, $pageNo, $page);
        $this->appName = $appName;
    }
    
    function changePass($file, $curUser, $newUser, $curPw, $newPw)
    {
        $output = "";
        $changed = false;
        if(!file_exists($file)) {   // true for optional application auth file
            $content = "$newUser:".Config::lighttpdRealm.":".md5("$newUser:".Config::lighttpdRealm.":$newPw");
            file_put_contents($file, $content);
            return true;
        }
        $in = fopen( $file, 'r' );
        while ( preg_match("/:/", $line = fgets($in) ) )
        {
            $line = rtrim( $line );
            $lineElems = explode( ':', $line );
            if ($lineElems[0]==$curUser)
            {
                if ($lineElems[2] == md5($curUser.":".$lineElems[1].":".$curPw))
                {
                    $lineElems[0] = $newUser;
                    $lineElems[2] = md5($newUser.":".$lineElems[1].":".$newPw);
                    $changed = true;
                }
            }
            $output .= implode( ':', $lineElems )."\n";
        }
        fclose($in);
        if($changed) {
            $out = fopen( "htdigest.new", 'w' );
            fwrite( $out, $output );
            fclose( $out );
            system("mv -f htdigest.new $file");
        }
        return $changed;
    }

    function changePostgresPw($currPw, $newPw, &$ret) {

        $db = @pg_connect("host='127.0.0.1' port='5432' dbname='postgres' user='postgres' password='$currPw' sslmode=disable");
        if($db !== false) {
            $cmd = "/usr/bin/sudo /usr/local/bin/config.sh postgres password ".$newPw." postgres";
            $error = MainMethods::callConfigSh($cmd, $ret);
            if($error != "") {
                $ret = 3;
                return $resPass;
            }
            $ret = 0;
        } else {
            $ret = 1;
            return tl::tl("Wrong password for postgres user");
        }
    }
    
    function changeRootPw($curPw, $newPw, &$ret) {
        $descriptorspec = array(
            0 => array("pipe", "rb"),  // STDIN ist eine Pipe, von der das Child liest
            1 => array("pipe", "wb"),  // STDOUT ist eine Pipe, in die das Child schreibt
            );
        $process = proc_open('sudo /usr/bin/pam-change-pw 2>&1', $descriptorspec, $pipes);
        $output = "";
        if (is_resource($process)) {
            // $pipes sieht nun so aus:
            // 0 => Schreibhandle, das auf das Child STDIN verbunden ist
            // 1 => Lesehandle, das auf das Child STDOUT verbunden ist
            fwrite($pipes[0], "$curPw\n");
            fwrite($pipes[0], "$newPw\n");
            fclose($pipes[0]);

            $output = stream_get_contents($pipes[1]);
            fclose($pipes[1]);

            // Es ist wichtig, dass Sie alle Pipes schlie�en bevor Sie
            // proc_close aufrufen, um Deadlocks zu vermeiden
            $ret = proc_close($process);
        }
        else {
            $ret = 255;
        }
        return $output;
    }
    
    function action(InputPage &$page, $phase) {
        global $mainSession;
        $error = false;
        $user = $page->getField("user");
        $currentUser = $page->getField("currentuser");
        $curPw = $page->getField("currentpw");
        $newPw = $page->getField("newpw");
        $verPw = $page->getField("verpw");
        $noMatchText = tl::tl("Current password does not match.");
        $app = null;
        if($this->appName != "") {
            $app = $mainSession->getApp($this->appName);
        }
        if($this->appName == "" || $user->value != "") {
            if(!$user->valid() || !$newPw->valid() || ($currentUser->value != "" && !$curPw->valid()) || !$verPw->valid()) {
                $page->render();
                return;
            }
        }
        else {
            if($currentUser->value != "") { // check current password
                if(!$curPw->valid()) {
                    $page->render();
                    return;
                }
                if($this->changePass($app->authFile, $currentUser->value, $currentUser->value, $curPw->value, $curPw->value) === false) {
                    $this->msg = $noMatchText;
                    $page->render();
                    return;
                }
            }
            if(file_exists($app->authFile)) {
                unlink($app->authFile);
            }
            $appName = "";
            $mainSession->webCfg->appAuth[$this->appName] = false;
            $mainSession->webCfg->write();
            $mainSession->webCfg->writeLighttpdConfig();
            $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs()."&changedappcred=".urlencode($appName));
            $mainSession->webCfg->reloadLighttpdConfig($ret);
            return;
        }
        
        if($currentUser->value == "" && $this->appName == "") {
            $error = tl::tl("Current user unknown.");
        } else {
            switch($page->currentMode) {
                case InnovaphoneAdminConsts::modeHttpPw:
                    if($this->changePass(Config::serverPassFile, $currentUser->value, $user->value, $curPw->value, $newPw->value) === false) {
                        $error = $noMatchText;
                    }
                    break;
                case InnovaphoneAdminConsts::modeWebdavPw:
                    if($this->changePass(Config::webdavPassFile, $currentUser->value, $user->value, $curPw->value, $newPw->value) === false) {
                        $error = $noMatchText;
                    }          
                    break;
                case InnovaphoneAdminConsts::modeAppPw:
                    if($this->changePass($app->authFile, $currentUser->value, $user->value, $curPw->value, $newPw->value) === false) {
                        $error = $noMatchText;
                    }          
                    break;
                case InnovaphoneAdminConsts::modeRootPw:
                    $t = $this->changeRootPw($curPw->value, $newPw->value, $ret);
                    if($ret != 0) {
                        $error = tl::tlx('New password for root could not be set: {$arg[1]} ({$arg[2]})', $t, $ret);
                    }
                    break;
                case InnovaphoneAdminConsts::modePgPw:
                    $t = $this->changePostgresPw($curPw->value, $newPw->value, $ret);
                    if($ret != 0) {
                        $error = tl::tlx('New password for postgres could not be set: {$arg[1]} ({$arg[2]})', $t, $ret);
                    }
                    break;
            }
        }

        if($error !== false) {
            $this->msg = $error;
            $page->render();
            return;
        }
        $add = "";
        switch($page->currentMode) {
            case InnovaphoneAdminConsts::modeWebdavPw: $add = "changedwebdavcred=1"; break;
            case InnovaphoneAdminConsts::modeAppPw: 
                $add = "changedappcred=".urlencode($this->appName); 
                $mainSession->webCfg->appAuth[$this->appName] = true;
                break;
            case InnovaphoneAdminConsts::modeHttpPw: $add = "changedwebcred=1"; break;
            case InnovaphoneAdminConsts::modeRootPw: $add = "changedrootpw=1"; break;
            case InnovaphoneAdminConsts::modePgPw: $add = "changedpostgrespw=1"; break;
        } 
        if($page->currentMode == InnovaphoneAdminConsts::modeAppPw) {
            $mainSession->webCfg->write();
            $mainSession->webCfg->writeLighttpdConfig();
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs()."&$add");
        if($page->currentMode == InnovaphoneAdminConsts::modeAppPw) {
            $mainSession->webCfg->reloadLighttpdConfig($ret);
        }
    }
}

class ButtonConfigureMTLS extends MainInputPageOkCancelButton {
    public function ButtonConfigureMTLS($name, $pageNo, InputPage &$page) {
        parent::__construct($name, $pageNo, $page);
    }
    
    function action(InputPage &$page, $phase) {
        global $mainSession;
        if($mainSession->webCfg->error != "") {
            $this->msg = $mainSession->webCfg->error;
            return $page->render();
        }
        $port = $page->getField("port");
        $active = $page->getField("active");
        $mainSession->webCfg->mtls->port = $port->value;
        $mainSession->webCfg->mtls->active = ($active->value["on"] ? true : false);
        if($mainSession->webCfg->write() === false || $mainSession->webCfg->writeLighttpdConfig() === false) {
            $this->msg = $mainSession->webCfg->error;
            return $page->render();
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs()."&mtls=1");
        $ret = 0;
        $mainSession->webCfg->reloadLighttpdConfig($ret);
    }
}

class ButtonCommitConfig extends MainInputPageOkCancelButton {
    public function ButtonCommitConfig($name, $pageNo, InputPage &$page) {
        parent::__construct($name, $pageNo, $page);
    }
    
    function pathExists($path, $paths) {
        foreach($paths as $p) {
            if($p->path == $path) {
                return true;
            }
        }
        return false;
    }
    
    function action(InputPage &$page, $phase) {
        global $mainSession;
        if($mainSession->webCfg->error != "") {
            $this->msg = $mainSession->webCfg->error;
            return $page->render();
        }
        $mainSession->webCfg->publicWebDavPaths = array();
        $mainSession->webCfg->publicWebPaths = array();
        $forceHttps = $page->getField("forcehttps");
        $listWeb = $page->getField("publicweb");
        $listWebDav = $page->getField("publicwebdav");
        
        $mainSession->webCfg->forceHttps = $forceHttps->value["on"];
        foreach($listWeb->value as $value) {
            if(($path = trim($value[$listWeb->name."path"], " ")) != "") {
                $value[$listWeb->name."path"] = "/".ltrim($path, "/");
                if(!$this->pathExists($value[$listWeb->name."path"], $mainSession->webCfg->publicWebPaths)) {
                    $mainSession->webCfg->publicWebPaths[] = new WebPath($value[$listWeb->name."path"]);
                }
            }
        }
        foreach($listWebDav->value as $value) {
            if(($path = trim($value[$listWebDav->name."path"], " ")) != "") {
                $value[$listWebDav->name."path"] = "/".ltrim($path, "/");
                if(!$this->pathExists($value[$listWebDav->name."path"], $mainSession->webCfg->publicWebDavPaths)) {
                    $write = false;
                    if($value[$listWebDav->name."write"]["on"]) {
                        $write = true;
                    }
                    $mainSession->webCfg->publicWebDavPaths[] = new WebPath($value[$listWebDav->name."path"], $write);
                }
            }
        }
        if($mainSession->webCfg->write() === false || $mainSession->webCfg->writeLighttpdConfig() === false) {
            $this->msg = $mainSession->webCfg->error;
            return $page->render();
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs()."&savedconfig=1");
        $mainSession->webCfg->reloadLighttpdConfig($ret);
    }
}

class ButtonTestEmail extends InputPageAction {
    public function ButtonTestEmail($name, $value) {
        parent::__construct($name, $value);
        $this->autorender = false;
    }

    function action(InputPage &$page, $phase) {
        $rec = $page->getField("recipient");
        $sender = $page->getField("sender");
        
        if(!$rec->valid() || !$sender->valid()) {
            return $page->render();
        }
        
        $mail = new InnovaphoneMailbox("127.0.0.1", "", "", "", "", false);
        if($mail->sendMail($rec->value, $sender->value, "Testmail", "Test") === false) {
            $sender->msg = $mail->getErrors();
        }
        $page->render();
    }

    function error(InputPage &$page, $error) {
        $em = $page->getField("emailaddresses");
        $em->msg = $error;
        $page->render();
    }
}

class ButtonAuthUrls extends MainInputPageOkCancelButton {
    public function ButtonAuthUrls($name, $pageNo, InputPage &$page) {
        parent::__construct($name, $pageNo, $page);
    }
    
    function action(InputPage &$page, $phase) {
        global $mainSession;
        if($mainSession->webCfg->error != "") {
            $this->msg = $mainSession->webCfg->error;
            return $page->render();
        }
        $mainSession->webCfg->authUrls = array();
        $listAuthUrls = $page->getField("authurls");
        foreach($listAuthUrls->value as $value) {
            if(($path = trim($value[$listAuthUrls->name."url"], " ")) != "") {
                $mainSession->webCfg->authUrls[] = new AuthenticatedUrl($path, $value[$listAuthUrls->name."user"], $value[$listAuthUrls->name."pw"]);
            }
        }
        if($mainSession->webCfg->write() === false) {
            $this->msg = $mainSession->webCfg->error;
            return $page->render();
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs()."&savedauthurls=1");
    }
}

class ButtonCreateCsr extends MainInputPageOkCancelButton {
    public function ButtonCreateCsr($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode, null, true);
        $this->anyway = false;
    }

    function action(InputPage &$page, $phase) {
        $keytype = $page->getField("keytype");
        $keyVal = intval(array_search(true, $keytype->value));
        $csrtype = $page->getField("csrtype");
        $csrVal = intval(array_search(true, $csrtype->value));
        $sigtype = $page->getField("sigtype");
        $sigVal = intval(array_search(true, $sigtype->value));
        $validity = $page->getField("yvalidity");
        $cn = $page->getField("commonname");
        $ou = $page->getField("orgunit");
        $o = $page->getField("organization");
        $st = $page->getField("stateprovince");
        $l = $page->getField("locality");
        $c = $page->getField("countrycsr");
        $dns0 = $page->getField("dnsname0");
        $dns1 = $page->getField("dnsname1");
        $dns2 = $page->getField("dnsname2");
        $ip0 = $page->getField("ipaddrcsr0");
        $ip1 = $page->getField("ipaddrcsr1");
        $tmpfname = tempnam("/tmp", "csr");
        $handle = fopen($tmpfname, "w");
        fprintf($handle, "[ req ]\n");
        if($keyVal === 0)
            fprintf($handle, "default_bits = 1024\n");
        else
            fprintf($handle, "default_bits = 2048\n");
        fprintf($handle, "prompt = no\n");
        fprintf($handle, "encrypt_key = no\n");
        if($cn->value != "" || $ou->value != "" || $o->value != "" || $st->value != "" || $l->value != "" || $c->value != "")
            fprintf($handle, "distinguished_name = dn\n");
        if($csrVal === 0) {
            fprintf($handle, "req_extensions = v3_req\n");
        }
        elseif($csrVal === 1) {
            fprintf($handle, "req_extensions = v3_ca\n");
        }
        else {
            unlink($tmpfname);
            return $this->error($page, "Unknown option");
        }
        if($cn->value != "" || $ou->value != "" || $o->value != "" || $st->value != "" || $l->value != "" || $c->value != "") {
            fprintf($handle, "[ dn ]\n");
            if($cn->value != "")
                fprintf($handle, "CN = %s\n", $cn->value);
            if($ou->value != "")
                fprintf($handle, "OU = %s\n", $ou->value);
            if($o->value != "")
                fprintf($handle, "O = %s\n", $o->value);
            if($st->value != "")
                fprintf($handle, "ST = %s\n", $st->value);
            if($l->value != "")
                fprintf($handle, "L = %s\n", $l->value);
            if($c->value != "")
                fprintf($handle, "C = %s\n", $c->value);
        }
        if($csrVal === 0) {
            fprintf($handle, "[ v3_req ]\n");
            fprintf($handle, "basicConstraints = CA:FALSE\n");
            fprintf($handle, "keyUsage = digitalSignature,keyEncipherment\n");
            fprintf($handle, "extendedKeyUsage = serverAuth,clientAuth\n");
        }
        elseif($csrVal === 1) {
            fprintf($handle, "[ v3_ca ]\n");
            fprintf($handle, "basicConstraints = critical,CA:TRUE,pathlen:0\n");
            fprintf($handle, "keyUsage = critical,digitalSignature,keyEncipherment,keyCertSign\n");
            fprintf($handle, "authorityKeyIdentifier=keyid, issuer\n");
            fprintf($handle, "subjectKeyIdentifier = hash\n");
        }
        else {
            unlink($tmpfname);
            return $this->error($page, "Unknown option");
        }
        if($dns0->value != "" || $dns1->value != "" || $dns2->value != "" || ($ip0->value != "" && $ip0->valid()) || ($ip1->value != "" && $ip1->valid())) {
            fprintf($handle, "subjectAltName = @alt_names\n\n");
            fprintf($handle, "[alt_names]\n");
            if($dns0->value != "")
                fprintf($handle, "DNS.1 = %s\n", $dns0->value);
            if($dns1->value != "")
                fprintf($handle, "DNS.2 = %s\n", $dns1->value);
            if($dns2->value != "")
                fprintf($handle, "DNS.3 = %s\n", $dns2->value);
            if($ip0->value != "" && $ip0->valid())
                fprintf($handle, "IP.1 = %s\n", $ip0->value);
            if($ip1->value != "" && $ip1->valid())
                fprintf($handle, "IP.2 = %s\n", $ip1->value);
        }
        fclose($handle);
        $sigOpt = "-sha1";
        if($sigVal == 1) $sigOpt = "-sha256";
        else if($sigVal == 2) $sigOpt = "-sha384";
        else if($sigVal == 3) $sigOpt = "-sha512";
        if($csrVal === 0) {
            if($keyVal === 0)
                $cmd="/usr/bin/sudo /usr/local/bin/config.sh ssl csr ".$tmpfname." ".$cn->value." ".$sigOpt." 1024";
            else
                $cmd="/usr/bin/sudo /usr/local/bin/config.sh ssl csr ".$tmpfname." ".$cn->value." ".$sigOpt." 2048";
            $csrret = MainMethods::callConfigSh($cmd, $retval);
            if($retval != 0) {
                return $this->error($page, $csrret);
            }
        }
        else {
            $days_val = intval($validity->value)*365;
            if($keyVal === 0)
                $cmd="/usr/bin/sudo /usr/local/bin/config.sh ssl cert ".$days_val." 1024 ".$tmpfname." ".$cn->value." ".$sigOpt;
            else
                $cmd="/usr/bin/sudo /usr/local/bin/config.sh ssl cert ".$days_val." 2048 ".$tmpfname." ".$cn->value." ".$sigOpt;
            $csrret = MainMethods::callConfigSh($cmd, $retval);
            if($retval != 0) {
                return $this->error($page, $csrret);
            }
        }
        unlink($tmpfname);
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&".$page->getArgs().$arg);
    }

    function error(InputPage &$page, $error) {
        $this->msg = $error;
        $page->render();
        return false;
    }
}

class PageAdministration extends PostgresPage {
    public function getHelpAnchor() {
		switch($this->parent->currentTab) {
            case InnovaphoneAdminConsts::tabWebServer: 
                switch($this->parent->currentMode) {
                    case InnovaphoneAdminConsts::modeServer: return "#Change_web_server_properties_and_public_access_to_the_web.2Fwebdav";
                    case InnovaphoneAdminConsts::modeHttpPw: return "#Change_the_Linux_web_server_credentials";
                    case InnovaphoneAdminConsts::modeWebdavPw: return "#Change_the_Linux_webdav_access_credentials";
                    case InnovaphoneAdminConsts::modeAppPw: return "#Change_application_access_credentials";
                    case InnovaphoneAdminConsts::modeMTLS: return "#Configure_mutual_TLS";
                    default : return "#Web_Server";
                }
            case InnovaphoneAdminConsts::tabGeneral: 
                switch($this->parent->currentMode) {
                    case InnovaphoneAdminConsts::modeRootPw: return "#Change_the_root_credentials";
                    case InnovaphoneAdminConsts::modeAuthUrls: return "#Configure_Authenticated_URLs";
                    case InnovaphoneAdminConsts::modeNTP: return "#Configure_NTP_server";
                    case InnovaphoneAdminConsts::modeTZ: return "#Configure_Timezone";
                    case InnovaphoneAdminConsts::modePgPw: return "#Change_postgresql_admin_password";
                    case InnovaphoneAdminConsts::modeIp: return "#Configure_IP";
                    default : return "#General";
                }
            case InnovaphoneAdminConsts::tabBackup: 
                switch($this->parent->currentMode) {
                    case InnovaphoneAdminConsts::modeSave: return "#Save_configuration_files.2Fdata";
                    case InnovaphoneAdminConsts::modeRestore: return "#Restore_configuration_files.2Fdata";
                    default : return "#Backup";
                }
            case InnovaphoneAdminConsts::tabSSL: 
                switch($this->parent->currentMode) {
                    default : return "#Certificates";
                }
            case InnovaphoneAdminConsts::tabPgsql: 
                switch($this->parent->currentMode) {
                    case PostgresPage::modeInRePgPw:
                        return "#Password";
                    default: return "#Database";
                }
            case InnovaphoneAdminConsts::tabAnnounce:
                return "#Announcements";
            case InnovaphoneAdminConsts::tabRelayHosts:
                return "#Relay_Hosts";
            default: return "";
		}
	}
	
	public function getDefaultTab() {
        return InnovaphoneAdminConsts::tabGeneral;
	}
	
	public function createFields() {
        $this->addSection(InnovaphoneAdminConsts::tabGeneral, 1);
        $this->addSection(InnovaphoneAdminConsts::tabWebServer, 1);
        $this->addSection(InnovaphoneAdminConsts::tabSSL, 1);
        $this->addSection(InnovaphoneAdminConsts::tabBackup, 2);
        $this->addSection(InnovaphoneAdminConsts::tabPgsql);
        
        $genericHint = "";
        
        $this->parent->addTabMenuItem(tl::tl("General"), "javascript:submitTab('".InnovaphoneAdminConsts::tabGeneral."')", $this->parent->currentTab == InnovaphoneAdminConsts::tabGeneral);
        $this->parent->addTabMenuItem(tl::tl("Web Server"), "javascript:submitTab('".InnovaphoneAdminConsts::tabWebServer."')", $this->parent->currentTab == InnovaphoneAdminConsts::tabWebServer);
        $this->parent->addTabMenuItem(tl::tl("Certificates"), "javascript:submitTab('".InnovaphoneAdminConsts::tabSSL."')", $this->parent->currentTab == InnovaphoneAdminConsts::tabSSL);
        $this->parent->addTabMenuItem(tl::tl("Backup"), "javascript:submitTab('".InnovaphoneAdminConsts::tabBackup."')", $this->parent->currentTab == InnovaphoneAdminConsts::tabBackup);
        $this->parent->addTabMenuItem(tl::tl("Database"), "javascript:submitTab('".InnovaphoneAdminConsts::tabPgsql."')", $this->parent->currentTab == InnovaphoneAdminConsts::tabPgsql);
        $this->parent->addTabMenuItem(tl::tl("Relay Hosts"), "javascript:submitTab('".InnovaphoneAdminConsts::tabRelayHosts."')", $this->parent->currentTab == InnovaphoneAdminConsts::tabRelayHosts);
        $this->parent->addTabMenuItem(tl::tl("Announcements"), "javascript:submitTab('".InnovaphoneAdminConsts::tabAnnounce."')", $this->parent->currentTab == InnovaphoneAdminConsts::tabAnnounce);
		
        switch($this->parent->currentTab) {
            case InnovaphoneAdminConsts::tabGeneral: 
                $this->tabGeneral();
                break;
            case InnovaphoneAdminConsts::tabWebServer: 
                $this->tabWebServer();
                break;
            case InnovaphoneAdminConsts::tabSSL:
                $this->tabSSL();
                break;
            case InnovaphoneAdminConsts::tabBackup:
                $this->tabBackup();
                break;
            case InnovaphoneAdminConsts::tabRelayHosts:
                $this->tabRelayHosts();
                break;
            case InnovaphoneAdminConsts::tabPgsql:
                $this->tabPgSql("innovaphone", InnovaphoneAdminConsts::pageAdmin);
                break;
            case InnovaphoneAdminConsts::tabAnnounce:
                $this->tabAnnouncements();
                break;
        }
    }
    
    function addActionText() {
        $text = $this->addField(new InputPageText("hinttext" , ""));
        $text->autorender = false;
        $text->fullline = true;
        $restoredDb = InputPage::formInput("restoreddb");
        $restoreConfig = InputPage::formInput("restoredconfig");
        $savedConfig = InputPage::formInput("savedconfig");
        $savedAuthUrls = InputPage::formInput("savedauthurls");
        $changedWebPw = InputPage::formInput("changedwebcred");
        $changedWebDavPw = InputPage::formInput("changedwebdavcred");
        $changedAppPw = InputPage::formInput("changedappcred");
        $changedRootPw = InputPage::formInput("changedrootpw");
        $changedPostgresPw = InputPage::formInput("changedpostgrespw");
        $createdRelay = InputPage::formInput("createdrelay");
        $editedRelay = InputPage::formInput("editedrelay");
        $deletedRelay = InputPage::formInput("deletedrelay");
        $newCert = InputPage::formInput("newcert");
        $ntpAdded = InputPage::formInput("ntpadded");
        $tzChanged = InputPage::formInput("tzchanged");
        $newtz = InputPage::formInput("newtimezone");
        $ntpDeleted = InputPage::formInput("ntpdeleted");
        $announce = InputPage::formInput("announcements");
        $mtls = InputPage::formInput("mtls");
        $reloadWebServer = " ".tl::tl("The web server config will be reloaded in a few seconds.");
        
        if(isset($restoredDb)) {
            $text->value = tl::tlx('Successfully restored the {$arg[1]} database.', $restoredDb);
        }
        elseif(isset($restoreConfig)) {
            if(!strcmp($restoreConfig,"innovaphone"))
                $text->value = tl::tl("Successfully restored configuration files for innovaphone application platform.").$reloadWebServer;
            else
                $text->value = tl::tl("Successfully restored configuration files for application $restoreConfig.").$reloadWebServer;
        }
        elseif(isset($tzChanged)) {
            $text->value = tl::tlx('Successfully changed the timezone to {$arg[1]}.', $newtz).$reloadWebServer;
        }
        elseif(isset($savedConfig)) {
            $text->value = tl::tl("Successfully saved the web server configuration.").$reloadWebServer;
        }
        elseif(isset($createdRelay)) {
            $text->value = tl::tlx('Relay host for {$arg[1]} successfully added.', $createdRelay);
        }
        elseif(isset($editedRelay)) {
            $text->value = tl::tlx('Relay host {$arg[1]} successfully updated.', $editedRelay);
        }
        elseif(isset($deletedRelay)) {
            $text->value = tl::tlx('Relay host {$arg[1]} successfully deleted.', $deletedRelay);
        }
        elseif(isset($changedWebPw)) {
            $text->value = tl::tl("Successfully changed the web server credentials.");
        }
        elseif(isset($changedWebDavPw)) {
            $text->value = tl::tl("Successfully changed the webdav credentials.");
        }
        elseif(isset($changedAppPw)) {
            $text->value = tl::tlx('Successfully changed the {$arg[1]} credentials.', $changedAppPw).$reloadWebServer;
        }
        elseif(isset($changedRootPw)) {
            $text->value = tl::tl("Successfully changed the root password.");
        }
        elseif(isset($changedPostgresPw)) {
            $text->value = tl::tl("Successfully changed the postgres password.");
        }
        elseif(isset($newCert)) {
            $text->value = tl::tl("Successfully installed new server certificate.").$reloadWebServer;
        }
        elseif(isset($savedAuthUrls)) {
            $text->value = tl::tl("Successfully saved the configuration for Authenticated URLs.");
        }
        elseif(isset($ntpAdded)) {
            $text->value = tl::tl("Successfully added ntp Server.");
        }
        elseif(isset($ntpDeleted)) {
            $text->value = tl::tl("Successfully deleted ntp Server.");
        }
        elseif(isset($announce)) {
            $text->value = tl::tlx('Announcement {$arg[1]} installed.', $announce);
        }
        elseif(isset($mtls)) {
            $text->value = tl::tl("Successfully changed mutual TLS configuration.").$reloadWebServer;
        }
        if($text->value != "") {
            $text->autorender = true;
            $text->value = "<i>{$text->value}</i><p>&nbsp;</p>";
        }    
    }
    
    public function tabAnnouncements() {
        /* @var $mainSession MainSession */ global $mainSession;
        $this->currentSection = InnovaphoneAdminConsts::tabAnnounce;
        $this->addActionText();
        $this->addField($txt = new InputPageText("txt", tl::tl("You can upload a 16bit,8khz,mono wave file, which will be converted to G711U/G711A/G723/G729.")));
        $path = "http://{$mainSession->dhcpAddr}/webdav/announcements/";
        $txt->value .= "<br>".tl::tlx('The resulting files will be stored under {$arg[1]}', "<a target='_blank' href='$path'>$path</a>");
        $txt->value .= "<br><br>";
        $this->addField($chk = new InputPageCheckboxField("zip", tl::tl("Return files as ZIP file")));
        $this->addField($upload = new InputPageFileUploadField("file", tl::tl("Wave File")));
        $upload->button = &$this->addField(new ButtonAnnouncement("buttonannounce", tl::tl("Ok")));
        $upload->emptyOk = true;
    }

    public function tabRelayHosts() {
        global $mainSession;
        $this->currentSection = InnovaphoneAdminConsts::tabRelayHosts;
        $this->addActionText();
        $createRelay = tl::tl("Add relay host");
        $relay = $this->addField(new InputPageText("addrelay", $this->getModeLink(InnovaphoneAdminConsts::modeRelay, $createRelay)));
        $relay->fullline = true;

        $dummy = $this->addField(new InputPageText("dummy11", "<br>"));
        $dummy->fullline = true;

        $this->addField($sender = new InputPageMailField("sender", tl::tl("Sender")));
        $this->addField($rec = new InputPageMailField("recipient", tl::tl("Recipient")));
        $rec->button = &$this->addField(new ButtonTestEmail("buttontestmail", tl::tl("Test")));
        $rec->emptyOk = $sender->emptyOk = true;
        $rec->attributes["onKeyDown"] = $sender->attributes["onKeyDown"] = "javascript:submitButton(event, '{$rec->button->name}')";

        switch($this->parent->currentMode) {
            case InnovaphoneAdminConsts::modeRelay:
                $this->modeRelay();
                break;
            case InnovaphoneAdminConsts::modeDeleteRelay:
                $this->modeDeleteRelay();
                break;
            case "":
                $this->addRelayHostList();
                break;
        }
    }
    
    function addRelayHostList() {
        $this->addSection(InnovaphoneAdminConsts::tabRelayHosts."_bottom", 1, false);
        $this->currentSection = InnovaphoneAdminConsts::tabRelayHosts."_bottom";
        
        $list = $this->addField(new MainInputPageListField("listrelay"));
        $emptyText = tl::tl("No relay hosts yet");
        $empty = $list->fieldOnEmptyTable = new InputPageText($list->name."emptytable", 
            "<b>$emptyText</b>");
        $empty->style = $this->currentSection;
        $list->addColumn($email = new InputPageText($list->name."email", null), tl::tl("Email")."/".tl::tl("Domain"));
        $email->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($server = new InputPageText($list->name."server", null), tl::tl("Server"));
        $server->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($id = new InputPageHiddenIdField($list->name."id"));
    }

    function modeDeleteRelay() {
        $this->currentSection = InnovaphoneAdminConsts::modeDeleteRelay;
        $this->addSection($this->currentSection, 1, true, tl::tlx('Delete the selected relay host \'{$arg[1]}\'?', htmlentities_utf8($this->parent->relayName)));
        $this->addField($del = new ButtonDeleteRelayHost("buttondeleterelay", InnovaphoneAdminConsts::pageAdmin, $this->parent, "", InnovaphoneAdminConsts::modeRelay));
    }
    
    function modeRelay() {
        global $mainSession;
        $this->currentSection = InnovaphoneAdminConsts::modeRelay;
        $this->addSection($this->currentSection, 1, true, tl::tl("Relay host parameters"));
        // protocol is currently just smtp
        //$prot = $this->addField(new InputPageStringField("protocol", tl::tl("Protocol"), $mainSession->relayHost->prot));
        //$prot->emptyOk = true;
        $server = $this->addField(new InputPageStringField("server", tl::tl("Server"), $mainSession->relayHost->server));
        $server->emptyOk = true;
        $server->hint = tl::tl("Enter your email server address/name, which will send your emails.");
        $port = $this->addField(new InputPageStringField("port", tl::tl("Port"), $mainSession->relayHost->port));
        $port->emptyOk = true;
        $mail = $this->addField(new InputPageStringField("email", tl::tl("Email")."/".tl::tl("Domain"), $mainSession->relayHost->email));
        $mail->emptyOk = true;
        $mail->hint = tl::tl("Enter the email address or the domain of the sender.");
        $user = $this->addField(new InputPageStringField("user", tl::tl("User"), $mainSession->relayHost->user));
        $user->emptyOk = true;
        $user->hint = tl::tl("The user used to authenticate on your email server.");
        $pwValue = "";
        if($this->parent->relayId > 0 && $mainSession->relayHost->password != "") {
            $pwValue = MainMethods::dummyPw;
        }
        $pass = $this->addField(new InputPagePasswordField("password", tl::tl("Password"), $pwValue));
        $pass->minlength = 0;
        $pass->emptyOk = true;
        $this->addSubSection(InnovaphoneAdminConsts::modeRelay, "_buttons", 1);
        $btn = $this->addField(new ButtonRelayHost("buttonrelay", InnovaphoneAdminConsts::pageAdmin, $this->parent));
        $btn->fullline = true;
        $pass->attributes["onKeyDown"] = /*$prot->attributes["onKeyDown"] = */$server->attributes["onKeyDown"] = 
            $port->attributes["onKeyDown"] = $mail->attributes["onKeyDown"] = $user->attributes["onKeyDown"] = 
                "javascript:submitButton(event, '{$btn->name}', '{$btn->name}_cancel')";
                
        if($this->parent->relayId > 0) {
            $this->addField($del = new ButtonAskDeleteRelayHost("buttondeleterelay", tl::tl("Delete")));
        }
    }

    public function tabBackup() {
        $this->currentSection = InnovaphoneAdminConsts::tabBackup;
        $this->addActionText();
        global $mainSession;
        $url = $this->addField(new MainInputPageWebPathField("backupurl", tl::tl("Command File URL"), $mainSession->webCfg->backupUrl));
        $url->emptyOk = true;
        $poll = $this->addField(new CronInputPageDigitStringField("poll", tl::tl("Poll timeout (minutes)"), $mainSession->webCfg->pollTimeout));
        $poll->emptyOk = true;
        if($mainSession->webCfg->error != "") {
            $url->msg = $mainSession->webCfg->error;
        }
        /*$ipServerAlarm = $this->addField(new InputPageIPField("alarmserverip", tl::tl("Alarm server IP"), $mainSession->webCfg->alarmServerIp));
        $ipServerAlarm->emptyOk = true;
        $ipServerAlarm->fullline = true;        */
        
        $btn = $this->addField(new ButtonCfgBackup("buttonbackup", tl::tl("Ok")));        
        $btn->fullline = true;
        
        $dummy = $this->addField(new InputPageText("dummy11", "<br>"));
        $dummy->fullline = true;
        
        $saveText = tl::tl("Save configuration files/data");
        $save = $this->addField(new InputPageText("save", $this->getModeLink(InnovaphoneAdminConsts::modeSave, $saveText)));
        $save->fullline = true;
        $restoreText = tl::tl("Restore configuration files/data");
        $restore = $this->addField(new InputPageText("restore", $this->getModeLink(InnovaphoneAdminConsts::modeRestore, $restoreText)));
        $restore->fullline = true;
        
        
        switch($this->parent->currentMode) {
            case InnovaphoneAdminConsts::modeSave:
                $this->modeSave($saveText);
                break;
            case InnovaphoneAdminConsts::modeRestore:
                $this->modeRestore($restoreText);
                break;
        }

        if($this->parent->currentMode == "") {
            $dummy = $this->addField(new InputPageText("dummy2", "<br>"));
            $dummy->fullline = true;
            
            global $mainSession;

            $list = $this->addField(new MainInputPageListField("backupserialslist"));
            $empty = $list->fieldOnEmptyTable = new InputPageText($list->name."emptytable", 
                "<i>".tl::tl("No backups yet")."</i>");
            $serial = $list->addColumn(new InputPageText($list->name."serial", null), tl::tl("Backup Serial"));
            $serial->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
            $num = $list->addColumn(new InputPageText($list->name."num", null), tl::tl("Current Index"));
            $num->sortType = genericInputPageItem::sortNumeric;
            $date = $list->addColumn(new InputPageText($list->name."date", null), tl::tl("Last Execution Time"));
            $date->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;

			$this->addSection($this->currentSection."_bottom", 1, false);
            $this->currentSection = $this->currentSection."_bottom";
            $log = $this->addField(new InputPageText("backuplog", ""));
            parse_log_file(Config::backupLog, $logVal, $error, false, true, 100);
            $log->value = $logVal;
        }
    }
    
    public function modeSave($saveText) {
        $backup = new MainBackup();
        global $mainSession;

        $this->currentSection = $this->parent->currentMode;
        $this->addSection($this->parent->currentMode, 1, true, $saveText);
        $error = $this->addField(new InputPageText("error", ""));
        $error->autorender = false;
        $btnCfg = $this->addField(new ButtonSave("buttoncfg", tl::tl("Ok"), new BackupCmd("", MainBackup::saveInnovaphoneCfgs)));
        $cfgLink = $this->addField(new InputPageText("saveinnovaphonecfgs", "<a class=\"novisit\" href=\"javascript:clickButton('{$btnCfg->name}');\">".tl::tl("Save innovaphone configuration files")."</a>"));
        $btnLogs = $this->addField(new ButtonSave("buttonlogs", tl::tl("Ok"), new BackupCmd("", MainBackup::saveInnovaphoneLogs)));
        $cfgLink = $this->addField(new InputPageText("saveinnovaphonelogs", "<a class=\"novisit\" href=\"javascript:clickButton('{$btnLogs->name}');\">".tl::tl("Save innovaphone log files")."</a>"));
        $btnDb = $this->addField(new ButtonSave("buttondb", tl::tl("Ok"), new BackupCmd("", MainBackup::saveInnovaphoneDb)));
        $dbLink = $this->addField(new InputPageText("saveinnovaphonedb", "<a class=\"novisit\" href=\"javascript:clickButton('{$btnDb->name}');\">".tl::tl("Save innovaphone database")."</a>"));
        $i = 0;
        foreach($mainSession->apps as $app) {
            foreach($app->backupCmds as $cmd) {
                $btnSave = $this->addField(new ButtonSave("buttonsave".$i, tl::tl("Ok"), $cmd));
                $dataLink = $this->addField(new InputPageText($cmd->cmd, "<a class=\"novisit\" href=\"javascript:clickButton('{$btnSave->name}');\">".$cmd->title."</a>"));
                $i++;
            }
        }
        $btnCancel = $this->addField(new ButtonCancel("buttoncancel", tl::tl("Cancel"), InnovaphoneAdminConsts::pageAdmin));
        $btnCancel->autorender = true;
    }
    
    public function modeRestore($restoreText) {
        global $mainSession;
        $hintName = "restorehint";
        $this->currentSection = $this->parent->currentMode;
        $this->addSection($this->parent->currentMode, 1, true, $restoreText);
        $restoreCfg = $this->addField(new InputPageFileUploadField(MainBackup::restoreInnovaphoneCfgs, tl::tl("Restore innovaphone configuration files")));
        $restoreCfg->button = &$this->addField(new ButtonRestore("buttonrestoreinnovaphonecfgs", tl::tl("Restore"), new RestoreCmd("", MainBackup::restoreInnovaphoneCfgs)));
        $restoreCfg->emptyOk = true;
        $restoreCfg->button->attributes["onClick"] = "javascript:display_restore_hint('{$hintName}');";
        $restoreDb = $this->addField(new InputPageFileUploadField(MainBackup::restoreInnovaphoneDb, tl::tl("Restore innovaphone database")));
        $restoreDb->button = &$this->addField(new ButtonRestore("buttonrestoreinnovaphonedb", tl::tl("Restore"), new RestoreCmd("", MainBackup::restoreInnovaphoneDb)));
        $restoreDb->emptyOk = true;
        $restoreDb->button->attributes["onClick"] = "javascript:display_restore_hint('{$hintName}');";
        $i = 0;
        foreach($mainSession->apps as $app) {
            foreach($app->restoreCmds as $cmd) {
                $restoreApp = $this->addField(new InputPageFileUploadField($cmd->cmd, $cmd->title));
                $restoreApp->button = &$this->addField(new ButtonRestore("button".$cmd->cmd, tl::tl("Restore"), $cmd));
                $restoreApp->button->attributes["onClick"] = "javascript:display_restore_hint('{$hintName}');";
                $restoreApp->emptyOk = true;
                $i++;
            }
        }
        $btnCancel = $this->addField(new ButtonCancel("buttoncancel", tl::tl("Cancel"), InnovaphoneAdminConsts::pageAdmin));
        $btnCancel->autorender = true;
        
        $this->addSection("restorehint", 1, false);
        $this->currentSection = "restorehint";
        $hint = $this->addField(new InputPageText($hintName, tl::tl("The restore process is running. Please be patient...")));
        $hint->attributes["style"] = "display:none";
        $hint->attributes["id"] = $hint->name;
    }
    
    public function modeCsr() {
        global $mainSession;
        $this->currentSection = InnovaphoneAdminConsts::modeCsr;
        $this->addSection($this->currentSection, 1, true, tl::tl("Create new application certificate"));
        $options1 = array(0 => tl::tl("Signing request"), 1 => tl::tl("Self-signed certificate"));
        $this->addField($csrtype = new InputPageDropdownField("csrtype", tl::tl("Type"), $options1));
        $csrtype->default[0] = true;
        $options = array(0 => "1024-bit", 1 => "2048-bit");
        $this->addField($keytype = new InputPageDropdownField("keytype", tl::tl("Key"), $options));
        $keytype->default[0] = true;
        $options2 = array(0 => "SHA1", 1 => "SHA256", 2 => "SHA384", 3 => "SHA512");
        $this->addField($sigtype = new InputPageDropdownField("sigtype", tl::tl("Signature"), $options2));
        $sigtype->default[0] = true;
        $validity = $this->addField(new InputPageStringField("yvalidity", tl::tl("Validity")));
        $validity->emptyOk = true;
        $validity->hint = tl::tl("Years(s), only for self-signed certificates.");
        $commonName = $this->addField(new InputPageStringField("commonname", tl::tl("Common Name")));
        $commonName->emptyOk = false;
        $commonName->attributes["maxlength"] = 64;
        $orgUnit = $this->addField(new InputPageStringField("orgunit", tl::tl("Organizational Unit")));
        $orgUnit->emptyOk = true;
        $organization = $this->addField(new InputPageStringField("organization", tl::tl("Organization")));
        $organization->emptyOk = true;
        $locality = $this->addField(new InputPageStringField("locality", tl::tl("Locality")));
        $locality->emptyOk = true;
        $stateProvince = $this->addField(new InputPageStringField("stateprovince", tl::tl("State or Province")));
        $stateProvince->emptyOk = true;
        $countryCsr = $this->addField(new InputPageStringField("countrycsr", tl::tl("Country")));
        $countryCsr->emptyOk = false;
        $countryCsr->attributes["minlength"] = 2;
        $countryCsr->attributes["maxlength"] = 2;
        $dnsName0 = $this->addField(new InputPageStringField("dnsname0", tl::tl("DNS Name")));
        $dnsName0->emptyOk = true;
        $dnsName1 = $this->addField(new InputPageStringField("dnsname1", tl::tl("DNS Name")));
        $dnsName1->emptyOk = true;
        $dnsName2 = $this->addField(new InputPageStringField("dnsname2", tl::tl("DNS Name")));
        $dnsName2->emptyOk = true;
        $ipAddr0 = $this->addField(new InputPageIPField("ipaddrcsr0", tl::tl("IP Address")));
        $ipAddr0->emptyOk = true;
        $ipAddr1 = $this->addField(new InputPageIPField("ipaddrcsr1", tl::tl("IP Address")));
        $ipAddr1->emptyOk = true;
        $this->addSubSection(InnovaphoneAdminConsts::modeCsr, "_buttons", 1);
        $btn = $this->addField(new ButtonCreateCsr("buttoncsr", InnovaphoneAdminConsts::pageAdmin, $this->parent));
        $btn->fullline = true;
        $pass->attributes["onKeyDown"] = /*$prot->attributes["onKeyDown"] = */$server->attributes["onKeyDown"] =
            $port->attributes["onKeyDown"] = $mail->attributes["onKeyDown"] = $user->attributes["onKeyDown"] =
                "javascript:submitButton(event, '{$btn->name}', '{$btn->name}_cancel')";
    }

    public function tabSSL() {
        $this->currentSection = InnovaphoneAdminConsts::tabSSL;
        $this->addActionText();

        $cmd="/usr/bin/sudo /usr/local/bin/config.sh ssl check ".Config::certServer;
        $md5sum = MainMethods::callConfigSh($cmd, $retval);
        if(!strcmp($md5sum,Config::certMD5sum)) {
            $listCurrent = $this->addSSLList("listcurrent", tl::tl("Current server certificate (Please install your own certificate)"), tl::tl("No server certificate installed."), false);
        } else {
            $listCurrent = $this->addSSLList("listcurrent", tl::tl("Current server certificate"), tl::tl("No server certificate installed."), false);
        }
        $dummy = $this->addField(new InputPageText("dummylistcurrent2", "<br>"));
        $text = $this->addField(new InputPageText("text", ""));
        $text->value = tl::tl("Select a SSL certificate (pem or crt format) and upload it to replace the current server certificate.<br>It will be automatically installed after the upload.");
        $upload = $this->addField(new InputPageFileUploadField("sslupload", tl::tl("File")));
        $upload->button = &$this->addField(new ButtonUploadSSL("buttonupload", tl::tl("Upload")));
        $upload->emptyOk = true;
        
        $listTrusted = $this->addSSLList("listtrusted", tl::tl("Trust list"), tl::tl("No trusted certificates yet."));
        $btn = $this->addField(new ButtonCerts("deletecerts", tl::tl("Remove"), $listTrusted->name, ButtonCerts::typeDelete));
        $listRejected = $this->addSSLList("listrejected", tl::tl("Rejected certificates"), tl::tl("No rejected certificates yet."));
        $btn = $this->addField(new ButtonCerts("trustcerts", tl::tl("Trust"), $listRejected->name, ButtonCerts::typeTrust));
        $btn = $this->addField(new ButtonCerts("deleterejectedcerts", tl::tl("Remove"), $listRejected->name, ButtonCerts::typeDeleteReject));
        $this->addField(new InputPageHorizontalRule());
        $this->addField(new InputPageText("dummymyappcerts", "<b>".tl::tl("Application certificates")."</b>"));
        $createCsr = tl::tl("Create new");
        $csr = $this->addField(new InputPageText("csr", $this->getModeLink(InnovaphoneAdminConsts::modeCsr, $createCsr)));
        $ssCerts = $this->addSSLList("listselfsigncerts");
        $btn = $this->addField(new ButtonCerts("installselfsigncerts", tl::tl("Install"), $ssCerts->name, ButtonCerts::typeInstallCert));
        $btn = $this->addField(new ButtonCerts("deleteselfsigncerts", tl::tl("Remove"), $ssCerts->name, ButtonCerts::typeDeleteRequest));
        $appCerts = $this->addSSLAppCertsList("listapprequests");
        $btn = $this->addField(new ButtonCerts("deleteappreqs", tl::tl("Remove"), $appCerts->name, ButtonCerts::typeDeleteRequest));

        switch($this->parent->currentMode) {
            case InnovaphoneAdminConsts::modeCsr:
                $this->modeCsr();
                break;
        }
    }
   
    public function addSSLAppCertsList($name, $text, $emptyText, $checkBox = true) {
        if($text != "") {
            $dummy = $this->addField(new InputPageText("dummy$name", "<b>$text:</b>"));
            if($checkBox) {
                $dummy->value = "<br>".$dummy->value;
            }
        }
        $list = $this->addField(new MainInputPageListField($name));
        $empty = $list->fieldOnEmptyTable = new InputPageText($name."emptytable",
            "<i>$emptyText</i>");
        if($checkBox) {
            $chkName = $name."chk";
            $list->addColumn(new InputPageCheckboxField($chkName, null), $this->parent->getUnCheckAllInput($chkName, false));
        }
        $request = $list->addColumn(new InputPageText($name."request", null), tl::tl("Request"));
        $request->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $strength = $list->addColumn(new InputPageText($name."strength", null), tl::tl("Strength"));
        $strength->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $download = $list->addColumn(new InputPageText($name."download", null), tl::tl("Download"));
        $download->sortType = genericInputPageItem::sortIgnore;
        $list->addColumn(new InputPageHiddenIdField($name."file", null), "");
        return $list;
    }
 
    public function addSSLList($name, $text, $emptyText, $checkBox = true) {
        if($text != "") {
            $dummy = $this->addField(new InputPageText("dummy$name", "<b>$text:</b>"));
            if($checkBox) {
                $dummy->value = "<br>".$dummy->value;
            }
        }
        $list = $this->addField(new MainInputPageListField($name));
        $empty = $list->fieldOnEmptyTable = new InputPageText($name."emptytable", 
            "<i>$emptyText</i>");
        if($checkBox) {
            $chkName = $name."chk";
            $list->addColumn(new InputPageCheckboxField($chkName, null), $this->parent->getUnCheckAllInput($chkName, false));
        }
        $subject = $list->addColumn(new InputPageText($name."subject", null), tl::tl("Subject"));
        $subject->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $issuer = $list->addColumn(new InputPageText($name."issuer", null), tl::tl("Issuer"));
        $issuer->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn(new MainInputPageNoBoxDateField($name."notbefore", null), tl::tl("Not before"));
        $list->addColumn(new MainInputPageNoBoxDateField($name."notafter", null), tl::tl("Not after"));
        $download = $list->addColumn(new InputPageText($name."download", null), tl::tl("Download"));
        $download->sortType = genericInputPageItem::sortIgnore;
        $list->addColumn(new InputPageHiddenIdField($name."file", null), "");
        return $list;
    }
	
    public function tabGeneral() {
        /* @var $mainSession MainSession*/ global $mainSession;
        $this->currentSection = InnovaphoneAdminConsts::tabGeneral;
        $this->addActionText();
        MainMethods::performChecks($this);
        $rootText = tl::tl("Change the root credentials");
        $authUrlsText = tl::tl("Configure Authenticated URLs");
        $ntpText = tl::tl("Configure NTP server");
        $tzText = tl::tl("Change Timezone");
        $pgText = tl::tl("Change postgresql admin password");
        $ipText = tl::tl("Configure IP");

        if($mainSession->isVm) {
            $this->addField($ip = new InputPageText("iplink", $this->getModeLink(InnovaphoneAdminConsts::modeIp, $ipText)));
        }
        $root = $this->addField(new InputPageText("rootpwlink", $this->getModeLink(InnovaphoneAdminConsts::modeRootPw, $rootText)));
        $authUrls = $this->addField(new InputPageText("authurlslink", $this->getModeLink(InnovaphoneAdminConsts::modeAuthUrls, $authUrlsText)));
        $ntpServer = $this->addField(new InputPageText("ntpserver", $this->getModeLink(InnovaphoneAdminConsts::modeNTP, $ntpText)));
        $timezone = $this->addField(new InputPageText("timezone", $this->getModeLink(InnovaphoneAdminConsts::modeTZ, $tzText)));
        $pg = $this->addField(new InputPageText("postgresqlpw", $this->getModeLink(InnovaphoneAdminConsts::modePgPw, $pgText)));
        switch($this->parent->currentMode) {
            case InnovaphoneAdminConsts::modeRootPw:
                $this->modePassword($rootText);
                break;
            case InnovaphoneAdminConsts::modeAuthUrls:
                $this->modeAuthUrls($authUrlsText);
                break;
            case InnovaphoneAdminConsts::modeNTP:
                $this->modeNTPserver($ntpText);
                break;
            case InnovaphoneAdminConsts::modeTZ:
                $this->modeTimezone($tzText);
                break;
            case InnovaphoneAdminConsts::modePgPw:
                $this->modePassword($pgText);
                break;
            case InnovaphoneAdminConsts::modeIp:
                if($mainSession->isVm) {
                    $this->modeIp($ipText);
                }
                break;
        }
    }
    
    public function tabWebServer() {
        $this->currentSection = InnovaphoneAdminConsts::tabWebServer;
        $this->addActionText();
        global $mainSession;
        $httpText = tl::tl("Change the Linux web server credentials");
        $webdavText = tl::tl("Change the Linux webdav access credentials");
        $serverText = tl::tl("Change web server properties and public access to the web/webdav");
        $mtlsText = tl::tl("Configure mutual TLS");
        $server = $this->addField(new InputPageText("server", $this->getModeLink(InnovaphoneAdminConsts::modeServer, $serverText)));
        $http = $this->addField(new InputPageText("httppw", $this->getModeLink(InnovaphoneAdminConsts::modeHttpPw, $httpText)));
        $webdav = $this->addField(new InputPageText("webdavpw", $this->getModeLink(InnovaphoneAdminConsts::modeWebdavPw, $webdavText)));
        $appText = "";
        $appName = InputPage::formInput("app");
        foreach($mainSession->apps as $app) {
            if($app->lighttpdAuth) {
                $text = tl::tlx('Change the {$arg[1]} access credentials', $app->displayName);
                $this->addField(new InputPageText("{$app->name}pw", $this->getModeLink(InnovaphoneAdminConsts::modeAppPw, $text, "", "javascript:submitApp('{$app->name}', '".InnovaphoneAdminConsts::modeAppPw."')")));
                if($app->name == $appName) {
                    $appText = $text;
                }
            }
        }
        $this->addField($mtls = new InputPageText("mtls", $this->getModeLink(InnovaphoneAdminConsts::modeMTLS, $mtlsText)));
        $this->addField(new InputPageHiddenIdField("app", ""));
        switch($this->parent->currentMode) {
            case InnovaphoneAdminConsts::modeHttpPw:
                $this->modePassword($httpText);
                break;
            case InnovaphoneAdminConsts::modeWebdavPw:
                $this->modePassword($webdavText);
                break;
            case InnovaphoneAdminConsts::modeAppPw:
                $this->modePassword($appText, $appName);
                break;
            case InnovaphoneAdminConsts::modeServer:
                $this->modeServer($serverText);
                break;
            case InnovaphoneAdminConsts::modeMTLS:
                $this->modeMTLS($mtlsText);
                break;
        }
    }
    
    public function modeMTLS($text) {
        /** @var $mainSession MainSession*/global $mainSession;
        $this->currentSection = $this->parent->currentMode;
        $this->addSection($this->parent->currentMode, 1, true, $text);
        
        $this->addField($txt = new InputPageText("txt", tl::tlx('If you activate mutual TLS, MTLS is done for files under "{$arg[1]}", which is also the document root for the configured port.', Config::mtlsPath)));
        $this->addSubSection($this->parent->currentMode, "_cfg", 1);        
        $this->addField($chk = new InputPageCheckboxField("active", tl::tl("Active")));
        $this->addField($port = new InputPageDigitstringField("port", tl::tl("MTLS Port"), $mainSession->webCfg->mtls->port));

        $portVal = InputPage::formInput($port->name);
        if(!isset($portVal)) {
            $chk->default["on"] = $mainSession->webCfg->mtls->active;
        }
        
        $this->addField($btn = new ButtonConfigureMTLS("buttonmtls", InnovaphoneAdminConsts::pageAdmin, $this->parent));
        if($mainSession->webCfg->error != "") {
            $btn->msg = $mainSession->webCfg->error;
        }
        $chk->setButtons($btn->name, $btn->name."_cancel");
        $port->setButtons($btn->name, $btn->name."_cancel");
    }
        
    public function modeServer($text) {
        global $mainSession;
        $this->currentSection = $this->parent->currentMode;
        $this->addSection($this->parent->currentMode, 1, true, $text);
        
        $forceHttps = $this->addField(new InputPageCheckboxField("forcehttps", tl::tl("Force HTTPS")));
        
        $this->addSubSection(InnovaphoneAdminConsts::modeServer, "_paths", 1);        
        $btnName = "buttoncommitcfg";
        $listWeb = $this->addPathList("publicweb", tl::tl("Public Web Paths"), $btnName);
        $listWebDav = $this->addPathList("publicwebdav", tl::tl("Public Webdav Paths"), $btnName, true);
        
        $btn = $this->addField(new ButtonCommitConfig($btnName, InnovaphoneAdminConsts::pageAdmin, $this->parent));
        if($mainSession->webCfg->error != "") {
            $btn->msg = $mainSession->webCfg->error;
        }
        $listPw->attributes["onKeyDown"] = $listUser->attributes["onKeyDown"] = "javascript:submitButton(event, '{$btn->name}', '{$btn->name}_cancel')";
    }
    
    public function addPathList($name, $title, $btnName, $webdav = false) {
        $dummy = $this->addField(new InputPageText("dummy_$name", "&nbsp;"));
        $list = $this->addField(new MainInputPageListField($name));
        $list->sortDirection = InputPageListField::sortNone;
        $list->oddAttributes["class"] = "";
        $list->evenAttributes["class"] = "";
        $list->headAttributes["class"] = "";
        $list->divattributes["class"] = "";
        $listPath = $list->addColumn(new MainInputPageWebPathField($list->name."path", null, "", null, !$webdav), $title);
        $listPath->sortType = genericInputPageItem::sortIgnore;
        $listPath->emptyOk = true;
        $listPath->hint = tl::tl("Enter a single '/' for a public root directory. If you enter e.g. '/update/', the directory 'update' and all sub directories will be public. If you enter e.g. '/update', only the update directory and its files will be public.");
        if($webdav) {
            $listWrite = $list->addColumn(new InputPageCheckboxField($list->name."write", null), tl::tl("Write"));
            $listWrite->hint = tl::tl("Check this to enable write access.");
            $listWrite->tdattributes["valign"] = "middle";
        }
        $list->removableRows = true;
        $listPath->attributes["onKeyDown"] = "javascript:submitButton(event, '{$btnName}', '{$btnName}_cancel')";
        return $list;
    }
    
    public function modeAuthUrls($text) {
        global $mainSession;
        $this->currentSection = $this->parent->currentMode;
        $this->addSection($this->parent->currentMode, 1, true, $text);
        
        $dummy = $this->addField(new InputPageText("dummy_auths", "&nbsp;"));
        $dummy->value .= "<br><b>".tl::tl("Authenticated URLs (e.g. for backup URLs)")."</b>";
        $listAuthUrls = $this->addField(new MainInputPageListField("authurls"));
        $listAuthUrls->sortDirection = InputPageListField::sortNone;
        $listAuthUrls->oddAttributes["class"] = "";
        $listAuthUrls->evenAttributes["class"] = "";
        $listAuthUrls->headAttributes["class"] = "";
        $listAuthUrls->divattributes["class"] = "";
        $listAuthUrls->removableRows = true;
        $listUrl = $listAuthUrls->addColumn(new MainInputPageWebPathField($listAuthUrls->name."url", null), tl::tl("URL"));
        $listUrl->hint = tl::tl("URL to use authentication for");
        $listUser = $listAuthUrls->addColumn(new InputPageStringField($listAuthUrls->name."user", null), tl::tl("User"));
        $listUser->hint = tl::tl("HTTP account name");
        $listPw = $listAuthUrls->addColumn(new InputPagePasswordField($listAuthUrls->name."pw", null), tl::tl("Password"));
        $listPw->hint = tl::tl("HTTP account password");
        $listPw->minlength = 1;
        $listUrl->sortType = $listUser->sortType = $listPw->sortType = genericInputPageItem::sortIgnore;
        $listUrl->emptyOk = $listUser->emptyOk = $listPw->emptyOk = true;
        
        $btn = $this->addField(new ButtonAuthUrls("buttonauth", InnovaphoneAdminConsts::pageAdmin, $this->parent));
        if($mainSession->webCfg->error != "") {
            $btn->msg = $mainSession->webCfg->error;
        }
        $listUrl->attributes["onKeyDown"] = "javascript:submitButton(event, '{$btn->name}', '{$btn->name}_cancel')";
    }
   
    static public function sortZones($a, $b) {
        if($a["offset"] == $b["offset"]) {
            return strcasecmp($a["zone"], $b["zone"]);
        }
        return $a["offset"] - $b["offset"];
    }
    
    private function getTimeZones($dir, &$arr) 
    { 
        $handle =  opendir($dir);
        while ($f = readdir($handle)) { 
            if ($f != "." && $f != "..") { 
                if (is_dir($dir.$f)) // Wenn Verzeichniseintrag ein Verzeichnis ist 
                { 
                    // Erneuter Funktionsaufruf, um das aktuelle Verzeichnis auszulesen
                    $this->getTimeZones($dir.$f.'/', $arr); 
                }
                else { 
                    // Wenn Verzeichnis-Eintrag eine Datei ist, diese ausgeben
                    $arr[] = $dir.$f;
                } 
            }
        } 
        closedir($handle);
    } 
    
    public function modeTimezone($text) {
        global $mainSession;
        if($mainSession->webCfg->error != "") {
            return $this->reRenderPage($page, $mainSession->webCfg->error);
        }
        $this->currentSection = $this->parent->currentMode;
        $this->addSection($this->parent->currentMode, 1, true, $text);

        $cmd = 'ls -l /etc/localtime | cut -d" " -f 11 | egrep -oe "[a-zA-Z_\-\+0-9]*$"';
        $res = exec($cmd, $retVal);
        if($res != "") { 
                $res_len = strlen($res);
                $toCmp = substr($mainSession->webCfg->timezone, -$res_len);
                if(($toCmp != "") && strcmp($toCmp, $res)) {
                    $mainSession->webCfg->timezone = "Timezone was changed! (".$res."?)";
                }
        }     
        
        $zones = array();
        $zoneInfo = array();
        $zoneFolder = "/usr/share/zoneinfo/";
        $this->getTimeZones($zoneFolder, $zoneInfo);
        $theTime = time(); # specific date/time we're checking, in epoch seconds.
        $zoneList = timezone_identifiers_list();
        foreach($zoneInfo as $info) {
            $info = str_replace($zoneFolder, "", $info);
            if(!in_array($info, $zoneList)) continue;
            try
            {
                $tz = new DateTimeZone($info);
                if($tz !== false) {
                    $transition = $tz->getTransitions($theTime,$theTime);
                    $offset = $transition[0]["offset"];
                    if($transition[0]["isdst"]) $offset -= 60*60;
                    $zones[$info] = array("offset" => $offset,
                                        "zone" => $info);
                }
            }
            catch(Exception $e) { continue; }
        }
        uasort($zones, array("self", "sortZones"));
        $dropZones = array();
        foreach($zones as $zone) {
            $sign = "";
            $offset = $zone["offset"];
            if($offset < 0) {
                $sign = "-";
                $offset = (-1) * $offset;
            }
            else $sign = "+";
            $dropZones[$zone["zone"]] = sprintf("(UTC %s%02d:%02d) %s", $sign, floor($offset/60/60), $offset%60, $zone["zone"]);
        }
        $tzField = $this->addField(new InputPageDropdownField("tz", tl::tl("Timezone"), $dropZones));
        $tzField->default[$mainSession->webCfg->timezone] = true;
        $tzField->button = &$this->addField(new ButtonChangeTz("buttonchangetz", tl::tl("Ok")));
        $tzField->emptyOk = true;
        $tzField->attributes["onKeyDown"] = "javascript:submitButton(event, '{$tzField->button->name}')";
    }
 
    public function modeNTPserver($text) {
        global $mainSession;
        if($mainSession->webCfg->error != "") {
            return $this->reRenderPage($page, $mainSession->webCfg->error);
        }
        $this->currentSection = $this->parent->currentMode;

        $this->addField(new InputPageHorizontalRule());
        
        $ip = $this->addField(new InputPageText("addntp", "<br>". tl::tl("Add a NTP server")));  
        $ntpField = $this->addField(new InputPageIPField("ntp", tl::tl("NTP server")));
        $ntpField->button = &$this->addField(new ButtonAddNtpServer("buttonaddntp", tl::tl("Ok")));
        $ntpField->emptyOk = true;
        $ntpField->attributes["onKeyDown"] = "javascript:submitButton(event, '{$ntpField->button->name}')";

        $this->addField(new InputPageHorizontalRule());
        
        
        $list = $this->addField( new MainInputPageListField("ntplist") );  // will be filled in fillList
        $list->fieldOnEmptyTable = new InputPageText($list->name."emptytable", "");
        // its columns
        $fld = $list->addColumn(new InputPageTextField($list->name."ntp", null), tl::tl("NTP server"));
        $chkName = $list->name."chkdelete";
        $chk = $list->addColumn(new InputPageCheckboxField($chkName, null), $this->parent->getUnCheckAllInput($chkName, false));
    
        $this->addField(new ButtonDeleteNtpServer("buttondeletentp", tl::tl("Delete")));
    }
    
    public function modeIp($text) {
        $this->currentSection = $this->parent->currentMode;
        $this->addSection($this->parent->currentMode, 1, true, $text);
        $this->addField($txt = new InputPageText("text", ""));
        $txt->autorender = false;
        $options = array(0 => tl::tl("Static"),
                        1 => tl::tl("DHCP Client"));
        /* @var $ipVals IPValues */ $ipVals = MainMethods::retrieveIPVals();
        $this->addField($dhcp = new InputPageDropdownField("dhcp", tl::tl("Mode"), $options));
        $dhcp->default[(int)$ipVals->dhcp] = true;
        $this->addField($ip = new InputPageIPField("ip", tl::tl("IP Address"), $ipVals->ip));
        $ip->emptyOk = true;
        $this->addField($mask = new InputPageIPField("mask", tl::tl("Network Mask"), $ipVals->mask));
        $mask->emptyOk = true;
        $this->addField($gw = new InputPageIPField("gw", tl::tl("Default Gateway"), $ipVals->gateway));
        $gw->emptyOk = true;
        $this->addField($dns1 = new InputPageIPField("dns1", tl::tl("DNS Server"), $ipVals->dns1));
        $dns1->emptyOk = true;
        $this->addField($dns2 = new InputPageIPField("dns2", tl::tl("Alternate DNS Server"), $ipVals->dns2));
        $dns2->emptyOk = true;
        $java = "changeIPMode('{$dhcp->name}[]', '{$ip->name}', '{$mask->name}', '{$gw->name}', '{$dns1->name}', '{$dns2->name}');";
        $dhcp->attributes["onChange"] = "javascript:$java";
        $dhcp->setJavaScriptInit("function JavaScriptInit_{$dhcp->name}() { $java }");
        $this->addField($btn = new ButtonIPConfig("buttonip", InnovaphoneAdminConsts::pageAdmin, $this->parent));
        $dhcp->attributes["onKeyDown"] = $ip->attributes["onKeyDown"] = 
            $gw->attributes["onKeyDown"] = $mask->attributes["onKeyDown"] = 
            $dns1->attributes["onKeyDown"] = $dns2->attributes["onKeyDown"] = 
            "javascript:submitButton(event, '{$btn->name}', '{$btn->name}_cancel')";
    }
    
    public function modePassword($text, $appName = "") {
        $this->currentSection = $this->parent->currentMode;
        $this->addSection($this->parent->currentMode, 1, true, $text);
        $user = $this->addField(new InputPageStringField("user", tl::tl("User")));
        $currentUser = $this->addField(new InputPageHiddenIdField("currentuser"));
        global $mainSession;
        $app = null;
        if($appName != "") {
            $app = $mainSession->getApp($appName);
        }
        $pwFile = "";
        switch($this->parent->currentMode) {
            case InnovaphoneAdminConsts::modeHttpPw:
                $pwFile = Config::serverPassFile;
                break;
            case InnovaphoneAdminConsts::modeWebdavPw:
                $pwFile = Config::webdavPassFile;
                break;
            case InnovaphoneAdminConsts::modeAppPw:
                $pwFile = $app->authFile;
                break;
        }
        if($this->parent->currentMode == InnovaphoneAdminConsts::modeRootPw) {
            if($_SERVER['HTTPS'] != "on") {
                header("location: https://".$_SERVER["HTTP_HOST"].$_SERVER["SCRIPT_NAME"]."?_page=".InnovaphoneAdminConsts::pageAdmin."&tab=".InnovaphoneAdminConsts::tabGeneral."&mode=".InnovaphoneAdminConsts::modeRootPw);
                exit;
            }
            $user->attributes["readonly"] = "readonly";
            $user->default = $currentUser->default = "root";
        } elseif($this->parent->currentMode == InnovaphoneAdminConsts::modePgPw) {
            $user->attributes["readonly"] = "readonly";
            $user->default = $currentUser->default = "postgres";
        }
        else {
            if(file_exists($pwFile)) {
                $cmd = "/bin/cat $pwFile | head -n 1 | cut -d':' -f1";
                exec($cmd,$retval);
                if(count($retval)==0) {
                    $user->msg = tl::tl("Could not find username");
                } else {
                    $user->default = $currentUser->default = $retval[0];
                }
            }
            $user->emptyOk = true;
        }
        $curPw = $this->addField(new InputPagePasswordField("currentpw", tl::tl("Current password")));
        $curPw->emptyOk = true;
        $newPw = $this->addField(new InputPagePasswordField("newpw", tl::tl("New password")));
        $newPw->emptyOk = true;
        $verPw = $this->addField(new InputPageRetypePasswordField($newPw, "verpw", tl::tl("Verify password")));
        $verPw->emptyOk = true;        
        $curPw->minlength = $newPw->minlength = $verPw->minlength = 1;
        $btn = $this->addField(new ButtonChangePassword("buttonpassword", InnovaphoneAdminConsts::pageAdmin, $this->parent, $appName));
        if($user->msg != "") $btn->disabled = true;
        $curPw->attributes["onKeyDown"] = $newPw->attributes["onKeyDown"] = 
            $verPw->attributes["onKeyDown"] = $user->attributes["onKeyDown"] = 
            "javascript:submitButton(event, '{$btn->name}', '{$btn->name}_cancel')";
    }
    
    public function fillServerPathList($name, $cfgVals) {
        $list = $this->parent->getField($name);
        $rows = 0;
        
        if(!$list->sent) {
            if(count($cfgVals)) {
                foreach($cfgVals as $path) {
                    $list->value[$rows][$list->name."path"] = $path->path;
                    $list->value[$rows][$list->name."write"]["on"] = $path->write;
                    $rows++;
                }
            }
            else {
                $list->value[0][$list->name."path"] = "";
                $list->value[0][$list->name."write"]["on"] = false;
            }
        }
        else {
        }
        return $list;
    }
    
    public function fillSSLList($name, $path) {
        $list = $this->parent->getField($name);
        $values = $list->value;
        $list->value = array();
        $certs = Certificates::getCertificates($path);
        $rows = 0;
        foreach($certs as $cert) {
            if($cert->file == Config::certDiffieHellman) continue;
            $subject = htmlentities_utf8($cert->getSubject());
            $list->value[$rows][$name."subject"] = "<a name=\"$subject\" class=\"myinno-listlink\" href=\"#\" onClick=\"javascript:self.open('certificate.php?type=show&file=".urlencode($cert->file)."','','width=600, height=400, resizable=yes, scrollbars=yes'); return;\">$subject</a>";
            $list->value[$rows][$name."issuer"] = htmlentities_utf8($cert->getIssuer());
            $list->value[$rows][$name."notbefore"] = $cert->notBefore;
            $list->value[$rows][$name."notafter"] = $cert->notAfter;
            if(!strcmp($name, "listcurrent"))
                $list->value[$rows][$name."download"] = "<a class=\"myinno-listlink\" href=\"certificate.php?type=download&format=pem&file=".urlencode($cert->file)."\">PEM</a>";
            else {
                $list->value[$rows][$name."download"] = "<a class=\"myinno-listlink\" href=\"certificate.php?type=download&format=pem&file=".urlencode($cert->file)."\">PEM </a><a class=\"myinno-listlink\" href=\"certificate.php?type=download&format=der&file=".urlencode($cert->file)."\">DER</a>";
            }
            $list->value[$rows][$name."file"] = $cert->file;
            $rows++;
        }
        if(count($values)) {   // assign certain columns now with former values, given by parameter $values
            unset($val);
            foreach($list->value as &$val) {
                foreach($values as $value) {
                    if($value[$name."file"] == $val[$name."file"]) {
                        $val[$name."chk"] = isset($value[$name."chk"]) ? $value[$name."chk"] : null;
                        break;
                    }
                }
            }
        }
        return $list;
    }

    public function fillSSLRequestsList($name) {
        $list = $this->parent->getField($name);
        $values = $list->value;
        $list->value = array();
        $rows = 0;
        $dir = Config::certRequests;
        if(is_dir($dir)) {
            if ($dh = opendir($dir)) {
                while (($file = readdir($dh)) !== false) {
                    if(strstr($file, ".csr")) {
                        $subject = strstr($file, '.csr', true);
                        $list->value[$rows][$name."request"] = "<a name=\"$subject\" class=\"myinno-listlink\" href=\"#\" onClick=\"javascript:self.open('certificate.php?type=show_req&file=".urlencode($file)."','','width=600, height=400, resizable=yes, scrollbars=yes'); return;\">$subject</a>";
                        $content = "";
                        $cmd = "openssl req -text -noout -in ".$dir."/".$file." | grep 'Public-Key' | egrep -oe '[0-9]* bit'";
                        exec($cmd, $content);
                        if(!$content) {
                            $strength = "Unknown";
                        }
                        else {
                            $strength = $content[0];
                        }
                        $list->value[$rows][$name."strength"] = $strength;
                        $derFile = str_replace(".csr",".der",$file);
                        $list->value[$rows][$name."download"] = "<a class=\"myinno-listlink\" href=\"certificate.php?type=download_req&file=".urlencode($dir."/".$file)."\">PEM</a>"." "."<a class=\"myinno-listlink\" href=\"certificate.php?type=download_req&file=".urlencode($dir."/".$derFile)."\">DER</a>";
                        $list->value[$rows][$name."file"] = $file;
                        $rows++;
                    }
                }
                closedir($dh);
             }
         }
         if(count($values)) {   // assign certain columns now with former values, given by parameter $values
             unset($val);
             foreach($list->value as &$val) {
                 foreach($values as $value) {
                    if($value[$name."file"] == $val[$name."file"]) {
                        $val[$name."chk"] = isset($value[$name."chk"]) ? $value[$name."chk"] : null;
                        break;
                    }
                }
             }
         }
         return $list;
    }   
 
    public function fillSSLLists() {
        $listCurrent = $this->fillSSLList("listcurrent", Config::certDirServer);   
        $listTrusted = $this->fillSSLList("listtrusted", Config::certDirTrusted);   
        $listRejected = $this->fillSSLList("listrejected", Config::certDirRejected);
        $listSsCerts = $this->fillSSLList("listselfsigncerts", Config::certSelfSignedCerts);
        $btnTrust = $this->parent->getField("trustcerts");
        $btnDeleteReject = $this->parent->getField("deleterejectedcerts");
        $btnTrust->autorender = $btnDeleteReject->autorender = (count($listRejected->value) ? true : false);
        $btnDelete = $this->parent->getField("deletecerts");
        $btnDelete->autorender = (count($listTrusted->value) ? true : false);
        $btnInstallSsCerts = $this->parent->getField("installselfsigncerts");
        $btnInstallSsCerts->autorender = (count($listSsCerts->value) ? true : false);
        $btnDeleteSsCerts = $this->parent->getField("deleteselfsigncerts");
        $btnDeleteSsCerts->autorender = (count($listSsCerts->value) ? true : false);
        $listRequests = $this->fillSSLRequestsList("listapprequests");
        $btnDeleteReq = $this->parent->getField("deleteappreqs");
        $btnDeleteReq->autorender = (count($listRequests->value) ? true : false);
    }
    
    public function fillServerList() {
        global $mainSession;
        $listWeb = $this->fillServerPathList("publicweb", $mainSession->webCfg->publicWebPaths);
        $listWebDav = $this->fillServerPathList("publicwebdav", $mainSession->webCfg->publicWebDavPaths);
        if(!$listWeb->sent) {
            $forceHttps = $this->parent->getField("forcehttps");
            $forceHttps->value["on"] = $mainSession->webCfg->forceHttps;
        }
    }
   
    public function fillNtpList() {
        $list = $this->parent->getField("ntplist");
        if(!is_object($list)) return;
        $values = $list->value;
        $list->value = array();
        $return = false;
        if(($ntps = read_ntps_server($error)) === false) {
            $list->fieldOnEmptyTable->value = "<div style=\"color:red\">$error</div>";
            $return = true;
        }
        else if(count($ntps) == 0) {
            $list->fieldOnEmptyTable->value = "<b>".tl::tl("No NTP servers defined yet.")."</b>";
            $return = true;
        }
        if($return) {
            $buttonDelete = $this->parent->getField("buttondeletentp");
            $buttonDelete->autorender = false;
            return;
        }
        $rows = 0;
        $list->value = array();
        foreach($ntps as $ntp) {
            $list->value[$rows][$list->name."ntp"] = $ntp;
            $rows++;
        }
        
        if(count($values)) {   // assign certain columns now with former values, given by parameter $values
            unset($val);
            foreach($list->value as &$val) {
                foreach($values as $value) {
                    if($value[$list->name."ntp"] == $val[$list->name."ntp"]) {
                        $val[$list->name."chkdelete"] = isset($value[$list->name."chkdelete"]) ? $value[$list->name."chkdelete"] : null;
                        break;
                    }
                }
            }
        }
    }
 
    public function fillAuthUrlList() {
        global $mainSession;
        $listAuthUrls = $this->parent->getField("authurls");
        $rows = 0;
        if(!$listAuthUrls->sent) {
            if(count($mainSession->webCfg->authUrls)) {
                foreach($mainSession->webCfg->authUrls as $authUrl) {
                    $listAuthUrls->value[$rows][$listAuthUrls->name."url"] = $authUrl->url;
                    $listAuthUrls->value[$rows][$listAuthUrls->name."user"] = $authUrl->user;
                    $listAuthUrls->value[$rows][$listAuthUrls->name."pw"] = $authUrl->pw;
                    $rows++;
                }
            }
            else {
                $listAuthUrls->value[0][$listAuthUrls->name."url"] = "";
                $listAuthUrls->value[0][$listAuthUrls->name."user"] = "";
                $listAuthUrls->value[0][$listAuthUrls->name."pw"] = "";
            }
        }
        else {
        }
    }
    
    public function fillSerialList()
    {
        global $mainSession;
        $list = $this->parent->getField("backupserialslist");
        $rows = 0;
        $this->parseSerialDirectory($list, $rows, Config::tmpBackupSerials);
        foreach($mainSession->apps as $app) {
            $appDir = Config::applicationsDir."/".$app->dir."/".Config::tmpBackupSerials;
            $this->parseSerialDirectory($list, $rows, $appDir);
        }   
    }
    
    public function parseSerialDirectory(MainInputPageListField &$list, &$rows, $dir) 
    {
        if (file_exists($dir) && $handle = opendir($dir)) {
            while (false !== ($file = readdir($handle))) {
                if ($file != "." && $file != "..") {
                    $list->value[$rows][$list->name."serial"] = $file;
                    $contentFile = file($dir.$file);
                    $list->value[$rows][$list->name."num"] = $contentFile[0];
                    $list->value[$rows][$list->name."date"] = $contentFile[1];
                    $rows++;
                }
            }
            closedir($handle);
        }
    }
    
    public function fillRelayHostList() {
        $list = $this->parent->getField("listrelay");
        if(!is_object($list)) return;
        $rows = 0;
        
        $innovaphoneDb = get_innovaphone_db();
        $query = "SELECT id, server, email FROM relay_hosts ORDER BY LOWER(email)";
        $result = $innovaphoneDb->run($query);
        while(($row = pg_fetch_object($result))) {
            $list->value[$rows][$list->name."email"] = "<a id=\"".htmlentities_utf8($row->email)."\" class=\"myinno-listlink\" href=\"javascript:submitRelayHost('{$row->id}', '".InnovaphoneAdminConsts::modeRelay."')\">".htmlentities_utf8($row->email)."</a>";
            $list->value[$rows][$list->name."server"] = $row->server;
            $list->value[$rows][$list->name."id"] = $row->id;
            $rows++;
        }
    }
    
    public function fillLists() {
        switch($this->parent->currentTab) {
            case InnovaphoneAdminConsts::tabGeneral:
                switch($this->parent->currentMode) {
                    case InnovaphoneAdminConsts::modeAuthUrls:
                        $this->fillAuthUrlList();
                        break;
                    case InnovaphoneAdminConsts::modeNTP:
                        $this->fillNtpList();
                        break;
                }
                break;
            case InnovaphoneAdminConsts::tabWebServer:
                switch($this->parent->currentMode) {
                    case InnovaphoneAdminConsts::modeServer:
                        $this->fillServerList();
                        break;
                }
                break;
            case InnovaphoneAdminConsts::tabSSL:
                $this->fillSSLLists();
                break;
            case InnovaphoneAdminConsts::tabBackup:
                if($this->parent->currentMode == "") {
                    $this->fillSerialList();
                }
                break;
            case InnovaphoneAdminConsts::tabPgsql:
                $this->fillIpList("innovaphone");
                break;
            case InnovaphoneAdminConsts::tabRelayHosts:
                switch($this->parent->currentMode) {
                    case "":
                        $this->fillRelayHostList();
                        break;
                }
                break;
        }
    }
}

final class InnovaphoneAdmin extends MainInputPage {
    var $relayId = 0;
    var $relayName = "";
	// constructor
    public function InnovaphoneAdmin() {
        parent::MainInputPage(tl::tl("innovaphone Administration"), "innovaphoneadmin");
		$this->setFieldSectionAttributes("inline", new _InputPageFieldSectionAttributes(array(), 1));
        /* @var $mainSession MainSession */ global $mainSession;
        
        // dummy fields to save certain parameters over all pages
        $this->addField(new InputPageHiddenIdField("relay_id", $this->relayId));
        $this->relayId = InputPage::formInput("relay_id");
        if(!isset($this->relayId)) {
            $this->relayId = 0;
        }
        else {
            $this->relayId = intval($this->relayId);
        }
        
        switch($this->currentMode) {
            case "":
                $mainSession->updateRelayHost(new RelayHost());
                break;
            case InnovaphoneAdminConsts::modeRelay:
            case InnovaphoneAdminConsts::modeDeleteRelay:
                if($mainSession->relayHost->id != $this->relayId) {
                    if($this->relayId > 0) {
                        $relayHost = new RelayHost();
                        $relayHost->read($this->relayId);
                        if($relayHost->id > 0) {
                            $mainSession->updateRelayHost($relayHost);
                        }
                        else {
                            $mainSession->updateRelayHost(new RelayHost());
                            $this->relayId = 0;
                        }
                    }
                    else {
                        $mainSession->updateRelayHost(new RelayHost());
                    }
                }
                break;
        }
        $this->relayName = $mainSession->relayHost->email;
        
        // add proper fields to pages
        $this->pages = array(InnovaphoneAdminConsts::pageAdmin => new PageAdministration($this, false));

        // update page
        $this->setDefaultTabProperties();
        $this->pages[$this->pageNo]->createFields();
    }
    
    public function getArgs() {
        $id = "";
        if($this->relayId > 0) {
            $id = "relay_id={$this->relayId}&";
        }
        return "{$id}tab={$this->currentTab}";
    }
}

function read_ntps_server(&$error) {
    $cmd="/usr/bin/sudo /usr/local/bin/config.sh ntp read";
    exec($cmd, $content);
    if(!$content) {
        return false;
    }
    $matches = array();
    $ntps = array();
    foreach($content as $line) {
        if(strstr($line,"ERROR")) {
            $error = $line;
            return false;
        }
        $ntp = trim($line);
        if(array_search($ntp, $ntps) === false) {
            $ntps[] = $ntp;
        }
    }
    return $ntps;
}

// create page
$page = new InnovaphoneAdmin;

if($page->pageNo == InnovaphoneAdminConsts::pageAdmin) {
	$page->updateFields();
    $page->pages[$page->pageNo]->fillLists();
}

$page->work();

?>
