<?php

/*
 * the home page
 */
require_once("include_main.php");

InputPage::$method = "post";

$app = null;
global $mainSession;

if (preg_match("/^.*\/apps\/(.*)\/.*.php$/", $_SERVER["SCRIPT_FILENAME"], $matches) !== false && isset($matches[1])) {
    $app = $mainSession->getApp($matches[1]);
}
if ($app == null) {
    class_alias('MainInputPage', 'ParentClass');
} else {
    switch ($app->name) {
        case "innovaphone-reporting":
            require_once("include.php");
            class_alias('ReportingInputPage', 'ParentClass');
            break;
        case "innovaphone-exchange":
            require_once("include.php");
            class_alias('ExchangeInputPage', 'ParentClass');
            break;
        case "innovaphone-faxserver":
            require_once("include.php");
            class_alias('FaxInputPage', 'ParentClass');
            break;
    }
    // reinit language
    global $translator;
    $translator->change($translator->currentLanguage());
}

class ButtonShutdown extends ButtonHidden {

    var $reboot;

    public function ButtonShutdown($name, $label, $reboot) {
        parent::__construct($name, $label);
        $this->reboot = $reboot;
    }

    function action(InputPage &$page, $phase) {
        $text = $page->getField("text");
        if ($this->reboot) {
            $text->value = tl::tl("The system reboots now. The web server will be available in a few seconds.");
        } else {
            $text->value = tl::tl("The system shuts down now.");
            if (runs_in_vm()) {
                $text->value .= " " . tl::tl("You can restart it with your VMWare Player.");
            } else {
                $text->value .= " " . tl::tl("You can restart it at your innovaphone device.")." ".tl::tl("Use Stop and Start to restart!");
            }
        }
        $text->autorender = true;
        $shutdown = $page->getField("shutdown");
        $shutdown->autorender = false;
        if (runs_in_vm()) {
            $reboot = $page->getField("reboot");
            $reboot->autorender = false;
        }
        $cmd = "/usr/bin/sudo /sbin/shutdown -";
        if ($this->reboot) {
            $cmd .= "r";
        }
        else {
            $cmd .= "h";
        }
        $cmd .= " -t 5 0 2>&1";
        $t = main_system($cmd, $ret);
        if ($ret != 0) {
            $text->value .= "<br><br><div style=\"color:red\">" . tl::tlx('Shutdown failed with: {$arg[1]}', $t) . "</div>";
        }
        $page->render();
    }

}

class ButtonDownloadLog extends ButtonHidden {

    public function ButtonDownloadLog($name, $label) {
        parent::__construct($name, $label);
    }

    function action(InputPage &$page, $phase) {
        $file = $page->getField("logfile");
        $fileName = $file->value;
        switch ($fileName) {
            case Config::debInstalledPackages:
                $cmd = "/usr/bin/sudo /usr/local/bin/config.sh debian-packages view " . Config::debInstalledPackages;
                exec($cmd);
                break;
            case Config::applInstallLog:
            case Config::applUninstallLog:
                $fileName = "$fileName.1";
                break;
            default:
                if ((!file_exists($fileName) || filesize($fileName) == 0) && strpos($fileName, "*") === false)
                    $fileName = "$fileName.1";
        }
        if(strpos($fileName, "*") !== false) {  // download an archive with log files
            $backup = new MainBackup();
            if ($backup->saveLogs("logs_#i_#d.tar.gz", "echo", $error, null, $fileName) === false) {
                $page->pages[$page->pageNo]->setLogError($error);
                $page->render();
            }
        }
        else if (file_exists($fileName)) {
            $size = filesize($fileName);
            if ($size > 0) {
                $fileNameHeader = trim(strrchr($fileName, "/"), "/");
                header('Pragma: private');
                header('Cache-control: private, must-revalidate');
                header("Content-Disposition: attachment; filename=\"$fileNameHeader\"");
                header('Content-type: text');
                header('Content-len: ' . $size);
                readfile($fileName);
            } else {
                $page->pages[$page->pageNo]->setLogError(tl::tlx('Log file {$arg[1]} is empty.', $file->value));
                $page->render();
            }
        } else {
            $page->pages[$page->pageNo]->setLogError(tl::tlx('Log file {$arg[1]} does not exist.', $file->value));
            $page->render();
        }
    }

}

class ButtonClearLog extends ButtonHidden {

    public function ButtonClearLog($name, $label) {
        parent::__construct($name, $label);
    }

    function action(InputPage &$page, $phase) {
        $file = $page->getField("logfile");
        $fileName = $file->value;
        if(strpos($fileName, "*") === false) {
            file_put_contents($fileName, "");
            if (file_exists($fileName) && filesize($fileName) > 0) {
                unlink($fileName);
                file_put_contents($fileName, "");
            }
            $index = 1;
            while ($index < 100) {
                if (file_exists("$fileName.$index")) {
                    unlink("$fileName.$index");
                } else if (file_exists("$fileName.$index.gz")) {
                    unlink("$fileName.$index.gz");
                } else if ($index >= 10) {
                    break;
                }
                $index++;
            }
        }
        else {
            system("rm -f $fileName");
        }
        $retval = 0;
        system("sudo /usr/local/bin/config.sh rsyslog 2>&1", $retval);
        $out = tl::tlx('Log file {$arg[1]} cleared.', $file->value);
        if ($retval > 0) {
            $out .= " " . tl::tl("rsyslog reload failed!");
        }
        system("kill -HUP `cat /var/run/lighttpd.pid` 2>&1", $retval);
        if ($retval > 0) {
            $out .= " " . tl::tl("SIGHUP to lighttpd failed!");
        }
        system("kill -HUP `cat /var/run/lighttpd/process.fcgi.pid` 2>&1", $retval);
        if ($retval > 0) {
            $out .= " " . tl::tl("SIGHUP to process.fcgi failed!");
        }
        $page->pages[$page->pageNo]->setLogError($out);
        $page->render();
    }

}

class ButtonReloadRsyslog extends ButtonHidden {

    public function ButtonReloadRsyslog($name, $label) {
        parent::__construct($name, $label);
    }

    function action(InputPage &$page, $phase) {
        system("sudo /usr/local/bin/config.sh rsyslog 2>&1", $retval);
        $out = "";
        if ($retval > 0) {
            $out = tl::tl("rsyslog reload failed!");
        } else {
            $out = tl::tl("rsyslog reloaded");
        }
        $page->pages[$page->pageNo]->setLogError($out);
        $page->render();
    }

}

class ButtonDownloadAllLogs extends ButtonHidden {

    public function ButtonDownloadLog($name, $label) {
        parent::__construct($name, $label);
    }

    function action(InputPage &$page, $phase) {
        $backup = new MainBackup();
        global $mainSession;
        if ($mainSession->webCfg->error != "") {
            $page->pages[$page->pageNo]->setLogError($mainSession->webCfg->error);
            $page->render();
            return;
        }
        if ($backup->saveLogs("logs_#i_#d.tar.gz", "echo", $error, $page->app) === false) {
            $page->pages[$page->pageNo]->setLogError($error);
            $page->render();
            return;
        }
    }

}

class ButtonViewLog extends ButtonHidden {

    public function ButtonViewLog($name, $label) {
        parent::__construct($name, $label);
    }

    function action(InputPage &$page, $phase) {
        $file = $page->getField("logfile");
        $fileName = $file->value;
        switch ($fileName) {
            case Config::debInstalledPackages:
                $cmd = "/usr/bin/sudo /usr/local/bin/config.sh debian-packages view " . Config::debInstalledPackages;
                exec($cmd);
                break;
            case Config::applInstallLog:
            case Config::applUninstallLog:
                $fileName = "$fileName.1";
                break;
            default:
                if (!file_exists($fileName) || filesize($fileName) == 0)
                    $fileName = "$fileName.1";
        }
        if (file_exists($fileName)) {
            $log = $page->getField("log");
            if (filesize($fileName)) {
                $content = file_get_contents($fileName);
                $content = iconv('utf-8', 'utf-8//IGNORE', $content);
                $content = htmlentities_utf8($content);
                $content = str_replace("\r", "", $content);
                $content = str_replace("\n", "<br>", $content);
                if ($fileName == Config::debInstalledPackages)
                    $log->value = "<pre>" . $content . "</pre>";
                else
                    $log->value = $content;
            } else {
                $log->value = tl::tlx('Log file {$arg[1]} is empty.', $file->value);
            }
        } else {
            $page->pages[$page->pageNo]->setLogError(tl::tlx('Log file {$arg[1]} does not exist.', $file->value));
        }
        $page->render();
    }

}

class ButtonRpcap extends InputPageButtonOnlyAction {

    public function ButtonRpcap($name, $label) {
        parent::__construct($name, $label);
        $this->anyway = false;
    }

    function action(InputPage &$page, $phase) {
        global $mainSession;
        if ($mainSession->webCfg->error != "") {
            return $this->reRenderPage($page, $mainSession->webCfg->error);
        }
        $rpcap = $page->getField("rpcapval");
        $resCheck = exec("sudo /usr/local/bin/config.sh rpcap check 2>&1", $retval);
        if ($rpcap->value["on"]) {
            if ($resCheck != 2) {
                if ($resCheck == 0) {
                    $cmd = "sudo /usr/local/bin/config.sh rpcap start 2>&1";
                } else {
                    $cmd = "sudo /usr/local/bin/config.sh rpcap restart 2>&1";
                }
                $error = MainMethods::callConfigSh($cmd, $retval);
                if ($error != "") {
                    $page->pages[$page->pageNo]->setLogError(tl::tl('RPCAP could not be started').": $error");
                }
            }
        } else {
            if ($resCheck != 0) {
                $cmd = "sudo /usr/local/bin/config.sh rpcap stop 2>&1";
                $error = MainMethods::callConfigSh($cmd, $retval);
                if ($error != "") {
                    $page->pages[$page->pageNo]->setLogError(tl::tl('RPCAP could not be stopped').": $error");
                }
            }
        }
        $mainSession->webCfg->rpcapOn = $rpcap->value["on"];
        $mainSession->webCfg->write();
        $page->render();
    }

}

class ButtonAlarms extends InputPageButtonOnlyAction {

    public function ButtonAlarms($name, $label) {
        parent::__construct($name, $label);
        $this->anyway = false;
    }

    function action(InputPage &$page, $phase) {
        global $mainSession;
        if ($mainSession->webCfg->error != "") {
            return $this->reRenderPage($page, $mainSession->webCfg->error);
        }
        $ip = $page->getField("ip");
        $port = $page->getField("port");
        $pw = $page->getField("pw");
        $user = $page->getField("user");
        $https = $page->getField("https");
        if($ip->value != "" && !$ip->valid()) return $page->render();
        if($port->value != "" && !$port->valid()) return $page->render();
        $mainSession->webCfg->alarmServer->ip = $ip->value;
        $mainSession->webCfg->alarmServer->port = $port->value;
        $mainSession->webCfg->alarmServer->pw = $pw->value;
        $mainSession->webCfg->alarmServer->user = $user->value;
        $mainSession->webCfg->alarmServer->https = $https->value["on"] ? true : false;
        $mainSession->webCfg->write();
        $page->render();
    }

}

class PageDiagnostics extends MyPage {

    public function getHelpAnchor() {
        switch ($this->parent->currentTab) {
            case InnovaphoneDiagnosticsConsts::tabLogs:
                return "#Logs";
            case InnovaphoneDiagnosticsConsts::tabReset:
                return "#Reset";
            case InnovaphoneDiagnosticsConsts::tabRpcap:
                return "#RPCAP";
            case InnovaphoneDiagnosticsConsts::tabDbExchange:
                return "#Exchange_Database";
            case InnovaphoneDiagnosticsConsts::tabFaxCalls:
                return "#Faxserver_Database";
            case InnovaphoneDiagnosticsConsts::tabAlarms:
                return "#Alarm_Server";
            case InnovaphoneDiagnosticsConsts::tabStatus:
                return "#Status";
            default: return "";
        }
    }

    public function getDefaultTab() {
        return InnovaphoneDiagnosticsConsts::tabLogs;
    }

    public function createFields() {
        $this->addSection(InnovaphoneDiagnosticsConsts::tabLogs, 2);
        $this->addSection(InnovaphoneDiagnosticsConsts::tabRpcap, 1);
        if ($this->parent->app === null) {
            $this->addSection(InnovaphoneDiagnosticsConsts::tabAlarms, 1);
            $this->addSection(InnovaphoneDiagnosticsConsts::tabReset, 1);
        }
        $this->addSection(InnovaphoneDiagnosticsConsts::tabStatus, 1);

        $genericHint = "";

        $this->parent->addTabMenuItem(tl::tl("Logs"), "javascript:submitTab('" . InnovaphoneDiagnosticsConsts::tabLogs . "')", $this->parent->currentTab == InnovaphoneDiagnosticsConsts::tabLogs);
        $this->parent->addTabMenuItem("RPCAP", "javascript:submitTab('" . InnovaphoneDiagnosticsConsts::tabRpcap . "')", $this->parent->currentTab == InnovaphoneDiagnosticsConsts::tabRpcap);
        if ($this->parent->app === null) {
            $this->parent->addTabMenuItem(tl::tl("Alarm Server"), "javascript:submitTab('" . InnovaphoneDiagnosticsConsts::tabAlarms. "')", $this->parent->currentTab == InnovaphoneDiagnosticsConsts::tabAlarms);
            $this->parent->addTabMenuItem(tl::tl("Reset"), "javascript:submitTab('" . InnovaphoneDiagnosticsConsts::tabReset . "')", $this->parent->currentTab == InnovaphoneDiagnosticsConsts::tabReset);
        }
        $this->parent->addTabMenuItem(tl::tl("Status"), "javascript:submitTab('" . InnovaphoneDiagnosticsConsts::tabStatus. "')", $this->parent->currentTab == InnovaphoneDiagnosticsConsts::tabStatus);
        if($this->parent->app != null) {
            switch($this->parent->app->name) {
                case "innovaphone-exchange":
                    $this->parent->addTabMenuItem(tl::tl("Database"), "javascript:submitTab('" . InnovaphoneDiagnosticsConsts::tabDbExchange . "')", $this->parent->currentTab == InnovaphoneDiagnosticsConsts::tabDbExchange);
                    break;
                case "innovaphone-faxserver":
                    $this->parent->addTabMenuItem(tl::tl("Calls"), "javascript:submitTab('" . InnovaphoneDiagnosticsConsts::tabFaxCalls. "')", $this->parent->currentTab == InnovaphoneDiagnosticsConsts::tabFaxCalls);
                    break;
            }
        }

        switch ($this->parent->currentTab) {
            case InnovaphoneDiagnosticsConsts::tabLogs:
                $this->tabLogs();
                break;
            case InnovaphoneDiagnosticsConsts::tabRpcap:
                $this->tabRpcap();
                break;
            case InnovaphoneDiagnosticsConsts::tabAlarms:
                $this->tabAlarms();
                break;
            case InnovaphoneDiagnosticsConsts::tabStatus:
                $this->tabStatus();
                break;
            case InnovaphoneDiagnosticsConsts::tabReset:
                if ($this->parent->app === null) {
                    $this->tabReset();
                }
                break;
            case InnovaphoneDiagnosticsConsts::tabDbExchange:
                if ($this->parent->app != null  && $this->parent->app->name == "innovaphone-exchange") {
                    $this->parent->tabDbExchange($this);
                }
                break;
            case InnovaphoneDiagnosticsConsts::tabFaxCalls:
                if ($this->parent->app != null  && $this->parent->app->name == "innovaphone-faxserver") {
                    $this->parent->tabFaxCalls($this);
                }
                break;
        }
    }

    public function setLogError($error) {
        $errorFld = $this->parent->getField("error");
        $errorFld->autorender = true;
        $errorFld->value = "<div style=\"color:red\">$error</div><br>";
    }

    public function tabStatus() {
        /* @var $mainSession MainSession*/ global $mainSession;
        $this->currentSection = InnovaphoneDiagnosticsConsts::tabStatus;
        $this->addField($txt = new InputPageText("txt", ""));
        $parts = MainMethods::getDiskInfo();
        $txt->value = "<table>";
        $txt->value .= "<tr><td>".tl::tl('Disk Size:')."</td><td>&nbsp;".$parts[MainMethods::diskSize]."</td></tr>";
        $txt->value .= "<tr><td>".tl::tl(/* Available Disk Space*/'Available:')."</td><td>&nbsp;".$parts[MainMethods::diskAvailable]."</td></tr>";
        $txt->value .= "<tr><td>".tl::tl(/* Used Disk Space*/'Used:')."</td><td>&nbsp;".$parts[MainMethods::diskUsed]."</td></tr>";
        $txt->value .= "<tr><td>".tl::tl(/* Used Disk Space*/'Used:')."</td><td>&nbsp;".$parts[MainMethods::diskUsePercent]."%</td></tr>";
        $txt->value .= "<tr><td>&nbsp;</td><td>&nbsp;</td></tr>";
        $ramI = MainMethods::getRamInfo();
        $txt->value .= "<tr><td>".tl::tl('Ram Size:')."</td><td>&nbsp;".$ramI[MainMethods::ramSize]."</td></tr>";
        $txt->value .= "<tr><td>".tl::tl('Free:')."</td><td>&nbsp;".$ramI[MainMethods::ramAvailable]."</td></tr>";
        $txt->value .= "<tr><td>".tl::tl('Used:')."</td><td>&nbsp;".$ramI[MainMethods::ramUsed]."</td></tr>";
        $txt->value .= "<tr><td>".tl::tl('Shared:')."</td><td>&nbsp;".$ramI[MainMethods::ramShared]."</td></tr>";
        $txt->value .= "<tr><td>".tl::tl('Buffers:')."</td><td>&nbsp;".$ramI[MainMethods::ramBuffers]."</td></tr>";
        $txt->value .= "<tr><td>".tl::tl('Cached:')."</td><td>&nbsp;".$ramI[MainMethods::ramCached]."</td></tr>";
        $txt->value .= "<tr><td>&nbsp;</td><td>&nbsp;</td></tr>";
        $swapI = MainMethods::getSwapInfo();
        $txt->value .= "<tr><td>".tl::tl('Swap Size:')."</td><td>&nbsp;".$swapI[MainMethods::swapSize]."</td></tr>";
        $txt->value .= "<tr><td>".tl::tl('Free:')."</td><td>&nbsp;".$swapI[MainMethods::swapAvailable]."</td></tr>";
        $txt->value .= "<tr><td>".tl::tl('Used:')."</td><td>&nbsp;".$swapI[MainMethods::swapUsed]."</td></tr>";
        $txt->value .= "<tr><td>&nbsp;</td><td>&nbsp;</td></tr>";
        $uptime = MainMethods::getUptimeInfo();
        $txt->value .= "<tr><td>".tl::tl('Uptime:')."</td><td>&nbsp;".$uptime."</td></tr>";
        $txt->value .= "</table>";
    }
    
    public function tabAlarms() {
        /* @var $mainSession MainSession*/ global $mainSession;
        $this->currentSection = InnovaphoneDiagnosticsConsts::tabAlarms;
        $this->addField($txt = new InputPageText("txt", tl::tl("Configure an innovaphone device as alarm server to receive alarms from the Linux Application Platform.")));
        $this->addField($ip = new InputPageIPField("ip", tl::tl("IP Address"), $mainSession->webCfg->alarmServer->ip));
        $ip->emptyOk = true;
        $this->addField($port = new InputPageDigitstringField("port", tl::tl("Port"), $mainSession->webCfg->alarmServer->port));
        $port->emptyOk = true;
        $this->addField($user = new InputPageStringField("user", tl::tl("User"), $mainSession->webCfg->alarmServer->user));
        $user->emptyOk = true;
        $this->addField($pw = new InputPagePasswordField("pw", tl::tl("Password"), $mainSession->webCfg->alarmServer->pw));
        $pw->emptyOk = true;
        $pw->minlength = 0;
        $this->addField($https = new InputPageCheckboxField("https", tl::tl("HTTPS")));
        $ipValue = InputPage::formInput($ip->name);
        if(!isset($ipValue)) {
            $https->default["on"] = $mainSession->webCfg->alarmServer->https;
        }
        $this->addField($btn = new ButtonAlarms("buttonalarm", tl::tl("Ok")));
    }
    
    public function tabRpcap() {
        global $mainSession;
        $this->currentSection = InnovaphoneDiagnosticsConsts::tabRpcap;
        exec("/sbin/ifconfig eth0 | grep inet | awk '{print $2}' | sed -e 's/Adresse://' -e 's/addr://'", $serverAddr);
        $rpcap = $this->addField(new InputPageCheckboxField("rpcapval", "rpcap (rpcap://{$serverAddr[0]}/eth0)"));
        $hidden = $this->addField(new InputPageHiddenIdField("hiddenrpcap", "1"));

        $hiddenVal = InputPage::formInput($hidden->name);
        if ($mainSession->webCfg->rpcapOn && !isset($hiddenVal)) {
            $rpcap->default["on"] = true;
        }
        $btn = $this->addField(new ButtonRpcap("buttonrpcap", tl::tl("Ok")));
    }

    public function tabReset() {
        $this->currentSection = InnovaphoneDiagnosticsConsts::tabReset;
        $text = $this->addField(new InputPageText("text", ""));
        $text->autorender = false;
        $btnShutdown = $this->addField(new ButtonShutdown("btnshutdown", tl::tl("Ok"), false));
        $this->addField(new InputPageText("shutdown", "<a class=\"novisit\" href=\"javascript:clickButton('{$btnShutdown->name}');\">" . tl::tl("Shutdown") . "</a>"));
        if (runs_in_vm()) {
            $btnReboot = $this->addField(new ButtonShutdown("btnreboot", tl::tl("Ok"), true));
            $this->addField(new InputPageText("reboot", "<a class=\"novisit\" href=\"javascript:clickButton('{$btnReboot->name}');\">" . tl::tl("Reboot") . "</a>"));
        }
    }

    public function tabLogs() {
        global $mainSession;

        $this->currentSection = InnovaphoneDiagnosticsConsts::tabLogs;

        // error field
        $errorFld = $this->addField(new InputPageText("error", ""));
        $errorFld->fullline = true;
        $errorFld->autorender = false;

        $logFileName = "logfile";
        $btnViewName = "viewlog";
        $btnDownloadName = "downloadlog";
        $btnDownloadAllName = "downloadalllogs";
        $btnClearName = "clearlog";
        $btnRsyslogName = "reloadrsyslogbtn";
        // rendered fields
        if ($this->parent->app === null) {
            $this->addLog("backup", tl::tl("Backup"), Config::backupLog, $logFileName, $btnViewName, $btnDownloadName, $btnClearName);
            $this->addLog("application", tl::tl("Application install"), Config::applInstallLog, $logFileName, $btnViewName, $btnDownloadName, $btnClearName);
            $this->addLog("uninstall", tl::tl("Application uninstall"), Config::applUninstallLog, $logFileName, $btnViewName, $btnDownloadName, $btnClearName);
            $this->addLog("kernel", "Kernel", Config::kernelLog, $logFileName, $btnViewName, $btnDownloadName);
            $this->addLog("syslog", "Syslog", Config::syslogLog, $logFileName, $btnViewName, $btnDownloadName);
            $this->addLog("lighttpd", "Lighttpd", Config::lighttpdLog, $logFileName, $btnViewName, $btnDownloadName, $btnClearName);
            $this->addLog("initinstall", tl::tl("Initial install"), Config::mainInstallLog, $logFileName, $btnViewName, $btnDownloadName);
            $this->addLog("innovaphonelog", "innovaphone", Config::lighttpdConfLog, $logFileName, $btnViewName, $btnDownloadName, $btnClearName);
            $this->addLog("pganalyzelog", "PostgreSQL Analyze", Config::pgAnalyzeLog, $logFileName, $btnViewName, $btnDownloadName, $btnClearName);
            $this->addLog("pgsql", "Postgresql-9.1", Config::postgreSqlLog, $logFileName, $btnViewName, $btnDownloadName, "");
            $this->addLog("mail", "Mail", Config::mailLog, $logFileName, $btnViewName, $btnDownloadName, "");
            $ind = 0;
            $logFiles = array();
            $logFiles[] = Config::postgreSqlLog;
            foreach ($mainSession->apps as $app) {
                foreach ($app->logFiles as $log) {
                    if(array_search($log->file, $logFiles) === false) {
                        $this->addLog("applicationglog" . $ind, $log->title, $log->file, $logFileName, $btnViewName, $btnDownloadName, $log->noClear ? "" : $btnClearName);
                        $ind++;
                        $logFiles[] = $log->file;
                    }
                }
            }

            $this->addLog("debpackages", tl::tl("Installed debian packages"), Config::debInstalledPackages, $logFileName, $btnViewName, $btnDownloadName);
        } else {
            $ind = 0;
            foreach ($this->parent->app->logFiles as $log) {
                $this->addLog("applicationglog" . $ind, $log->title, $log->file, $logFileName, $btnViewName, $btnDownloadName, $log->noClear ? "" : $btnClearName);
                $ind++;
            }
        }
        $dummy = $this->addField(new InputPageText("dummy", "<br>"));
        $dummy->fullline = true;
        $btn = $this->addField(new InputPageText("downloadall", "<a href=\"#\" class=\"novisit\" onClick=\"javascript:clickButton('{$btnDownloadAllName}');\">" . tl::tl("Download all") . "</a>"));
        $btn->fullline = true;
        $btn = $this->addField(new InputPageText("reloadrsyslog", "<a href=\"#\" class=\"novisit\" onClick=\"javascript:clickButton('{$btnRsyslogName}');\">" . tl::tl("Reload rsyslog") . "</a>"));
        $btn->fullline = true;
        $btn->hint = tl::tl("Reload rsyslog, if you deleted/moved log files with webdav or putty etc.");

        // hidden fields
        $btnDownload = $this->addField(new ButtonDownloadLog($btnDownloadName, ""));
        $btnView = $this->addField(new ButtonViewLog($btnViewName, ""));
        $btnClear = $this->addField(new ButtonClearLog($btnClearName, ""));
        $btnDownloadAll = $this->addField(new ButtonDownloadAllLogs($btnDownloadAllName, ""));
        $btnRsyslog = $this->addField(new ButtonReloadRsyslog($btnRsyslogName, ""));
        $logFile = $this->addField(new InputPageHiddenIdField($logFileName));

        $this->addSection($this->currentSection . "_bottom", 1, false);
        $this->currentSection = $this->currentSection . "_bottom";
        $log = $this->addField(new InputPageText("log", ""));
        $log->tdattributes["nowrap"] = "nowrap";
    }
    
    public function addLog($type, $title, $file, $logFile, $viewButton, $downloadButton, $clearButton = "") {
        $prompt = $this->addField(new InputPageText($type, $title));
        $prompt->tdattributes["nowrap"] = "nowrap";
        $text = $this->addField(new InputPageText($type . "_text", ""));
        if(strpos($file, "*") === false) {
            $text->value .= "<a class=\"novisit\" href=\"#\" onClick=\"javascript:logFileAction('$file', '$logFile', '$viewButton');\">" . tl::tl("View") . "</a>";
            $text->value .= "&nbsp;&nbsp;&nbsp;";
        }
        $text->value .= "<a class=\"novisit\" href=\"#\" onClick=\"javascript:logFileAction('$file', '$logFile', '$downloadButton');\">" . tl::tl("Download") . "</a>";
        if ($clearButton != "") {
            $text->value .= "&nbsp;&nbsp;&nbsp;";
            $text->value .= "<a class=\"novisit\" href=\"#\" onClick=\"javascript:logFileAction('$file', '$logFile', '$clearButton');\">" . tl::tl("Clear") . "</a>";
        }
    }

    public function fillLists() {
        switch($this->parent->currentTab) {
            case InnovaphoneDiagnosticsConsts::tabDbExchange:
                $this->parent->fillListExchange();
                break;
            case InnovaphoneDiagnosticsConsts::tabFaxCalls:
                $this->parent->fillListFax();
                break;
        }
    }
}

final class InnovaphoneDiagnostics extends ParentClass {

    var $app = null;

    // constructor
    public function InnovaphoneDiagnostics() {
        parent::__construct(tl::tl("innovaphone Diagnostics"), "innovaphonediagnostics");
        $this->setFieldSectionAttributes("inline", new _InputPageFieldSectionAttributes(array(), 1));
        global $app;
        $this->app = $app;

        // add proper fields to pages
        $this->pages = array(InnovaphoneDiagnosticsConsts::pageDiagnostics => new PageDiagnostics($this, false));

        // update page
        $this->setDefaultTabProperties();
        $this->pages[$this->pageNo]->createFields();
    }

    public function getArgs() {
        return "tab={$this->currentTab}";
    }

}

// create page
$page = new InnovaphoneDiagnostics;

if ($page->pageNo == InnovaphoneDiagnosticsConsts::pageDiagnostics) {
    $page->updateFields();
    $page->pages[$page->pageNo]->fillLists();
}

$page->work();
?>
