<?php
ini_set("mbstring.func_overload", "7");

require_once("config.php");
require_once("classes/innoinputpage.class.php");
require_once("classes/translator.class.php");
require_once("classes/html.class.php");
html::$encoding = "UTF-8";
//require_once("classes/htmlemail.class.php");
require_once("classes/mysession.class.php");
require_once("classes/rc4.php");
require_once("classes/mylogin.class.php");
//phpinfo();

/*// check for SSL
// do not redirect localhost and direct php (server="") calls
if($_SERVER['HTTPS'] != "on" && $_SERVER["SERVER_NAME"] != "localhost" && $_SERVER["SERVER_NAME"] != "127.0.0.1" && $_SERVER["SERVER_NAME"] != "") {
    header("location: https://".$_SERVER["HTTP_HOST"].$_SERVER["REQUEST_URI"]);
    exit;
}
}*/
//phpinfo();

// redirect to installation.php while first time installation is still running
if(file_exists(Config::installRunning) && strpos($_SERVER["SCRIPT_NAME"], "installation.php") === false) {
    header("location: installation.php");
    exit;
}

function htmlentities_utf8($text) {
    return htmlentities($text, ENT_COMPAT, "UTF-8");
}

function runs_in_vm($init = false) {
    global $mainSession;
    if($init || $mainSession == null) {
        $str = exec("/bin/uname -m | egrep -oe 'i[0-9]{1}86'", $t, $ret);
        if($ret == 0 && strlen($str) > 0) {
            return true;
        }
        return false;
    }
    return $mainSession->isVm;
}

abstract class MainInnovaphonePage extends InnoInputPage {
    // the current page #
    var $pageNo = 1;
    var $maxPage = 0;
    var $pages = null; 
    var $currentTab;
    var $currentMode;
    
    // controls whether the translation dropdown box will be visible or not, each application can set this global, if translations are provided
    // for each language, a translation file must exist, e.g. /var/www/innovaphone/lang/lang-de.inc, even if this file is empty
    var $enableTranslations = false;
    
    public function MainInnovaphonePage($site, $name, $printMenu = true) {
        parent::__construct($site);
        $this->metas["content-type"] = "text/html; charset=UTF-8";
        $this->name = $name;
        $this->printAdditionalLinks = false;
        $this->baseURI = "";
        $this->logoImg = "/images/logo_linux.png";
        unset($this->fieldSectionAttributes["inline"]->attributes["align"]);
        $this->sectionInlineTdAttributes["width"] = "100%";
        $this->sectionInlineTdAttributes["valign"] = "top";
        $this->sectionInlineTdAttributes["align"] = "left";
        $this->sectionLeftTdAttributes["valign"] = "top";
        
        $this->scriptfiles += array("java_main" => "/java_main.js");
        
        // dummy fields to save certain parameters over all pages
        // add hidden field which holds current tab section
        $this->addField(new InputPageHiddenIdField("tab"));
        $this->currentTab = InputPage::formInput("tab");
        // add hidden field which holds current mode used for sub section stuff
        $this->addField(new InputPageHiddenIdField("mode"));
        $this->currentMode = InputPage::formInput("mode");
        
        // print header
        $this->printHead();
        if($printMenu) {
            // add menu
            $this->addMenu();
        }
    }
    
    // call this function first when pages are instanciated
    function setDefaultTabProperties() {
        $this->currentTab != "" ? $this->currentTab : $this->currentTab = $this->pages[$this->pageNo]->getDefaultTab();
        $ls = new InputPageLanguageSelectorDropdownField("lang");
        // only use languages, which are translated for all stuff and apps
        foreach($ls->options as $lang => $name) {
            switch($lang) {
                case "de": break;
                case "it": break;
                case "en": break;
                case "pl": break;
                case "fr": break;
                case "ru": break;
                case "cs": break;
                default: unset($ls->options[$lang]);
            }
        }
        $ls->attributes["onchange"] = "document.getElementsByName('{$this->name}')[0].submit()";
        global $l;
        $ls->value[$l] = true;
        $this->loginArea = "<table>";
        $this->loginArea .= "    <tr>";
        $this->loginArea .= "        <td valign=\"top\">".($this->enableTranslations ? $ls->render_input() : "")."</td>";
        $td = $this->getLoginAreaTd();
        if($td != "") {
            $this->loginArea .= "<td>$td</td>";
        }
        $this->loginArea .= "        <td valign=\"top\" style=\"padding-left:20px\"><a href=\"".$this->getHelpLink()."\" target=\"_blank\">".tl::tl("Help")."</a></td>";
        $this->loginArea .= "    </tr>";
        $this->loginArea .= "</table>";
    }
    
    function getLoginAreaTd() {
        return "";
    }
    
    function getHelpLink() {
        return Config::wikiHelp.$this->pages[$this->pageNo]->getHelpAnchor();
    }
    
    function getUnCheckAllInput($chkName, $checked = true, $java = "") {
        return "<input type=\"checkbox\" name=\"{$chkName}_all\" ".($checked ? "checked=\"checked\"" : "")." onClick=\"javascript:toggleCheck('{$chkName}');$java\" title=\"".tl::tl("Un-/Check all")."\"/>";
    }
    
    function addMenu() {
    }
    
    function printHead() {
        // language selector field
        /*$ls = new InputPageLanguageSelectorDropdownField("lang");
        $ls->attributes["onchange"] = "document.getElementsByName('{$this->name}')[0].submit()";
        global $l;
        $ls->value[$l] = true;
        $ls->render_input();*/
    }
    
    public function renderPrefix() {
        $ret = "";
        $ret .= parent::renderProlog();
        $ret .= parent::renderPrefix();
        return $ret;
    }
    
    public function renderProlog() {
        return '';
    }
    
    public function renderSuffix() {
        $ret = "";
        $ret .= parent::renderEpilog();
        $ret .= parent::renderSuffix();
        return $ret;
    }
    
    public function renderEpilog() {
        return '';
    }
    
    function getArgs() {
        return "";
    }
}

class CronInputPageDigitStringField extends InputPageDigitstringField {
    const minTime = 1;
    const maxTime = 60;
    public function CronInputPageDigitStringField($name, $prompt = "", $default = "", InputPageAction $button = null) {
        parent::__construct($name, $prompt, $default, $button);
        $this->placeHolder = tl::tl("e.g.")." 12";
    }
    
    public function valid() {
        if(!parent::valid()) return false;
        if((int)$this->value < self::minTime) {
            $this->msg = tl::tlx('Set at least {$arg[1]} minutes.', self::minTime);
            return false;
        }
        elseif((int)$this->value > self::maxTime) {
            $this->msg = tl::tlx('Set max {$arg[1]} minutes.', self::maxTime);
            return false;
        }
        return true;
    }
}

abstract class MainInputPage extends MainInnovaphonePage {
    var $version;
    var $hotfix;
    const dateFormat = "Y-m-d H:i:s";
    
    public function MainInputPage($site, $name, $printMenu = true) {
        $this->enableTranslations = true;
        parent::MainInnovaphonePage($site, $name, $printMenu);
        $this->searchArea = "<br><br><b>".self::getVersions(Config::webRoot, $this->version, $this->hotfix)."</b>";
        $dateField = new InputPageDateField("123", "", time(), null, true);
        $this->infoArea = $dateField->makeDisplayString();
    }
    
    static function getVersions($path, &$version, &$hotfix) {
        $version = 0;
        $hotfix = 0;
        if(file_exists("$path/version")) {
            $version = trim(str_replace(array("\r", "\n"), array("", ""), file_get_contents("$path/version")), " ");
        }
        if(file_exists("$path/hotfix")) {
            $hotfix = trim(str_replace(array("\r", "\n"), array("", ""), file_get_contents("$path/hotfix")), " ");
        }
        $beta = file_exists("$path/beta");
        $dvl = file_exists("$path/dvl");
        return self::getVersionStr($version, $hotfix, $beta, $dvl);
    }
    
    static function getVersionStr($version, $hotfix, $beta = false, $dvl = false) {
        $v = trim(file_get_contents(Config::majorVersion));
        $str = "V$v";
        if($beta) {
            $str .= " RC1";
        }
        else if($hotfix > 0 && $hotfix != "") {
            $str .= " SR{$hotfix}";
        }
        else {
            $str .= " Final";
        }
        $str .= " [$version]";
        if($dvl) {
            $str .= " (DVL)";
        }
        return $str;
    }
    
    function addMenu() {
        if($this->name == "innovaphoneinstallation") {
            $this->addMainMenuItem(tl::tl("Installation"), "installation.php", $this->name == "innovaphoneinstallation");
        }
        else {
            $this->addMainMenuItem(tl::tl("Applications"), "application.php", $this->name == "innovaphoneapplication");
            $this->addMainMenuItem(tl::tl("Administration"), "administration.php", $this->name == "innovaphoneadmin");
            $this->addMainMenuItem(tl::tl("Diagnostics"), "diagnostics.php", $this->name == "innovaphonediagnostics");
        }
    }
}

// helper classes to define individual pages in an InnoInputPage
abstract class MyPage {
    // parent innoInputPage
    var $parent;
    var $pageNo;
    var $myFields = array();
    var $site;
    var $currentSection = "";
    
    // constructor
    public function MyPage(innoInputPage &$parent, $create = true, $site = "") {
        $this->parent = &$parent;
        $this->site = $site;
        $this->pageNo = ++$parent->maxPage;
        if ($this->pageNo > $this->parent->maxPage) $this->parent->maxPage = $this->pageNo;
        if ($create) $this->createFields();
    }
    
    // add field if proper page, hide otherwise
    public function addField(&$field) {
        if($this->currentSection != "") {
            $field->style = $this->currentSection;
            if(isset($field->buttonCancel)) {
                $field->buttonCancel->style = $this->currentSection;
            }
        }
        $newfield = &$this->parent->addField($field);
        $this->myFields[] = &$newfield;
        if ($this->pageNo != $this->parent->pageNo) $field->hidden = true;
        return $newfield;
    }
    // create page fields
    abstract public function createFields();
    
    // function which need to be overridden if tab pages are used
    public function getHead() { return "";}
    public function getDefaultTab() { return "";}
    public function getHelpAnchor() {
        return "";
    }
    
    public function updateTitle() {
        if($this->site != "") {
            $this->parent->site = $this->site;
            $this->parent->title = $this->site;
        }
    }
    
    function addSection($name, $columns = 0, $fieldSet = true, $legend = "", $javaHide = false, $divEnclosed = false) {
        $this->parent->setFieldSectionAttributes($name, new _InputPageFieldSectionAttributes(array(), $columns != 0 ? $columns : $this->parent->fieldSectionAttributes["inline"]->columns, 4, $fieldSet, $legend, $javaHide, $divEnclosed));
    }
    
    function hideSection($name) {
        foreach($this->parent->fields as $field) {
            if($field->style == $name) {
                $field->hidden = true;
            }
        }
        // hide all sub sections too
        foreach($this->parent->subSections as $parent => $sections) {
            if($parent == $name) {
                foreach($sections as $section => $value) {
                    if($section != $parent) {
                        foreach($this->parent->fields as $field) {
                            if($field->style == $section) {
                                $field->hidden = true;
                            }
                        }
                    }
                }
                break;
            }
        }
    }
    
    function getModeLink($mode, $text, $title = "", $href = "") {
        if($href == "") {
            $href = "javascript:submitMode('$mode')";
        }
        return "<img src=\"/images/link_arrow.png\" height=\"9\"/> <a class=\"inno-tab-table-content\" href=\"$href\">$text</a>";
    }

    function getModeLinkOnly($mode, $text, $title = "", $href = "") {
        if($href == "") {
            $href = "javascript:submitMode('$mode')";
        }
        return " <a class=\"inno-tab-table-content\" href=\"$href\">$text</a>";
    }
    
    function addSubSection($parent, $postFix, $columns = 1, $javaHide = false, $divEnclosed = false) {
        $this->currentSection = $parent.$postFix;
        $this->addSection($this->currentSection, $columns, false, "", $javaHide, $divEnclosed);
        $this->parent->addSubSection($this->currentSection, $parent);
    }
}

class MainInputPageListField extends InputPageListField {
    public function MainInputPageListField($name, $rows = array()) {
        $this->InputPageListField($name, $rows);
        $this->sortDirection = InputPageListField::sortAscending;
        $this->hasColumnHeaderOnEmptyTable = false;
        $this->repeatHeader = 9999;
        $this->headAttributes["class"] = "headlist-separator";
        $this->oddAttributes["class"] = "listfield-tr-separator";
        $this->evenAttributes["class"] = "listfield-tr-separator";
        $this->colHeadClass = "myinno-columnhead";
        $this->colSortHeadClass = "myinno-sortcolumnhead";
        $this->colNoLinkHeadClass = "columnnolinkhead";
        $this->fullline = true;
    }
    
    public function addColumn($field, $header = null) {
        $field->attributes["valign"] = "top";
        $field->tdattributes["valign"] = "top";
        unset($field->columnsToRender["prompt"]);
        $field->tdattributes["nowrap"] = "nowrap";
        return parent::addColumn($field, $header);
    }
}

abstract class ButtonHidden extends InputPageAction {
    public function ButtonHidden($name, $label) {
        parent::__construct($name, $label);
        $this->anyway = true;
        $this->autorender = true;
        $this->attributes["style"] = "display:none";
        $this->formNoValidate = true;
    }
}

class ButtonCancel extends InputPageButtonOnlyAction {
    var $pageNo;
    public function ButtonCancel ($name, $label, $pageNo) {
        parent::__construct($name, $label);
        $this->autorender = false;
        $this->anyway = true;
        $this->pageNo = $pageNo;
        $this->tdattributes["style"] = "padding-top: 10px;padding-bottom: 10px";
        $this->formNoValidate = true;
    }
    
    function action(InputPage &$page, $phase) {
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page={$this->pageNo}&".$page->getArgs());
        exit;
    }
}

class Certificate {
    var $file;
    var $subject;
    var $issuer;
    var $notBefore;
    var $notAfter;
    var $pathLen;
    var $ca;
    var $purposes;
    var $serial;
    var $bits;
    var $subjectKeyId;
    var $authKeyId;
    var $version;
    
    public function Certificate() {
        $this->file = "";
        $this->subject = array();
        $this->issuer = array();
        $this->notBefore = "";
        $this->notAfter = "";
        $this->pathLen = "";
        $this->ca = false;
        $this->purposes = "";
        $this->serial = "";
        $this->bits = 0;
        $this->subjectKeyId = "";
        $this->authKeyId = "";
        $this->version = 0;
    }
    
    public function getSubject() {
        return $this->getDesc($this->subject);
    }
    
    public function getIssuer() {
        return $this->getDesc($this->issuer);
    }
    
    private function getDesc($desc) {
        if(isset($desc["CN"]) && $desc["CN"] != "") return $desc["CN"];
        elseif($desc["OU"] != "" || is_array($desc["OU"])) {
            if(is_array($desc["OU"])) {
                return $desc["OU"][0];
            }
            else {
                return $desc["OU"];
            }
        }
        else return $desc["O"];
    }
}

class Certificates {
    static function getCertificates($path) {
        $files = get_files_from_dir($path);
        $certs = array();
        foreach($files as $file) {
            if(ends_with($file, ".pem")) {
                $cmd = "/usr/bin/sudo /usr/local/bin/config.sh ssl read ".$path."/".$file;
                $cert = MainMethods::callConfigSh($cmd, $retval);
                if($retval != 0) {
                    // return error!
                } else {
                    $certs[] = self::getCertificate($cert, $path."/".$file);
                }
            }
        }
        return $certs;
    }
    
    static function getCertificate($cert, $file = "") {
        $x509 = openssl_x509_parse($cert);
        $publicRessource = openssl_pkey_get_public($cert);
        $publicVals = openssl_pkey_get_details($publicRessource);
        $c = new Certificate();
        $c->file = $file;
        $c->serial = $x509["serialNumber"];
        $c->version = $x509["version"] + 1;
        $c->notBefore = $x509["validFrom_time_t"];
        $c->notAfter = $x509["validTo_time_t"];
        /*date_default_timezone_set("UTC");
        echo date("Y-m-d H:i:s", $c->notAfter)."<br>";
        echo date("Y-m-d H:i:s", $c->notBefore)."<br>";*/        
        $c->subject = $x509["subject"];
        $c->issuer = $x509["issuer"];
        if(is_array($x509["purposes"])) {
            foreach($x509["purposes"] as $purpose) {
                if(($purpose[0] === true || $purpose[1] === true) && $purpose[2] != "" && $purpose[2] != "any") {
                    $c->purposes .= "{$purpose[2]} ";
                }
            }
        }
        $c->subjectKeyId = $x509["extensions"]["subjectKeyIdentifier"];
        $c->authKeyId = substr($x509["extensions"]["authorityKeyIdentifier"], 6, 59);
        $c->bits = $publicVals["bits"];
        if(strpos($x509["extensions"]["basicConstraints"], "CA:TRUE") !== false) {
            $c->ca = true;
        }
        /*var_dump($x509);
        var_dump($publicVals);
        var_dump($c);*/
        return $c;
    }
    
    static function getServerCertificate($url, &$error) {
        $urlParts = parse_url($url);
        if(!isset($urlParts["port"])) {
            $urlParts["port"] = 443;
        }
        $server = $urlParts["host"].":".$urlParts["port"];
        $openSSL = "/usr/bin/openssl s_client -CApath /etc/ssl/certs/ -showcerts -rand /dev/urandom -connect $server < ".Config::quitFile." 2>&1";
        $response = $orgResponse = main_system($openSSL, $ret);
        $firstFound = false;
        $i = 0;
        $certsFound = array();
        do
        {
            $invalid = false;
            $response = strstr($response, "-----BEGIN CERTIFICATE-----");
            if($response === false) {
                if(!$firstFound) {
                    $error = "no certificate for server '$server' found ($ret): $orgResponse";
                    return false;
                }
                else {
                    $invalid = true;
                }
            }
            $end = "-----END CERTIFICATE-----";
            $endPos = strpos($response, $end);
            if($endPos === false) {
                if(!$firstFound) {
                    $error = "no certificate end for '$server' found ($ret): $orgResponse";
                    return false;
                }
                else {
                    $invalid = true;
                }
            }
            if($invalid) {
                break;
            }
            $firstFound = true;
            $cert = substr($response, 0, $endPos + strlen($end));
            $response = substr($response, $endPos + strlen($end));
            if(!file_exists(Config::certDirRejected)) {
                exec("/bin/mkdir -p \"".Config::certDirRejected."\"");
            }
            $c = self::getCertificate($cert);
            $certName = str_replace(" ", "", $c->getSubject())."_".$urlParts["host"]."_".$urlParts["port"];
            if(isset($certsFound[$certName])) {
                $i++;
                $certName .= "_$i";
            }
            $certsFound[$certName] = true;
            $certName .= ".pem";
            if(!file_exists(Config::certDirTrusted."/".$certName)) {
                file_put_contents(Config::certDirRejected."/".$certName, $cert);
            }
        } while ($response !== false);
        return true;
    }
}

class MainBackup {
    const saveInnovaphoneCfgs = "saveinnovaphonecfgs";
    const saveInnovaphoneLogs = "saveinnovaphonelogs";
    const saveInnovaphoneDb = "saveinnovaphonedb";
    const restoreInnovaphoneCfgs = "restoreinnovaphonecfgs";
    const restoreReportingDb  = "restoreinnovaphone-reportingdb";
    const restoreExchangeDb  = "restoreinnovaphone-exchangedb";
    const restoreFaxserverDb  = "restoreinnovaphone-faxserverdb";
    const restoreInnovaphoneDb  = "restoreinnovaphone-db";
    const cmdTimes = "times";
    
    function MainBackup() {
        
    }
    
    function applyMacros(&$url, $cmd, $app = "") {
        // Replace IP address
        if(strpos($url, "#i") !== false) {
            ob_start();
            passthru("/sbin/ifconfig eth0 | grep inet | awk '{print $2}' | sed -e 's/Adresse://' -e 's/addr://'");
            $ip = ob_get_contents();
            $ip = str_replace("\n", "", $ip);
            ob_end_clean();
            $url = str_replace("#i", $ip, $url);
        }
        // Replace MAC address
        if(strpos($url, "#m") !== false) {
            ob_start();
            passthru('/sbin/ifconfig eth0 | grep eth0');
            $dump = ob_get_contents();
            preg_match('/[A-F0-9]{2}:[A-F0-9]{2}:[A-F0-9]{2}:[A-F0-9]{2}:[A-F0-9]{2}:[A-F0-9]{2}/i', $dump, $mac);
            $macAddress = str_replace(":", "-", $mac[0]);
            ob_end_clean();
            $url = str_replace("#m", $macAddress, $url);
        }
        // Replace date/time
        $url = str_replace("#d", date("Ymd-His"), $url);
        // Index File
        if($app) {
            $fileTmp = tempnam(Config::webRoot."/".Config::applicationsDir."/".$app."/tmp", "INDEX");
            $file = Config::webRoot."/".Config::applicationsDir."/".$app."/tmp/backup_serials/".$cmd;
        } else {
            $fileTmp = tempnam(Config::webRoot."/tmp", "INDEX");
            $file = Config::webRoot."/tmp/backup_serials/".$cmd;
        }
        // Replace current index
        if(preg_match('/#b[0-9]+/', $url, $match)) {
            $currentIndex = -1;
            if(file_exists($file)) {
                $currentIndex = (int)file_get_contents($file);
            }
            $maxIndex = (int)substr($match[0], 2);
            if(++$currentIndex >= $maxIndex) {
                $currentIndex = 0;
            }
            $url = str_replace($match[0], $currentIndex, $url);
            file_put_contents($fileTmp, "$currentIndex"."\n");
        } else
            file_put_contents($fileTmp, "\n");
        file_put_contents($fileTmp, date(MainInputPage::dateFormat)."\n", FILE_APPEND);

        $url = rtrim($url);

        return $fileTmp; 
    }

    function grabHeaderVar($var, $header) {
        preg_match("/$var=\"([^\"]*)\"|$var=([^,]*)/", $header, $matches);
        if(isset($matches[2])) return trim($matches[2], " ");
        if(isset($matches[1])) return $matches[1];
        return "";
    }
    
    function curlCmd($userCmd, $file, $auth, $ca, $header, &$ret) {
        $curlCmd = "/usr/bin/curl -f -i -s -S --url $file -T - -H \"Expect: \" --anyauth --location-trusted --max-redirs 4 -E \"".Config::certServer."\" $auth $ca $header";
        return main_system("$userCmd | $curlCmd 2>&1", $ret);
    }

    function curlAuth($userCmd, $file, $auth, $user, $pw, $ca, $resCheck, &$ret) {
        $header = "";
        if($auth != "") {
            if($ca == "-k") {
                $resCheck = $this->curlCmd("echo \"1\"", $file, $auth, $ca, "", $ret);
            }
            $lines = explode("\n", str_replace("\r\n", "\n", $resCheck));
            foreach($lines as $line) {
                if(strpos($line, "WWW-Authenticate: Digest") === 0) {
                    $qop = $this->grabHeaderVar("qop", $line);
                    $nonce = $this->grabHeaderVar("nonce", $line);
                    $realm = $this->grabHeaderVar("realm", $line);
                    $opaque = $this->grabHeaderVar("opaque", $line);
                    $method = "PUT";
                    $cnonce = mt_rand(123400, 9999999);
                    $nc = "00000001";
                    $parts = parse_url($file);
                    $uri = $parts["path"];
                    if($parts["query"] != "") $uri .= "?".$parts["query"];
                    if($parts["fragment"] != "") $uri .= "#".$parts["fragment"];
                    $a1 = "$user:$realm:$pw";
                    $a1_hashed = md5($a1);
                    $a2 = "$method:$uri";
                    $a2_hashed = md5($a2);
                    $response_unhashed = "";
                    if ($qop == "auth")
                        $response_unhashed =  "$a1_hashed:$nonce:$nc:$cnonce:$qop:$a2_hashed";
                    else
                        $response_unhashed = "$a1_hashed:$nonce:$a2_hashed";
                    $response = md5($response_unhashed);
                    $header = "-H \"Authorization: Digest username=\\\"$user\\\",realm=\\\"$realm\\\",nonce=\\\"$nonce\\\",uri=\\\"$uri\\\",algorithm=\\\"MD5\\\",response=\\\"$response\\\"";
                    if($qop == "auth") {
                        $header .= ",cnonce=\\\"$cnonce\\\",nc=$nc,qop=\\\"$qop\\\"";
                    }
                    if($opaque != "") $header .= ",opaque=\\\"$opaque\\\"";
                    $header .= "\"";
                    break;
                }
                else if(strpos($line, "WWW-Authenticate: Basic") === 0) {
                    $header = "-H \"Authorization: Basic ".base64_encode("$user:$pw")."\"";
                }
            }
        }
        //echo "auth: $auth; header: $header<br><br>";
        return $this->curlCmd($userCmd, $file, $auth, $ca, $header, $ret);
    }

    function saveBackupApp($file, $userCmd, $httpMethod, &$error) {
        global $mainSession;
        if(!strcmp($httpMethod, "post")) {
            $auth = "";
            $user = "";
            $pw = "";
            if(($authUrl = $mainSession->webCfg->getAuth($file)) != null) {
                $user = $authUrl->user;
                $pw = $authUrl->pw;
                $auth ="-u \"$user:$pw\"";
            }
            // Test attemp to access the server
            $resCheck = $this->curlCmd("echo \"1\"", $file, $auth, "--capath ".Config::certDirTrusted, "", $retval);
            if($retval == 60) {
                if(Certificates::getServerCertificate($file, $error) === false) {
                    return false;
                }
                else {
                    $error = "server certificate rejected, look at your SSL reject list";
                    return false;
                }
            } elseif($retval == 51) {
                $resCheck = $this->curlAuth($userCmd, $file, $auth, $user, $pw, "-k", $resCheck, $retval);
            } else {
                $resCheck = $this->curlAuth($userCmd, $file, $auth, $user, $pw, "--capath ".Config::certDirTrusted, $resCheck, $retval);
            }
            if($retval != 0) {
                $error = "$resCheck ($retval)";
                return false;
            } else {
                self::doLog("Backup successfully uploaded to $file");
            }
        } else {
            $this->echoFile($file, $userCmd);
        }
        return true;
    }
    
    function saveCfgs($file, $httpMethod, &$error) {
        $fileIndexBackup = $this->applyMacros($file, self::saveInnovaphoneCfgs);
        $cmd = "/usr/bin/sudo /usr/local/bin/config.sh innovaphone cfg-backup \"".Config::serverCfgFile."\" \"".Config::lighttpdCfgFile."\" \"".Config::certServer."\" \"".Config::cronInnovaphoneBackup."\" \"".Config::ntpConf."\" \"".Config::pgHba."\" \"".Config::pgConf."\" \"".Config::certServerKey."\" \"".Config::certServerCert."\"";
        if(file_exists(Config::certDirTrusted)) {
            $cmd .= " \"".Config::certDirTrusted."\"";
        }
        if($this->saveBackupApp($file, $cmd, $httpMethod, $error) === true) {
            copy($fileIndexBackup, Config::webRoot."/tmp/backup_serials/".self::saveInnovaphoneCfgs);
            unlink($fileIndexBackup);
            return true;
        }
        unlink($fileIndexBackup);
        return false;
    }
    
    function saveLogs($file, $httpMethod, &$error, $app = null, $logs = null) {
        global $mainSession;

        $fileIndexBackup = $this->applyMacros($file, self::saveInnovaphoneLogs);
        $cmd="/usr/bin/sudo /usr/local/bin/config.sh innovaphone cfg-backup ";
        if($app === null && $logs == null) {
            $cmdDeb = "/usr/bin/sudo /usr/local/bin/config.sh debian-packages download \"".Config::debInstalledPackages."\"";
            exec($cmdDeb);        
            $cmd .= " \"".Config::backupLog."*\"";
            $cmd .= " \"".Config::debInstalledPackages."*\"";
            $cmd .= " \"".Config::applInstallLog."*\"";
            $cmd .= " \"".Config::kernelLog."*\"";
            $cmd .= " \"".Config::aptHistoryLog."*\"";
            $cmd .= " \"".Config::syslogLog."*\"";
            $cmd .= " \"".Config::lighttpdLog."*\"";
            $cmd .= " \"".Config::pgAnalyzeLog."*\"";
            $cmd .= " \"".Config::postgreSqlLog."*\"";
            $cmd .= " \"".Config::lighttpdConfLog."*\"";
            $cmd .= " \"".Config::mailAllLog."*\"";
            $cmd .= " \"".Config::mainInstallLog."\"";
            $cmd .= " \"".Config::webRoot."/version\"";
            $cmd .= " \"".Config::webRoot."/hotfix\"";
            foreach($mainSession->apps as $app)
            {
                $cmd .= " \"".Config::webRoot."/".Config::applicationsDir."/".$app->file."\"";
                foreach($app->logFiles as $log) {
                    if(strpos($cmd, " \"".$log->file) === false) {
                        $cmd .= " \"".$log->file."*\"";
                    }
                }
            }
        }
        else if ($app != null) {
            $cmd .= " \"".Config::webRoot."/".Config::applicationsDir."/".$app->file."\"";
            foreach($app->logFiles as $log) {
                $cmd .= " \"".$log->file."*\"";
            }
        }
        else if($logs != null) {
            $cmd .= " \"".$logs."\"";
        }
        if($this->saveBackupApp($file, $cmd, $httpMethod, $error) === true) {
            copy($fileIndexBackup, Config::webRoot."/tmp/backup_serials/".self::saveInnovaphoneLogs);
            unlink($fileIndexBackup);
            return true;
        }
        unlink($fileIndexBackup);
        return false;
    }
    

    function saveDefaultCmd($file, $saveCmd, $httpMethod, &$error, $script = "") {
        global $mainSession;
        
        if(preg_match('/save(.*)cfgs/', $saveCmd, $matches) == 1) {
            $app = $mainSession->getApp($matches[1]);
            if(isset($app) && count($app->configFiles)) {
                if($file == "") {
                    $file = $matches[1]."_cfgs_#i.tar.gz";
                }
                $fileIndexBackup = $this->applyMacros($file, $saveCmd, $matches[1]);
                $cmd = "/usr/bin/sudo /usr/local/bin/config.sh application cfg-backup {$app->name} \"".$app->configFiles[0]."\"";
                for($i=1;$i<count($app->configFiles);$i++) {
                    $cmd .= " \"".$app->configFiles[$i]."\"";
                }
                if($this->saveBackupApp($file, $cmd, $httpMethod, $error) === true) {
                    copy($fileIndexBackup, Config::webRoot."/".Config::applicationsDir."/".$matches[1]."/tmp/backup_serials/".$saveCmd);
                    unlink($fileIndexBackup);
                    return true;
                }                
                unlink($fileIndexBackup);
                return false;
            } else {
                $error = tl::tlx('No configuration files defined in {$arg[1]}', $matches[1].".xml");
                return false;
            }
        }
        else if(preg_match('/save(.*)logs/', $saveCmd, $matches)) {
            $app = $mainSession->getApp($matches[1]);
            if(isset($app) && count($app->logFiles)) {
                if($file == "") {
                    $file=$matches[1]."_logs_#i.tar.gz";
                }
                $fileIndexBackup = $this->applyMacros($file, $saveCmd, $matches[1]);
                $cmd = "/usr/bin/sudo /usr/local/bin/config.sh application cfg-backup {$app->name} \"".$app->logFiles[0]->file."\"";
                for($i=1;$i<count($app->logFiles);$i++) {
                    $cmd .= " \"".$app->logFiles[$i]->file."\"";
                }
                if($this->saveBackupApp($file, $cmd, $httpMethod, $error) === true) {
                    copy($fileIndexBackup, Config::webRoot."/".Config::applicationsDir."/".$matches[1]."/tmp/backup_serials/".$saveCmd);
                    unlink($fileIndexBackup);
                    return true;
                }
                unlink($fileIndexBackup);
                return false; 
            } else {
                $error = tl::tlx('No log files defined in {$arg[1]}', $matches[1].".xml");
                return false;
            }  
        }
        else if(preg_match('/save(.*)db/', $saveCmd, $matches)) {
            if($matches[1]) {
                $app = $mainSession->getApp($matches[1]);
                $appName = null;
                $appDbName = null;
                if($saveCmd == MainBackup::saveInnovaphoneDb) {
                    $appName = $appDbName = "innovaphone";
                }
                elseif(isset($app) && $app->db->name != "") {
                    $appName = $app->name;
                    $appDbName = $app->db->name;
                }
                if(isset($appName)) {
                    if($file == "") {
                        $file=$appName."_#i.gz";
                    }
                    $fileIndexBackup = $this->applyMacros($file, $saveCmd, $appName);
                    $cmd = "/usr/bin/sudo /usr/local/bin/config.sh postgres db-backup \"{$appDbName}\"";
                    if($this->saveBackupApp($file, $cmd, $httpMethod, $error) === true) {
                        copy($fileIndexBackup, Config::webRoot."/".Config::applicationsDir."/".$appName."/tmp/backup_serials/".$saveCmd);
                        unlink($fileIndexBackup);
                        return true;
                    }
                    unlink($fileIndexBackup);
                    return false;                     
                }
                else {
                    $error = tl::tlx('No database for application {$arg[1]} found.', $matches[1]);
                    return false;
                }
            }
        }
        else {
            foreach($mainSession->apps as $app) {
                foreach($app->backupCmds as $cmd) {
                    if($cmd->cmd == $saveCmd && $cmd->script != "") {
                        if($file == "") {
                            $file = $cmd->cmd."_#i.tar.gz";
                        }
                        $fileIndexBackup = $this->applyMacros($file, $saveCmd, $app->name);
                        $userCmd = "/usr/bin/sudo /usr/local/bin/config.sh application script {$app->name} ".$cmd->script;
                        if($this->saveBackupApp($file, $userCmd, $httpMethod, $error) === true) {
                            copy($fileIndexBackup, Config::webRoot."/".Config::applicationsDir."/".$app->name."/tmp/backup_serials/".$saveCmd);
                            unlink($fileIndexBackup);
                            return true;
                        }
                        unlink($fileIndexBackup);
                        return false;    
                    }
                }
            }
        }
        $error = tl::tlx('No custom script for {$arg[1]} defined', $saveCmd);
        return false;
    }

    function echoFile($file, $cmd) {
        $descriptors = array(
            0 => array("pipe", "rb"),
            1 => array("pipe", "wb")
            );
        header('Pragma: private');
        header('Cache-control: private, must-revalidate');
        header("Content-Disposition: attachment; filename=\"$file\"");
        header('Content-type: application/gz');
        $process = proc_open($cmd, $descriptors, $pipes);
        if (is_resource($process)) {
            while (!feof($pipes[1])) {
                echo fgets($pipes[1]);
            }
        }
    }

    static function doLog($text) {
        echo "LOG: ".self::getDate()." - $text\n";
    }

    static function error($text, $httpMethod = "", $fileToDelete = "", $dirToDelete = "") {
        if($fileToDelete != "") unlink($fileToDelete);
        if($dirToDelete != "") exec("/bin/rm -R \"$dirToDelete\"");
        if(!strcmp($httpMethod,"post")) echo "ERROR: ".self::getDate()." - $text\n";
        return false;
    }

    static function getDate() {
        return date(MainInputPage::dateFormat);
    }
}

class MainInputPageWebPathField extends InputPageStringField {
    var $checkWebdav;
    
    public function MainInputPageWebPathField($name, $prompt = "", $default = "", InputPageAction $button = null, $checkWebdav=false) {
        parent::__construct($name, $prompt, $default, $button);
        $this->checkWebdav = $checkWebdav;
        unset($this->attributes["maxlength"]);
    }
    
    public function valid() {
        if(!parent::valid()) return false;
        if(strpos($this->value, '"') !== false) {
            $this->msg = tl::tlx('The character {$arg[1]} can\'t be used in an URL.', '"');
            return false;
        }
        else if(strpos($this->value, "\\") !== false) {
            $this->msg = tl::tlx('The character {$arg[1]} can\'t be used in an URL.', '\\');
            return false;
        }
        if($this->checkWebdav === true && strpos(trim($this->value, "/\\"), "webdav") === 0) {
            $this->msg = tl::tl("The webdav directory mustn't be used as a public web path.");
            return false;
        }
        return true;
    }
}

abstract class MainInputPageOkCancelButton extends InputPageButtonOnlyAction {
    /**
     *
     * @var ButtonCancel the cancel button
     */
    var $buttonCancel;
    /**
     *
     * @var string the next mode
     */
    var $nextMode;
    /**
     *
     * @var string the prev mode
     */
    var $prevMode;
    /**
     *
     * @var InputPageAction an additional button, which is rendered at last
     */
    var $addButton;
    /**
     *
     * @var MainInputPageOkCancelButton the previous button, if the same button as $this should be used as previous button
     */
    var $prevButton = null;
    const prevPrefix = "_prev_";
    
    /**
     *
     * @param string $name
     * @param string $pageNo
     * @param InputPage $page
     * @param string $nextMode
     * @param string $prevMode
     * @param InputPageAction $addButton a further button which should be rendered
     * @param boolean $prevButtonSubmits set true if the same button should be used as previous button (you have to distinguish the clicked button inside your action with $this->prevButton)
     */
    function MainInputPageOkCancelButton($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null, $addButton = null, $prevButtonSubmits = false) {
        parent::__construct($name, $nextMode == null ? tl::tl("Ok") : tl::tl("Next"));
        $this->columnsToRender = array("input" => 1, "msg" => 2);
        $this->nextMode = $nextMode;
        $this->prevMode = $prevMode;
        if($prevButtonSubmits && !ends_with($this->name, self::prevPrefix)) {
            $copy = clone($this);
            $copy->name = $this->name.self::prevPrefix;
            $copy->autorender = false;
            $copy->value = $copy->default = tl::tl("Previous");
            $this->prevButton = $page->addField($copy);
        }
        $this->buttonCancel = $page->addField(new ButtonCancel($this->name."_cancel", tl::tl("Cancel"), $pageNo));
        $this->addButton = $addButton;
        if($addButton != null) {
            $addButton->autorender = false;
            $page->addField($addButton);
        }
        $this->setTdAttribute("style", "padding-top: 10px;padding-bottom: 10px");
    }
    
    function setTdAttribute($name, $value) {
        $this->tdattributes[$name] = $value;
        $this->buttonCancel->tdattributes[$name] = $value;
        if($this->addButton != null) {
            $this->addButton->tdattributes[$name] = $value;
        }
        if($this->prevButton != null) {
            $this->prevButton->tdattributes[$name] = $value;
        }
    }
    
    function renderParentInput() {
        return parent::render_input();
    }
    
    function render_input() {
        if($this->addButton != null && $this->addButton->msg != "") {
            $this->msg = $this->addButton->msg;
        }
        $ret = "<td ".html::formatAttributes($this->tdattributes).">
                  <table>
                    <tr>";
        if($this->prevMode != null) {               // render PREVIOUS button, if given
            if($this->prevButton == null) {
                $ret .= $this->render_dummy_button(tl::tl("Previous"), $this->prevMode, "button", array("onClick" => "javascript:submitMode('{$this->prevMode}')", "formnovalidate" => "formnovalidate"));
            }
            else {
                $ret .= $this->prevButton->renderParentInput();
            }
        }
        else if($this->nextMode != null) {
            $ret .= $this->render_dummy_button(tl::tl("Previous"), "_prev", "button", array("disabled" => "disabled"));
        }
        if($this->prevMode != null || $this->nextMode != null) {
            $ret .= "<td>&nbsp;</td>";
        }
        if(!$this->buttonCancel->hidden) {          // render CANCEL button
            $ret .= $this->buttonCancel->render_input();
        }
        $ret .= "<td>&nbsp;</td>";
        $ret .= parent::render_input();             // render OK/NEXT button, which is always the last confirmation button if nextMode == null
        /*else {
            $ret .= $this->render_dummy_button(tl::tl("Ok"), "_ok", "submit", array("disabled" => "disabled"));
        }*/
        if($this->addButton != null) {
            $ret .= "<td>&nbsp;&nbsp;&nbsp;&nbsp;</td>";
            $ret .= $this->addButton->render_input();
        }
        $ret .=    "</tr>
                  </table>
                </td>";
        return $ret;
    }
    
    function render_dummy_button($value, $mode, $type, $attributes) {
        return "<td class='input' ".html::formatAttributes($this->tdattributes).">"
            . "<input value='$value' "
            . html::formatAttributes($attributes)
            . "name='{$this->name}$mode' "
            . "type='$type' "
            . "/>" .
            "</td>"; 
    }
}

class MainInputPageNoBoxDateField extends InputPageDateField {
    var $renderHidden = false;
    public function MainInputPageNoBoxDateField($name, $prompt = "", $default = 0, InputPageAction $button = null, $showtime = false) {
        parent::__construct($name, $prompt, $default, $button, $showtime);
        $this->tdattributes["nowrap"] = "nowrap";
        $this->tdattributes["valign"] = "top";
        $this->columnsToRender = array("input" => 1);
    }
    
    // render the field in $columns columns
    public function render_input() {
        if ($this->hidden) return "";
        $tdattribs = html::formatAttributes($this->tdattributes);
        return "     <td {$tdattribs} name='" . $this->name . "'>"
            . $this->makeDisplayString()
            . "</td>".$this->render_hidden();
    }
    
    public function render_hidden() {
        if($this->renderHidden) {
            return parent::render_hidden();
        }
        else {
            return "";
        }
    }
    
    public function computeFromForm() {
        parent::computeFromForm();
        // reset msg, otherwise field does not validate
        $this->msg = "";
    }
    
    public function valid() {
        return true; // just for display, no input validation needed
    }
}

class CronJob {
    var $min;
    var $hour;
    var $dayOfMonth;
    var $month;
    var $dayOfWeek;
    var $cmd;
    
    const typeBackup = 1;
    
    function CronJob ($min = "*", $hour = "*", $dayOfMonth = "*", $month = "*", $dayOfWeek = "*", $cmd = "") {
        $this->min = $min;
        $this->hour = $hour;
        $this->dayOfMonth = $dayOfMonth;
        $this->month = $month;
        $this->dayOfWeek = $dayOfWeek;
        $this->cmd = $cmd;
    }
    
    function toLine() {
        return "{$this->min} {$this->hour} {$this->dayOfMonth} {$this->month} {$this->dayOfWeek} {$this->cmd}\n";
    }
}

class IPValues {
    var $dhcp = null;
    var $ip = "";
    var $mask = "";
    var $gateway = "";
    var $dns1 = "";
    var $dns2 = "";
}

class MainMethods {
    const diskPart = 0, diskSize = 1, diskUsed = 2, diskAvailable = 3, diskUsePercent = 4;
    const ramSize = 1, ramUsed = 2, ramAvailable = 3, ramShared = 4, ramBuffers = 5, ramCached = 6;
    const swapSize = 1, swapUsed = 2, swapAvailable = 3;
    const dummyPw = "********";
    /** 
    * @return IPValues 
    */
    static function retrieveIPVals() {
        $ipVals = new IPValues();
        $netLines = file(Config::networkFile);
        foreach($netLines as $line) {
            if(preg_match("/^[^#]*eth0.*dhcp.*$/", $line)) {
                $ipVals->dhcp = true;   // break, as other values are determined by DHCP-Server
                break;
            }
            elseif(preg_match("/^[^#]*eth0.*static.*$/", $line)) $ipVals->dhcp = false;
            elseif(isset($ipVals->dhcp)) {
                if(preg_match("/^[^#]*address(.*)$/", $line, $matches)) {
                    $ipVals->ip = trim($matches[1]);
                }
                else if(preg_match("/^[^#]*gateway(.*)$/", $line, $matches)) {
                    $ipVals->gateway = trim($matches[1]);
                }
                else if(preg_match("/^[^#]*netmask(.*)$/", $line, $matches)) {
                    $ipVals->mask = trim($matches[1]);
                }
            }
        }
        $dnsLines = file(Config::dnsFile);
        foreach($dnsLines as $line) {
            if(preg_match("/^[^#]*nameserver(.*)$/", $line, $matches)) {
                $dns = trim($matches[1]);
                if($ipVals->dns1 == "") $ipVals->dns1 = $dns;
                elseif($ipVals->dns2 == "") $ipVals->dns2 = $dns;
                else break;
            }
        }
        if($ipVals->dhcp) {
            $dhcpLines = file(Config::dhcpFile);
            $currentLeaseIndex = 0; 
            foreach($dhcpLines as $index => $line) {    // search for the last lease, which is the current lease
                if(strpos($line, "{") !== false) {
                    $currentLeaseIndex = $index;
                }
            }
            foreach($dhcpLines as $index => $line) {
                if($index < $currentLeaseIndex) continue;   // ignore lines of outdated leases
                if(preg_match("/^[^#]*fixed-address(.*)$/", $line, $matches)) {
                    $ipVals->ip = rtrim(trim($matches[1]), ";");
                }
                elseif(preg_match("/^[^#]*subnet-mask(.*)$/", $line, $matches)) {
                    $ipVals->mask = rtrim(trim($matches[1]), ";");
                }
                elseif(preg_match("/^[^#]*routers(.*)$/", $line, $matches)) {
                    $routers = rtrim(trim($matches[1]), ";");
                    $routerVals = explode(",", $routers);
                    if(isset($routerVals[0])) {
                        $ipVals->gateway = $routerVals[0];
                    }
                }
                if(strpos($line, "}") !== false) break; // end of lease
            }
        }
        return $ipVals;
    }
    
    static function performChecks(MyPage &$page = null) {
        $value = "";
        $value .= self::checkKernel($page);
        $value .= self::checkMissingPackages($page);
        return $value;
    }
    
    static function checkKernel(MyPage &$page = null) {
        $value = "";
        if(runs_in_vm()) return $value;         // no kernel check for vm
        $out = trim(main_system("uname -r", $ret));
        $kernelV = trim(file_get_contents(Config::kernelVersion));
        if($out != $kernelV) {
            $value = "<div style=\"color:red\">".tl::tlx('You\'re not running the latest kernel {$arg[1]}!', "kernel-armv5te-$kernelV");
            $value .= "<br>".tl::tlx(/* $1 is our Wiki */ 'Take a look at our {$arg[1]} to see, what you have to do now!', "<a target='_blank' href='".Config::wikiHelp."#Configuring_a_new_Kernel'>Wiki</a>");
            $value .= "</div><br>";
            if($page != null) {
                $page->addField($text = new InputPageText("oldkernel", $value));
            }
            else {
                $value = "<br>".$value;
            }
        }
        return $value;
    }
    
    static function checkMissingPackages(MyPage &$page = null) {
        $value = "";
        if(file_exists(Config::missingPackages) && (($content = file_get_contents(Config::missingPackages)) != "")) {
            $value = "<div style=\"color:red\">".tl::tlx('The following packages couldn\'t be upgraded due to missing dependencies: {$arg[1]}', $content);
            $value .= "<br>".tl::tlx(/* $1 is our Wiki */ 'Take a look at our {$arg[1]} to see, what you have to do now!', "<a target='_blank' href='".Config::wikiHelp."#Manual_Debian_Upgrade'>Wiki</a>");
            $value .= "</div><br>";
            if($page != null) {
                $page->addField($text = new InputPageText("missingpackages", $value));
            }
            else {
                $value = "<br>".$value;
            }
        }
        return $value;
    }

    static function getUptimeInfo() {
        $ret = 0;
        $cmd = "uptime";
        $up = main_system($cmd, $ret);
        return $up;
    }

    static function getSwapInfo() {
        $ret = 0;
        $cmd = "free -h /";
        $free = main_system($cmd, $ret);
        $lines = explode("\n", $free);
        if(isset($lines[3])) {
            $parts = explode(" ", $lines[3]);
            foreach($parts as $i => $part) {
                if($part == "") unset($parts[$i]);
            }
            $parts = array_values($parts);
            return $parts;
        }
        return array();
    }

    static function getRamInfo() {
        $ret = 0;
        $cmd = "free -h /";
        $free = main_system($cmd, $ret);
        $lines = explode("\n", $free);
        if(isset($lines[1])) {
            $parts = explode(" ", $lines[1]);
            foreach($parts as $i => $part) {
                if($part == "") unset($parts[$i]);
            }
            $parts = array_values($parts);
            return $parts;
        }
        return array();
    }
    
    static function getDiskInfo() {
        $ret = 0;
        $cmd = "df -h /";
        $df = main_system($cmd, $ret);
        $lines = explode("\n", $df);
        if(isset($lines[1])) {
            $parts = explode(" ", $lines[1]);
            foreach($parts as $i => $part) {
                if($part == "") unset($parts[$i]);
            }
            $parts = array_values($parts);
            $parts[self::diskUsePercent] = intval(trim($parts[self::diskUsePercent], "%"));
            return $parts;
        }
        return array();
    }
    
    public static function evaluateConfigShError($ret) {
        $error = "";
        switch($ret) {
            case Config::configErrorSsl: 
                $error = tl::tl("The SSL settings are currently modified.");
                break;
            case Config::configErrorApplication: 
                $error = tl::tl("An application script is currently running.");
                break;
            case Config::configErrorCron: 
                $error = tl::tl("A cron job is currently modified.");
                break;
            case Config::configErrorInnovaphone: 
                $error = tl::tl("The Linux Application Platform settings are currently locked.");
                break;
            case Config::configErrorNtp: 
                $error = tl::tl("The NTP settings are currently modified.");
                break;
            case Config::configErrorPostgres: 
                $error = tl::tl("The database is currently locked.");
                break;
            case Config::configErrorReload: 
                $error = tl::tl("The web server is currently reloaded.");
                break;
            case Config::configErrorRpcap: 
                $error = tl::tl("RPCAP is currently reconfigured.");
                break;
            case Config::configErrorTimezone: 
                $error = tl::tl("The timezone settings are currently modified.");
                break;
        }
        if($error != "") {
            $error .= " ".tl::tl("Please wait and try again later...");
        }
        return $error;
    }

    public static function callConfigSh($cmd, &$ret) {
        if(strpos($cmd, "(") === 0) {
            $cmd = "(/usr/bin/setsid ".substr($cmd, 1);
        }
        else {
            $cmd = "/usr/bin/setsid $cmd";
        }
        $out = main_system($cmd, $ret);
        $error = self::evaluateConfigShError($ret);
        if($error == "") return $out;
        else return $error;
    }
    
    public static function checkRunningScript($script) {
        $ret = 0;
        switch($script) {
            case Config::configErrorSsl: 
                if(file_exists(Config::configCheckSsl)) {
                    $ret = Config::configErrorSsl;
                }
                break;
            case Config::configErrorCron:
                if(file_exists(Config::configCheckCron)) {
                    $ret = Config::configErrorCron;
                }
                break;
            case Config::configErrorReload:
                if(file_exists(Config::configCheckReload)) {
                    $ret = Config::configErrorReload;
                }
                break;
            case Config::configErrorTimezone:
                if(file_exists(Config::configCheckTimezone)) {
                    $ret = Config::configErrorTimezone;
                }
                break;
            case Config::configErrorNtp:
                if(file_exists(Config::configCheckNtp)) {
                    $ret = Config::configErrorNtp;
                }
                break;
            case Config::configErrorRpcap:
                if(file_exists(Config::configCheckRpcap)) {
                    $ret = Config::configErrorRpcap;
                }
                break;
            case Config::configErrorPostgres:
                if(file_exists(Config::configCheckPostgres)) {
                    $ret = Config::configErrorPostgres;
                }
                break;
            case Config::configErrorApplication:
                if(file_exists(Config::configCheckApplication)) {
                    $ret = Config::configErrorApplication;
                }
                break;
            case Config::configErrorInnovaphone:
                if(file_exists(Config::configCheckInnovaphone)) {
                    $ret = Config::configErrorInnovaphone;
                }
                break;
        }
        return self::evaluateConfigShError($ret);
    }
}

class CronJobs {
    var $jobs = array();
    var $pre = "";
    var $error = "";
    const path = "PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin";
    
    function CronJobs($cronFile) {
        $cmd="/usr/bin/sudo /usr/local/bin/config.sh cron read $cronFile";
        $out = MainMethods::callConfigSh($cmd, $ret);
        if($ret == 0) {
            //var_dump($out);
            $preEnd = false;
            $out = explode("\n", $out);
            foreach($out as $line) {
                $matches = array();
                if(preg_match("/^[^\S]*([0-9\/\*,-]+)[^\S]*([0-9\/\*,-]+)[^\S]*([0-9\/\*,-]+)[^\S]*([0-9\/\*,-]+)[^\S]*([0-9\/\*,-]+)[^\S]*(.*)$/", $line, $matches)) {
                    $this->jobs[] = new CronJob($matches[1], $matches[2], $matches[3], $matches[4], $matches[5], $matches[6]);
                    $preEnd = true;
                }
                else if(!$preEnd && trim($preEnd) != ""){
                    $this->pre .= "$line\n";
                }
            }
        }
        else {
            $this->error = tl::tl("Could not read current cron jobs.");
        }
    }
    
    function add($script, CronJob $job) {
        foreach($this->jobs as $index => $j) {
            if(strpos($j->cmd, $script) !== false) {
                $this->jobs[$index] = $job;
                return;
            }
        }
        $this->jobs[] = $job;
    }
    
    function remove($script) {
        foreach($this->jobs as $index => $job) {
            if(strpos($job->cmd, $script) !== false) {
                unset($this->jobs[$index]);
                break;
            }
        }
    }
    
    function submit($cronFile) {
        if(($fileTmp = tempnam("/tmp","CRON")) !== false) {
            $f = @fopen($fileTmp, "w");
            if($f) {
                if(strpos($this->pre, "PATH=") === false) {
                    fwrite($f, self::path."\n");
                }
                fwrite($f, $this->pre);
                foreach($this->jobs as $job) {
                    fwrite($f, $job->toLine());
                }
                fclose($f);
                $cmd="/usr/bin/sudo /usr/local/bin/config.sh cron write $cronFile $fileTmp";
                $error = MainMethods::callConfigSh($cmd, $retval);
                if($error != "") {
                    $this->error = tl::tl('Cron file could not be written').": $error";
                }
                $cmd="/usr/bin/sudo /usr/local/bin/config.sh cron restart";
                $error = MainMethods::callConfigSh($cmd, $retval);
                if($error != "") {
                    $this->error = tl::tl('Cron process could not be restarted').": $error";
                }
            }
            else {
                $this->error = tl::tlx('Could not open cron file {$arg[1]}', $cronFile);
            }
            if(file_exists($fileTmp)) {
                unlink($fileTmp);
            }
        } else {
            $this->error = tl::tl('Could not create temp file');
        }
        if($this->error != "") {
            return false;
        }
        return true;
    }

    static function encodePHPParameters($str) {
        return str_replace("%", "\\%", urlencode($str));
    }
}

class AuthenticatedUrl {
    var $url;
    var $user;
    var $pw;
    var $matchUrl;
    
    public function AuthenticatedUrl($url, $user, $pw) {
        $this->url = $url;
        $this->user = $user;
        $this->pw = $pw;
        $this->matchUrl = self::removeDefaultPort($this->url);
    }
    
    static function removeDefaultPort($url) {
        $tmpUrl = $url;
        if(strpos(strtolower($url), "https:") === 0) {
            $tmpUrl = str_replace(":443/", "/", $url);
        }
        else if(strpos(strtolower($url), "http:") === 0) {
            $tmpUrl = str_replace(":80/", "/", $url);
        }
        return $tmpUrl;
    }
    
    public function match($url) {
        if(strpos($url, $this->matchUrl) === 0) {
            return true;
        }
        return false;
    }
}

class SemaphoreCommand {
    private $semaphoreId;
    private $sharedMemId;
    private $myPid;
    var $semaphoreKey = "4118992381";
    var $sharedMemKey = "4022473221";
    var $gotSemaphore = false;

    public function __construct() {
        $this->semaphoreId = sem_get($this->semaphoreKey, 1, 0666, 0);
        $this->sharedMemId = shm_attach($this->sharedMemKey, 512, 0660);
    }

    public function getSemaphore() {
        if(($this->semaphoreId === false) || ($this->sharedMemId === false)) return false;

        $runningPid = @shm_get_var ($this->sharedMemId, 2);
        $lock = @shm_get_var ($this->sharedMemId, 1);
        if (!strcmp($lock,"busy")) {
            // Check if running pid is really running...only happens when the executing php process was killed somehow...
            if((strpos(exec("ps p".$runningPid),"php5-cgi")) === false) {
                sem_remove($this->semaphoreId);
                sem_release($this->semaphoreId);
                shm_remove($this->sharedMemId);
                shm_detach($this->sharedMemId);
                // Create them new
                $this->semaphoreId = sem_get($this->semaphoreKey, 1, 0666, 0);
                $this->sharedMemId = shm_attach($this->sharedMemKey, 512, 0660);
            } else
                return false;
        }

        if (!sem_acquire($this->semaphoreId)) {
            return false;
        }
      
        $this->myPid = getmypid();
        if ((!shm_put_var($this->sharedMemId, 1, "busy")) || (!shm_put_var($this->sharedMemId, 2, $this->myPid))) {
            sem_remove($this->semaphoreId);
            shm_remove($this->sharedMemId);
            return false;
        }

        return true;
    }

    public function releaseSemaphore() {
        if ((!shm_put_var($this->sharedMemId, 1, "free")) || (!shm_put_var($this->sharedMemId, 2, "0"))) {
            sem_remove($this->semaphoreId);
            sem_release($this->semaphoreId);
            shm_remove($this->sharedMemId);
            shm_detach($this->sharedMemId);
        } else {
            shm_detach($this->sharedMemId);
            sem_release($this->semaphoreId);
        } 
    }

    public function __destruct() {
        if($this->myPid == @shm_get_var ($this->sharedMemId, 2)) {
            $this->releaseSemaphore();
        } else { 
            @shm_detach($this->sharedMemId);
            @sem_release($this->semaphoreId);
        }
    }

    function genRandomString($length) {
        //$characters = "0123456789abcdefghijklmnopqrstuvwxyz";
        $characters = "0123456789";
        for ($p = 0; $p < $length; $p++) {
            $string .= $characters[mt_rand(0, strlen($characters))];
        }
        return $string;
    }
}

class DataBase
{
    var $name;
    var $user;
    
    function DataBase($name = "", $user = "") {
        $this->name = $name;
        $this->user = $user;
    }
}

class LogFile
{
    var $title;
    var $file;
    var $noClear;
    
    function LogFile($title = "", $file = "", $noClear = false) 
    {
        $this->title = $title;
        $this->file = $file;
        $this->noClear = $noClear;
    }
}

class BackupCmd
{
    var $title = "";
    var $cmd = "";
    var $script = "";
    var $appName = "";
    
    function BackupCmd($title = "", $cmd = "", $script = "", $appName = "") 
    {
        $this->title = $title;
        $this->cmd = $cmd;
        $this->script = $script;
        $this->appName = $appName;
    }
}

class RestoreCmd
{
    var $title = "";
    var $cmd = "";
    var $script = "";
    var $actions = array();
    var $appName = "";
    
    function RestoreCmd($title = "", $cmd = "", $script = "", $actions = array(), $appName = "") 
    {
        $this->title = $title;
        $this->cmd = $cmd;
        $this->script = $script;
        $this->actions = $actions;
        $this->appName = $appName;
    }
}

class XmlCronJob {
    var $user;
    var $script;
    
    function XmlCronJob($user = "", $script = "") {
        $this->user = $user;
        $this->script = $script;
    }
}

class Application
{
    var $mtime = 0;
    var $file = "";
    var $name = "";
    var $displayName = "";
    var $desc = "";
    var $dir = "";
    var $href = "";
    var $user = "";
    var $packages = array();
    var $db = null;
    var $cronJobs = array();
    var $logFiles = array();
    var $configFiles = array();
    var $backupCmds = array();
    var $restoreCmds = array();
    var $uninstallScript = "";
    var $version = "";
    var $hotfix = "";
    var $hotfixOffset = 0;
    var $versionStr = "";
    var $lighttpdAuth = false;
    var $authFile = "";
    var $publicWebPaths = array();
    
    function Application() 
    {
        $this->db = new DataBase();
    }
}

class RelayHost {
    var $id = 0;
    var $prot = "";
    var $server = "";
    var $email = "";
    var $port = "";
    var $user = "";
    var $password = "";
    var $error = "";
    
    public function RelayHost() {
        
    }
    
    function read($id) {
        $this->id = $id;
        $innovaphoneDb = get_innovaphone_db();
        $query = "SELECT * FROM relay_hosts WHERE id=$id";
        $result = $innovaphoneDb->run($query);
        if(($row = pg_fetch_object($result))) {
            $this->server = $row->server;
            $this->email = $row->email;
            $this->user = $row->username;
            $this->password = $row->password;
            $this->port = $row->port;
            return true;
        }
        else {
            $this->error = pg_last_error($innovaphoneDb->db);
        }
        return false;
    }
    
    function checkEmail($email) {
        $innovaphoneDb = get_innovaphone_db();
        $query = "SELECT id FROM relay_hosts WHERE email='".pg_escape_string(strtolower($email))."' AND id!={$this->id};";
        $result = $innovaphoneDb->run($query);
        if(pg_num_rows($result) > 0) {
            $this->error = tl::tlx('A relay host with email {$arg[1]} already exists!', $email);
            return false;
        }
        return true;
    }
    
    function write() {
        $innovaphoneDb = get_innovaphone_db();
        if($innovaphoneDb->run("BEGIN") === false) {
            $this->error = pg_last_error($innovaphoneDb->db);
            return false;
        }
        $query = "LOCK TABLE relay_hosts IN EXCLUSIVE MODE";
        if($innovaphoneDb->run($query) === false) {
            $this->error = pg_last_error($innovaphoneDb->db);
            return false;
        }
        if($this->checkEmail($this->email) === false) {
            return false;
        }
        if($this->id > 0 && $this->delete() === false) {
            return false;
        }
        if($this->port == "") $this->port = "25";
        $idCol = ($this->id > 0 ? "id, " : "");
        $idVal = ($this->id > 0 ? "{$this->id}, " : "");
        $query = "INSERT INTO relay_hosts ($idCol server, email, port, username, password) VALUES ($idVal".
                    "'".pg_escape_string(strtolower($this->server))."',".
                    "'".pg_escape_string(strtolower($this->email))."',".
                    pg_escape_string($this->port).",".
                    "'".pg_escape_string($this->user)."',".
                    "'".pg_escape_string($this->password)."') RETURNING id";
        $result = $innovaphoneDb->run($query);
        if($result === false || $innovaphoneDb->run("COMMIT") === false) {
            $this->error = pg_last_error($innovaphoneDb->db);
            return false;
        }
        if($this->id == 0) {
            if(($row = pg_fetch_object($result))) {
                $this->id = $row->id;
            }
        }
        return true;
    }
    
    function delete() {
        if($this->id == 0) return true;
        $innovaphoneDb = get_innovaphone_db();
        $query= "DELETE FROM relay_hosts WHERE id={$this->id};";
        if($innovaphoneDb->run($query) === false) {
            $this->error = pg_last_error($innovaphoneDb->db);
            return false;
        }
        return true;
    }
}

class WebPath {
    var $path;
    var $write;
    
    function WebPath($path, $write = false) {
        $this->path = $path;
        $this->write = $write;
    }
}

function rc4_encrypt($key, $pw) {
    if($key == "" || $pw == "") return "";
    $enc = rc4Encrypt($key, $pw);
    $hex = "";
    $i = 0;
    do {
        $hex .= sprintf("%02x", ord($enc{$i}));
        $i++;
    } while ($i < strlen($enc));
    return $hex;
}

function rc4_decrypt($key, $enc) {
    if($key == "" || $enc == "") return "";
    $bin = "";
    $i = 0;
    do {
        $bin .= chr(hexdec($enc{$i}.$enc{($i + 1)}));
        $i += 2;
    } while ($i < strlen($enc));
    return rc4Decrypt($key, $bin);    
}

class MTLSConfig {
    var $active = false;
    var $port = Config::mtlsPort;
}

class AlarmServer {
    var $ip = "";
    var $port = "";
    var $pw = "";
    var $user = "";
    var $https = false;
}

// the webserver config file is xml formatted!
class WebServerConfig {
    var $forceHttps = false;
    var $publicWebPaths = array();
    var $publicWebDavPaths = array();
    var $authUrls = array();
    var $backupUrl = "";
    /**
     *
     * @var AlarmServer
     */
    var $alarmServer = null;
    var $ntpServer = null;
    var $pollTimeout = "15";
    var $error = "";
    var $rpcapOn = false;
    var $mtime = 0;
    var $timezone = "Europe/Berlin";
    var $appAuth = array();
    
    /**
     *
     * @var MTLSConfig
     */
    var $mtls = null;
    
    function WebServerConfig() {
        $doc = new DOMDocument();
        $doc->encoding = "utf-8";
        $doc->preserveWhiteSpace = false;
        @$doc->load(Config::serverCfgFile);
        $rewriteConfig = false;
        $this->alarmServer = new AlarmServer();
        $this->mtls = new MTLSConfig();
        if($doc !== false) {
            if(file_exists(Config::serverCfgFile)) {
                $this->mtime = filemtime(Config::serverCfgFile);
            }
            $config = $doc->getElementsByTagName ("config");
            if($config->length > 0) {
                $config = $config->item(0);
                foreach($config->childNodes as $node) {
                    switch($node->nodeName) {
                        case "mtls":
                            foreach($node->attributes as $attrib) {
                                switch($attrib->name) {
                                    case "port":
                                        $this->mtls->port = intval($attrib->value);
                                        break;
                                    case "active":
                                        if(strtolower($attrib->value) == "true" || $attrib->value == "1") {
                                            $this->mtls->active = true;
                                        }
                                        break;
                                }
                            }
                            break;
                        case "timezone":
                            $this->timezone = $node->textContent;
                            break;
                        case "force-https":
                            $this->forceHttps = (bool)$node->textContent;
                            break;
                        case "public-paths":
                            foreach($node->childNodes as $path) {
                                switch($path->nodeName) {
                                    case "web": $this->publicWebPaths[] = new WebPath($path->textContent); break;
                                    case "webdav": 
                                        $write = false;
                                        foreach($path->attributes as $attrib) {
                                            if($attrib->name == "write" && (strtolower($attrib->value) == "true" || $attrib->value == "1")) {
                                                $write = true;
                                            }
                                        }
                                        $this->publicWebDavPaths[] = new WebPath($path->textContent, $write);
                                        break;
                                }
                            }
                            break;
                        case "authenticated-urls":
                            foreach($node->childNodes as $url) {
                                switch($url->nodeName) {
                                    case "url": 
                                        $user = "";
                                        $pw = "";
                                        foreach($url->attributes as $attrib) {
                                            switch($attrib->name) {
                                                case "user": $user = $attrib->value; break;
                                                case "password": $pw = $attrib->value; break;
                                            }
                                        }
                                        $this->authUrls[] = new AuthenticatedUrl($url->textContent, $user, rc4_decrypt(strrev($url->textContent), $pw));
                                        break;
                                }
                            }
                            break;
                        case "backup":
                            foreach($node->childNodes as $url) {
                                switch($url->nodeName) {
                                    case "url":
                                        $this->backupUrl = $url->textContent;
                                        foreach($url->attributes as $attrib) {
                                            switch($attrib->name) {
                                                case "poll":
                                                    $this->pollTimeout = $attrib->value; 
                                                    break;
                                            }
                                        }
                                        break;
                                    /*case "alarm-server-ip":
                                        $this->alarmServerIp = $url->textContent;
                                        break;                                    */
                                }
                            }
                            break;
                        case "rpcap":
                            $this->rpcapOn = (bool)$node->textContent;
                            break;                                    
                        case "ntp-servers":
                            foreach($node->childNodes as $ntp) {
                                switch($ntp->nodeName) {
                                    case "ntp-server":
                                        $this->ntpServer[] = $ntp->textContent;
                                        break;
                                }
                            }
                            break;
                        case "application-auth":
                            foreach($node->childNodes as $auth) {
                                if($auth->nodeName == "auth") {
                                    $this->appAuth[$auth->textContent] = true;
                                }
                            }
                            break;
                        case "alarm-server":
                            foreach($node->childNodes as $child) {
                                switch($child->nodeName) {
                                    case "ip": $this->alarmServer->ip = $child->textContent; break;
                                    case "port": $this->alarmServer->port = $child->textContent; break;
                                    case "password": $this->alarmServer->pw = $child->textContent; break;
                                    case "user": $this->alarmServer->user = $child->textContent; break;
                                    case "https": $this->alarmServer->https = (bool)$child->textContent; break;
                                }
                            }
                            if($this->alarmServer->pw != "") {
                                $this->alarmServer->pw = rc4_decrypt(strrev($this->alarmServer->ip), $this->alarmServer->pw);
                            }
                            break;
                                
                    }
                }
            }
        }
    }

    public function reloadLighttpdConfig(&$ret) {
        $reload = "(/usr/bin/sudo /usr/local/bin/config.sh reload ".Config::confTempFile." ".Config::lighttpdCfgFile." >/dev/null 2>&1) >/dev/null &";
        return MainMethods::callConfigSh($reload, $ret);
    }
    
    public function getPathMatch(&$path) {
        if(ends_with($path, "/")) {
            $path = rtrim($path, "/");
            return "($|/.*)";
        }
        else {
            return "($|/[^/]*$)";
        }
    }
    
    /**
     * Summary of publicRoot
     * @param WebPath[] $paths 
     * @return WebPath
     */
    public function publicRoot($paths) {
        foreach($paths as $path) {
            if($path->path == "/") {
                return $path;
            }
        }
        return null;
    }
    
    static public function sortWebdavPaths($left, $right)
    {
        return strcmp($right->path, $left->path);
    }
    
    public function writeLighttpdConfig() {
        global $mainSession;
        usort($this->publicWebDavPaths, array("WebServerConfig", "sortWebdavPaths"));
        $config = file_get_contents(Config::confTemplate);
        $passwdTmp = "";
        $passwdTmp .= "    \$HTTP[\"url\"] =~ \"^/((classes/|)[^/]*(.js|.css|.xsl|.ico)$)|((classes/|)images/[^/]*\.[^/]*$)\"  {\n";
        $passwdTmp .= "        # dir-listing.encoding        = \"utf-8\"\n";
        $passwdTmp .= "    }";
        foreach($mainSession->apps as $app) {
            foreach($app->publicWebPaths as $path) {
                $p = $path;
                $match = $this->getPathMatch($p);
                $passwdTmp .= " else \$HTTP[\"url\"] =~ \"^/apps/{$app->name}/{$p}{$match}\"  {\n";
                $passwdTmp .= "        # dir-listing.encoding        = \"utf-8\"\n";
                $passwdTmp .= "    }";
            }
        }
        if(count($this->publicWebPaths) == 0 && count($this->publicWebDavPaths) == 0) {
            $passwdTmp .= " else ";
            $passwdTmp .= file_get_contents(Config::confPasswdWebDav);
            foreach($this->appAuth as $appName => $useAuth) {
                $app = $mainSession->getApp($appName);
                if($app && $useAuth && file_exists($app->authFile)) {
                    $passwdTmp .= "    ".$this->additionalAuth($app->dir, $app->authFile);
                }
            }
            $passwdTmp .= " else ";
            $passwdTmp .= file_get_contents(Config::confPasswdWeb);
            $passwdTmp .= "\n";
        }
        else {
            $publicWebRoot = $this->publicRoot($this->publicWebPaths);
            $publicWebDavRoot = $this->publicRoot($this->publicWebDavPaths);
            
            if($publicWebRoot === null) {
                foreach($this->publicWebPaths as $path) {
                    $ignorePath = false;
                    foreach($this->appAuth as $appName => $useAuth) {
                        if($useAuth && ($path->path == "/apps/$appName" || $path->path == "/apps/$appName/")) {
                            $ignorePath = true;
                            break;
                        }
                    }
                    foreach($mainSession->apps as $app) {
                        foreach($app->publicWebPaths as $appPath) {
                            if($path->path == "/apps/{$app->name}/$appPath" || $path->path == "/apps/{$app->name}/$appPath/") {
                                $ignorePath = true;
                                break;
                            }
                        }
                        if($ignorePath) {
                            break;
                        }
                    }
                    if($ignorePath) {
                        continue;
                    }
                    $p = $path->path;
                    $match = $this->getPathMatch($p);
                    $passwdTmp .= " else \$HTTP[\"url\"] =~ \"^{$p}{$match}\"  {\n";
                    $passwdTmp .= "        # dir-listing.encoding        = \"utf-8\"\n";
                    $passwdTmp .= "    }";
                }
            }
            foreach($this->appAuth as $appName => $useAuth) {
                $app = $mainSession->getApp($appName);
                if($app && $useAuth && file_exists($app->authFile)) {
                    $passwdTmp .= $this->additionalAuth($app->dir, $app->authFile);
                }
            }
            foreach($this->publicWebDavPaths as $path) {
                $passwdTmp .= " else \$HTTP[\"url\"] =~ \"^/webdav";
                $p = $path->path;
                $match = $this->getPathMatch($p);
                $passwdTmp .= $p;
                $passwdTmp .= "$match\"  {\n";
                $passwdTmp .= "        server.dir-listing = \"enable\"\n";
                $passwdTmp .= "        webdav.activate = \"enable\"\n";
                if($path->write) {
                    $passwdTmp .= "        webdav.is-readonly = \"disable\"\n";
                }
                else {
                    $passwdTmp .= "        webdav.is-readonly = \"enable\"\n";
                    $passwdTmp .= "        webdav.auth-backend = \"htdigest\"\n";
                    $passwdTmp .= "        webdav.htdigest-webdav = \"/home/lighttpd/webdav_htdigest.user\"\n";
                    $passwdTmp .= "        webdav.realm-webdav = \"Linux Webdav\"\n";
                }
                $passwdTmp .= "        webdav.sqlite-db-name = \"/var/run/lighttpd/webdav_props.db\"\n";
                $passwdTmp .= "    }";
            }
            if($publicWebDavRoot === null) {
                $passwdTmp .= " else ";
                $passwdTmp .= file_get_contents(Config::confPasswdWebDav);
            }
            if($publicWebRoot !== null) {
                $passwdTmp .= " else \$HTTP[\"url\"] =~ \"^.*\"  {\n";
                $passwdTmp .= "        # dir-listing.encoding        = \"utf-8\"\n";
                $passwdTmp .= "    }\n";
            }
            else {
                $passwdTmp .= " else ";
                $passwdTmp .= file_get_contents(Config::confPasswdWeb);
                $passwdTmp .= "\n";
            }
        }
        if($this->forceHttps) {
            $config .= "\$SERVER[\"socket\"] == \":80\" {\n";
            $config .= "  \$HTTP[\"host\"] =~ \"(.*)\" {\n";
            $config .= "      url.redirect = ( \"^/(.*)\" => \"https://%1/$1\" )\n";
            $config .= "  }\n";
            $config .= "}\n";
        }
        else {
            $config .= "\$SERVER[\"socket\"] == \":80\" {\n";
            $config .= $passwdTmp;
            $config .= "}\n";
        }
        $config .= "\n";
        $config .= "\$SERVER[\"socket\"] == \":443\" {\n";
        $config .= "    server.document-root = \"/var/www/innovaphone\"\n";
        $config .= "    ssl.engine = \"enable\"\n";
        $config .= "    ssl.use-sslv3 =\"disable\"\n";
        $config .= "    ssl.pemfile = \"".Config::certServer."\"\n";
        $config .= "    ssl.dh-file = \"".Config::certDiffieHellman."\"\n";
        $config .= "# for PHP, \$_SERVER['HTTPS'] equals \"on\"\n";
        $config .= "    setenv.add-environment = (\n";
        $config .= "        \"HTTPS\" => \"on\"\n";
        $config .= "    )\n";
        $config .= $passwdTmp;
        $config .= "}\n";
        
        if($this->mtls->active) {
            $config .= "\n";
            $config .= "\$SERVER[\"socket\"] == \":".$this->mtls->port."\" {\n";
            $config .= "    server.document-root = \"".Config::mtlsPath."\"\n";
            $config .= "    ssl.engine = \"enable\"\n";
            $config .= "    ssl.use-sslv3 =\"disable\"\n";
            $config .= "    ssl.pemfile = \"".Config::certServer."\"\n";
            $config .= "    ssl.dh-file = \"".Config::certDiffieHellman."\"\n";
            $config .= "    ssl.ca-file = \"".Config::mtlsCert."\"\n";
            $config .= "    # allow client side certificates\n";
            $config .= "    ssl.verifyclient.activate = \"enable\"\n";
            $config .= "    # enforce client side certificates\n";
            $config .= "    ssl.verifyclient.enforce = \"enable\" \n";
            $config .= "    # tell PHP we have HTTPS\n";
            $config .= "    setenv.add-environment = ( \"HTTPS\" => \"on\" )\n";
            $config .= "}\n";
        }
        if(file_put_contents(Config::confTempFile, $config) === false) {
            $this->error = tl::tl("Rewriting lighttpd config failed.");
            return false;
        }
        return true;
    }
    
    public function additionalAuth($app, $authFile) {
        $ret = "";
        $ret .= " else \$HTTP[\"url\"] =~ \"^/apps/$app($|/.*)\"  {\n";
        $ret .= "        server.dir-listing = \"disable\"\n";
        $ret .= "        webdav.activate = \"disable\"\n";
        $ret .= "        auth.backend = \"htdigest\"\n";
        $ret .= "        auth.backend.htdigest.userfile = \"$authFile\"\n";
        $ret .= "        auth.require = (\n";
        $ret .= "                \"\" => ( \"method\" => \"digest\",\n";
        $ret .= "                \"realm\" => \"Linux Web Server\",\n";
        $ret .= "                \"require\" => \"valid-user\" )\n";
        $ret .= "        )\n";
        $ret .= "    }";  
        return $ret;
    }
    
    public function write() {
        $doc = new DOMDocument("1.0", "utf-8");
        $doc->formatOutput = true;
        $config = $doc->appendChild($doc->createElement("config"));
        $config->appendChild($doc->createElement("timezone", $this->timezone));
        $config->appendChild($doc->createElement("force-https", $this->forceHttps));
        $paths = $config->appendChild($doc->createElement("public-paths"));
        foreach($this->publicWebPaths as $path) {
            $paths->appendChild($doc->createElement("web", xmlentities($path->path)));
        }
        foreach($this->publicWebDavPaths as $path) {
            $p = $paths->appendChild($doc->createElement("webdav", xmlentities($path->path)));
        $p->setAttribute("write", $path->write ? "true" : "false");
        }
        $authUrls = $config->appendChild($doc->createElement("authenticated-urls"));
        foreach($this->authUrls as $authUrl) {
            $url = $authUrls->appendChild($doc->createElement("url", xmlentities($authUrl->url)));
            $url->setAttribute("user", $authUrl->user);
            $url->setAttribute("password", rc4_encrypt(strrev($authUrl->url), $authUrl->pw));
        }
        $backup = $config->appendChild($doc->createElement("backup"));
        $backupUrl = $backup->appendChild($doc->createElement("url", xmlentities($this->backupUrl)));
        $backupUrl->setAttribute("poll", $this->pollTimeout);
        //$alarmServerIp = $backup->appendChild($doc->createElement("alarm-server-ip", xmlentities($this->alarmServerIp)));
        $rpcap = $config->appendChild($doc->createElement("rpcap", $this->rpcapOn));
        $ntpServers = $config->appendChild($doc->createElement("ntp-servers"));
        foreach($this->ntpServer as $ntpServer) {
            $ntp = $ntpServers->appendChild($doc->createElement("ntp-server", $ntpServer));
        }
        $appAuths = $config->appendChild($doc->createElement("application-auth"));
        foreach($this->appAuth as $appName => $useAuth) {
            if($useAuth) {
                $appAuths->appendChild($doc->createElement("auth", xmlentities($appName)));
            }
        }
        $mtls = $config->appendChild($doc->createElement("mtls"));
        $mtls->setAttribute("active", $this->mtls->active ? "1" : "0");
        $mtls->setAttribute("port", $this->mtls->port);
        
        $alarmServer = $config->appendChild($doc->createElement("alarm-server"));
        $alarmServer->appendChild($doc->createElement("ip", xmlentities($this->alarmServer->ip)));
        $alarmServer->appendChild($doc->createElement("port", xmlentities($this->alarmServer->port)));
        $alarmServer->appendChild($doc->createElement("password", xmlentities(rc4_encrypt(strrev($this->alarmServer->ip), $this->alarmServer->pw))));
        $alarmServer->appendChild($doc->createElement("user", xmlentities($this->alarmServer->user)));
        $alarmServer->appendChild($doc->createElement("https", xmlentities($this->alarmServer->https)));
        
        if($doc->save(Config::serverCfgFile) === false) {
            $this->error = tl::tlx('Couldn\'t rewrite web server config \'{$arg[1]}\'.', Config::serverCfgFile);
            return false;
        }
        global $mainSession;
        $mainSession->updateWebCfg($this);
        return true;
    }
    
    public function getAuth($url) {
        $bestMatch = null;
        $tmpUrl = AuthenticatedUrl::removeDefaultPort($url);
        foreach($this->authUrls as $authUrl) {
            if($authUrl->match($tmpUrl)) {
                if($bestMatch == null) {
                    $bestMatch = $authUrl;
                }
                elseif(strlen($bestMatch->url) < strlen($authUrl->url)) {
                    $bestMatch = $authUrl;
                }
            }
        }
        return $bestMatch;
    }
}

class InnovaphoneApplicationConsts
{
    const pageApplication = 1;
    const tabList = "list";
    const tabUpload = "upload";
    const modeUninstall = "modeuninstall", modeAskUninstall = "modeaskuninstall";
}

class InnovaphoneDiagnosticsConsts
{
    const pageDiagnostics = 1;
    const tabLogs = "logs", tabReset = "reset", tabRpcap = "rpcap", tabDbExchange = "dbexchange", tabAlarms = "alarms", 
            tabStatus = "status", tabFaxCalls = "faxcalls";
    const modeExchangeUsers = "modeusers", modeExchangeItems = "modeitems", modeFaxDbCalls = "modefaxdbcalls", modeFaxShmCalls = "modefaxshmcalls";
}

class InnovaphoneInstallationConsts
{
    const pageInstall = 1;
    const tabInstall = "install";
    const modeAdminPw = "modeadminpw";
}

class InnovaphoneAdminConsts
{
    const pageAdmin = 1;
    const tabWebServer = "server", tabSSL = "sslupload", tabBackup = "backup", 
    tabRelayHosts = "mails", tabGeneral = "general", tabPgsql = "psql", tabAnnounce = "announce";
    const modeWebdavPw = "modewebdavpw", modeHttpPw = "modehttppw", modeServer = "modeserver",
        modeAuthUrls = "modeauthurls", modeRestore = "moderestore", modeSave = "modesave",
        modeRootPw = "moderootpw", modeNTP = "modentpserver", modePgPw = "pgpw", modeTZ = "modetimezone",
        modeAppPw = "modeapppw", modeIp = "modeip", modeMTLS = "modemtls";
    const modeRelay = "moderelay", modeCsr = "modeCsr", modeDeleteRelay = "modedeleterelay";
}

class MainSession extends mySession {
    var $error = false;
    var $apps = array();
    /**
     *
     * @var WebServerConfig
     */
    var $webCfg = null;
    /**
     *
     * @var string 
     */
    var $dhcpAddr = "";
    var $isVm = false;
    
    /**
     * @var RelayHost
    */
    var $relayHost = null;
    
    public function MainSession($timeout) {
        parent::__construct($timeout, "innovaphone_com", "UTF-8");
        if ($this->fresh()) {
            $this->webCfg = new WebServerConfig();
            $this->save($this->webCfg, "webcfg");
            $this->save($this->apps, "apps");
            $this->dhcpAddr = get_dhcp_address(true);
            $this->isVm = runs_in_vm(true);
            $this->save($this->dhcpAddr, "dhcpaddr");
            $this->save($this->isVm, "isvm");
            $this->relayHost = new RelayHost();
            $this->save($this->relayHost, "relayhost");
        }
        $this->restore($this->apps, "apps");
        $this->restore($this->webCfg, "webcfg");
        $this->restore($this->dhcpAddr, "dhcpaddr");
        $this->restore($this->isVm, "isvm");
        $this->restore($this->relayHost, "relayhost");
        $this->updateApps();
        $this->checkWebCfg();
    }
    
    public function restore(&$o, $from, $setError = true) {
        // restore object from name $from into $o
        if (!isset($_SESSION[$from])) {
            //die("mySession::restore: '$from' not in session");
            if($setError) {
                $this->error = tl::tlx('Couldn\'t restore session var \'{$arg[1]}\'', $from);
            }
            return false;
        }
        $o = $_SESSION[$from];
        return true;
    }
    
    function updateRelayHost($relayHost) {
        $this->relayHost = $relayHost;
        $this->save($this->relayHost, "relayhost");
    }
    
    function checkWebCfg() {
        if(file_exists(Config::serverCfgFile)) {
            $mtime = filemtime(Config::serverCfgFile);
            if($mtime != $this->webCfg->mtime) {
                $this->updateWebCfg(new WebServerConfig());
            }
        }
    }
    
    function getApp($appName) {
        if(isset($this->apps[$appName])) {
            return $this->apps[$appName];
        }
        return null;
    }
    
    function updateWebCfg($cfg) {
        $this->webCfg = $cfg;
        $this->save($this->webCfg, "webcfg");
    }
    
    function updateApps() {
        $this->updateApplicationFiles();
        $this->save($this->apps, "apps");
    }
    
    function updateApplicationFiles()
    {
        $appCount = 0;
        $apps = array();
        if($dir = opendir(Config::webRoot."/".Config::applicationsDir)) {
            while ($fileTmp = readdir($dir)) {
                if (preg_match("/.*\.xml$/", $fileTmp)) {
                    $fullPath = Config::webRoot."/".Config::applicationsDir."/".$fileTmp;
                    $appName = substr($fileTmp, 0, strlen($fileTmp) - 4);
                    $app = $this->getApp($appName);
                    $mtime = filemtime($fullPath);
                    if(!isset($app) || $app->mtime != $mtime) { // new or modified application
                        $this->readApplicationFile($fileTmp, $fullPath, $appName, $mtime);
                    }
                    if(isset($this->apps[$appName])) {
                        $apps[$appName] = true;
                    }
                }
            }
            closedir($dir);
        }
        foreach($this->apps as $appName => $app) {
            if(!isset($apps[$appName])) {   // uninstalled application
                unset($this->apps[$appName]);
            }
        }
    }
    
    function readApplicationFile($fileName, $fullPath, $appName, $mtime) {
        if(isset($this->apps[$appName])) {
            unset($this->apps[$appName]);
        }
        $doc = new DOMDocument();
        $doc->encoding = "utf-8";
        $doc->preserveWhiteSpace = false;
        @$doc->load($fullPath);
        if($doc !== false) {
            $app = new Application();
            $app->name = $appName;
            $app->file = $fileName;
            $app->mtime = $mtime;
            $root = $doc->getElementsByTagName ("application");
            if($root->length > 0) {
                $root = $root->item(0);
                foreach($root->childNodes as $node) {
                    switch($node->nodeName) {
                        case "name":
                            $app->displayName = $node->textContent;
                            break;
                        case "directory":
                            $app->dir = $node->textContent;
                            break;
                        case "desc":
                            $app->desc = $node->textContent;
                            break;
                        case "href":
                            $app->href = $node->textContent;
                            break;
                        case "linux-username":
                            $app->user = $node->textContent;
                            break;
                        case "database":
                            foreach($node->attributes as $attrib) {
                                switch($attrib->name) {
                                    case "name": 
                                        $app->db->name = $attrib->value; 
                                        break;
                                    case "username": 
                                        $app->db->user = $attrib->value; 
                                        break;
                                }
                            }
                            break;
                        case "cronjob":
                            $cron = new XmlCronJob();
                            foreach($node->attributes as $attrib) {
                                switch($attrib->name) {
                                    case "username": 
                                        $cron->user = $attrib->value;
                                        break;
                                    case "script": 
                                        $cron->script = $attrib->value;
                                        break;
                                }
                            }
                            if($cron->script != "" && $cron->user != "") {
                                $app->cronJobs[] = $cron;
                            }
                            break;
                        case "packages":
                            foreach($node->childNodes as $package) {
                                $packageName = $package->textContent;
                                if($package->nodeName == "package" && $packageName != "" && array_search($packageName, $app->packages) === false) {
                                    $app->packages[] = $packageName;
                                }
                            }
                            break;
                        case "log-files":
                            foreach($node->childNodes as $log) {
                                if($log->nodeName == "log") {
                                    $logFile = new LogFile();
                                    $logFile->file = $log->textContent;
                                    foreach($log->attributes as $attrib) {
                                        switch($attrib->name) {
                                            case "title": $logFile->title = $attrib->value; break;
                                            case "no-clear": 
                                            {
                                                if($attrib->value == "1" || strtolower($attrib->value) == "true") {
                                                    $logFile->noClear = true;
                                                }
                                                else {
                                                    $logFile->noClear = false;
                                                }
                                            }   break;
                                        }
                                    }
                                    if($logFile->file != "") {
                                        $app->logFiles[] = $logFile;
                                    }
                                }
                            }
                            break;
                        case "config-files":
                            foreach($node->childNodes as $cfg) {
                                if($cfg->nodeName == "conf") {
                                    if($cfg->textContent != "") {
                                        $app->configFiles[] = $cfg->textContent;
                                    }
                                }
                            }
                            break;
                        case "backup":
                            foreach($node->childNodes as $command) {
                                if($command->nodeName == "command") {
                                    $cmd = new BackupCmd();
                                    $cmd->appName = $appName;
                                    foreach($command->attributes as $attrib) {
                                        switch($attrib->name)
                                        {
                                            case "title":
                                                $cmd->title = $attrib->value;
                                                break;
                                            case "cmd":
                                                $cmd->cmd = $attrib->value;
                                                break;
                                            case "script":
                                                $cmd->script = $attrib->value;
                                                break;
                                        }
                                    }
                                    if($cmd->cmd != "") {
                                        $app->backupCmds[$cmd->cmd] = $cmd;
                                    }
                                }
                            }
                            break;
                        case "restore":
                            foreach($node->childNodes as $command) {
                                if($command->nodeName == "command") {
                                    $cmd = new RestoreCmd();
                                    $cmd->appName = $appName;
                                    foreach($command->attributes as $attrib) {
                                        switch($attrib->name)
                                        {
                                            case "title":
                                                $cmd->title = $attrib->value;
                                                break;
                                            case "cmd":
                                                $cmd->cmd = $attrib->value;
                                                break;
                                            case "script":
                                                $cmd->script = $attrib->value;
                                                break;
                                        }
                                    }
                                    foreach($command->childNodes as $action) {
                                        if($action->nodeName == "action") {
                                            $cmd->actions[] = $action->textContent;
                                        }
                                    }
                                    if($cmd->cmd != "") {
                                        $app->restoreCmds[$cmd->cmd] = $cmd;
                                    }
                                }
                            }
                            break;
                        case "uninstall":
                            foreach($node->attributes as $attrib) {
                                switch($attrib->name) {
                                    case "script":
                                        $app->uninstallScript = $attrib->value;
                                        break;
                                }
                            }
                            break;
                        case "version":
                            $app->version = $node->textContent;
                            break;
                        case "hotfix":
                            $app->hotfix = $node->textContent;
                            break;
                        case "hotfix-offset":
                            $app->hotfixOffset = $node->textContent;
                            break;
                        case "lighttpd-auth":
                            $app->lighttpdAuth = ($node->textContent == "1" ? true : false) ;
                            break;
                        case "public-web-paths":
                            foreach($node->childNodes as $path) {
                                if($path->nodeName == "path") {
                                    $app->publicWebPaths[] = $path->textContent;
                                }
                            }
                            break;
                    }
                }
                if(strpos($app->dir, "innovaphone") === 0) {
                    $beta = file_exists(Config::webRoot."/".Config::applicationsDir."/{$app->name}/beta");
                    $dvl = file_exists(Config::webRoot."/".Config::applicationsDir."/{$app->name}/dvl");
                    $app->versionStr = MainInputPage::getVersionStr($app->version, $app->hotfix, $beta, $dvl);
                }
                else {
                    $app->versionStr = $app->version;
                }
                if($app->lighttpdAuth) {
                    $app->authFile = "/home/lighttpd/{$app->name}-htdigest.user";
                }
                $this->apps[$app->name] = $app;
            }
        }
    }
    
    // updates package file, this function is called after an install or uninstall of an application/hotfix
    function checkPackages() {
        $packages = array();
        if(file_exists(Config::rootfsPackages)) {
            $content = file_get_contents(Config::rootfsPackages);
            $content = str_replace("\r\n", "\n", $content);
            $packs = explode("\n", $content);
            foreach($packs as $p) {
                if($p != "") {
                    $packages[$p] = 1;
                }
            }
        }
        foreach($this->apps as $app) {
            foreach($app->packages as $package) {
                if(!isset($packages[$package])) {
                    $packages[$package] = 1;
                }
                else {
                    $packages[$package]++;
                }
            }
        }
        $cmd = "/usr/bin/sudo /usr/local/bin/config.sh packages ".Config::installedPackages." ";
        foreach($packages as $package => $value) {
            $cmd .= $package." ".$value." ";
        }
        $ret = MainMethods::callConfigSh($cmd, $resUpdate);
    }
}

function xmlentities($string)
{
    return str_replace ( array ( '&', '"', "'", '<', '>' ), array ( '&amp;' , '&quot;', '&apos;' , '&lt;' , '&gt;' ), $string );
}

function parse_log_file($logFile, &$log, &$error, $returnOnError = true, $reverseLog = false, $maxLines = 0) {
    if(file_exists($logFile)) {
        $installLog = file($logFile);
        if(count($installLog)) {
            if($reverseLog) {
                $installLog = array_reverse ($installLog);
            }
            $lineNum = 0;
            foreach ($installLog as $line) {
                //do not change "Installation was successful", it will be matched!
                if(stristr($line, "Installation was successful") != "" ) {
                    $log .= htmlentities_utf8(substr($line, 5)); // cut "LOG: "
                    $log = str_replace("\n", "<br>", $log);
                    $log .= MainMethods::performChecks();
                    return true;
                } else {
                    if(strpos($line, "ERROR: ") === 0 ) {
                        $error = tl::tl("Installation failed.");
                        $log .= "<div style=\"color:red\">".htmlentities_utf8(substr($line, 7))."</div>";
                        $log = str_replace("\n", "<br>", $log);
                        if($returnOnError) {
                            $log .= MainMethods::performChecks();
                            return false;
                        }
                        $lineNum++;
                    }
                    elseif(strpos($line, "LOG: ") === 0 ) {
                        $log .= htmlentities_utf8(substr($line, 5)); // cut "LOG: "
                        $lineNum++;
                    }
                }
                if($maxLines > 0 && $lineNum >= $maxLines) {
                    $log = str_replace("\n", "<br>", $log);
                    return true;
                }
            }
        } else {
            //$error = tl::tlx('{$arg[1]} file is empty', $logFile);
            return false;
        }
    } else {
        return false;
    }
    $log = str_replace("\n", "<br>", $log);
    $error="";
    return false;
}    

function get_rnd_iv($iv_len)
{
    $iv = '';
    while ($iv_len-- > 0) {
        $iv .= chr(mt_rand() & 0xff);
    }
    return $iv;
}

function tempdir($path, $prefix = "") {
    $count = 0;
    do {
        $tmpDir = tempnam($path, $prefix);
        unlink($tmpDir);
        $t = main_system("/bin/mkdir $tmpDir", $ret);
        $count++;
    } while($ret != 0 && $count < 5);
    return $tmpDir;
}

function rrmdir($dir) {
    if (is_dir($dir)) {
        $objects = scandir($dir);
        foreach ($objects as $object) {
            if ($object != "." && $object != "..") {
                if (filetype($dir."/".$object) == "dir") rrmdir($dir."/".$object); else unlink($dir."/".$object);
            }
        }
        reset($objects);
        rmdir($dir);
    }
}

function get_files_from_dir($path) {
    $files = array();
    if($dir = @opendir($path))
    {
        while($file = readdir($dir))
        {
            if (!is_dir($file) && $file != "." && $file != "..")
            {
                $files[] = $file;
            }
        }
        closedir($dir);
    }
    return $files;
}

function ends_with($haystack, $needle){
    // Recommended version, using strpos
    return strrpos($haystack, $needle) === strlen($haystack)-strlen($needle);
}

function main_system($cmd, &$ret) {
    ob_start();
    system($cmd, $ret);
    $out = ob_get_contents();
    ob_end_clean();
    return $out;
}

function str_insert($insertstring, $intostring, $offset) {
    $part1 = substr($intostring, 0, $offset);
    $part2 = substr($intostring, $offset);
    
    $part1 = $part1 . $insertstring;
    $whole = $part1 . $part2;
    return $whole;
}

function home_die($error, $script = "application.php") {
    if(!isset($_GET["error"])) {
        header("location: $script?error=".urlencode($error));
        exit;
    }
}

function disable_link($text, $hint = "") {
    return "<div style='color:grey' title=\"".htmlentities($hint)."\">$text</div>";
}

function get_dhcp_address($init = false) {
    global $mainSession;
    if($init || $mainSession == null) {
        $ipVals = MainMethods::retrieveIPVals();
        return $ipVals->ip;
    }
    if(!isset($_SERVER["SERVER_ADDR"]) || strpos($mainSession->dhcpAddr, $_SERVER["SERVER_ADDR"]) === false) {
        return get_dhcp_address(true);
    }    
    return $mainSession->dhcpAddr;
}

/**
 *
 * @global MainSession $mainSession
 * @param string $text Shown in the alarm/event list on the gateway
 * @param string $msg Shown as details string or parsed as XML if $xsl is set
 * @param int $errorCode
 * @param boolean $clear If set, alarm is cleared and alarm file deleted
 * @param AlarmServer $alarmServer use this AlarmServer instead of the default configured alarm server
 * @param string $xsl xsl file to display the alarm/event, if !="" $msg won't be modified with xmlentities
 * @param boolean $event if set, type="error"
 * @return null 
 */
function send_alarm($text, $msg, $errorCode, $clear = false, AlarmServer &$alarmServer = null, $xsl = "", $event = false, $app="") {
    /* @var $mainSession MainSession*/ global $mainSession;
    /* @var $server AlarmServer */$server = null;
    if($mainSession->webCfg->alarmServer->ip == "" && ($alarmServer == null || $alarmServer->ip == "")) return;
    if($mainSession->webCfg->alarmServer->ip != "") {   // if given, use configured alarm server
        $server = $mainSession->webCfg->alarmServer;
    }   
    else {                                              // otherwise use given alarm server
        $server = $alarmServer;
    }
    $code = Alarms::getFullCode($errorCode);
    $http = "http";
    if($server->https) $http .= "s";
    $uri = "$http://";
    $uri .= $server->ip;
    if($server->port != "") {
        $uri .= ":".$server->port;
    }
    $uri .= "/LOG0/FAULT/post-msg";

    $link = utf8_encode($uri);

    $datetime = date("U");
    $type = "alarm";
    if($clear) $type="clear";
    elseif($event) $type="error";
    $useDefaultXsl = true;
    if($xsl != "") {
        $useDefaultXsl = false;
    }
    else {
        $xsl = "fault_details.xsl";
    }
    $vars = "_fault_xml_post <fault type=\"$type\" severity=\"1\" active=\"false\" code=\"$code\" date=\"".$datetime."\" host=\"".  get_dhcp_address()."\" xsl=\"$xsl\"><text>";
    $vars .= xmlentities($text);
    $src = "LINUX";
    if($app != "") {
        $src .= "/$app";
    }
    $src = xmlentities($src);
    $vars .= "</text><source>$src</source><details><msg>";
    if($useDefaultXsl) {
        $vars .= xmlentities($msg);
    }
    else {
        $vars .= $msg;
    }
    $vars .= "</msg></details></fault>";

    $c = curl_init();
    curl_setopt($c, CURLOPT_HEADER, 1);
    //curl_setopt($c, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
    curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($c, CURLOPT_URL, $link);
    curl_setopt($c, CURLOPT_POST, 1);
    curl_setopt($c, CURLOPT_POSTFIELDS, $vars);
    curl_setopt($c, CURLOPT_HTTPAUTH, CURLAUTH_ANYSAFE);
    curl_setopt($c, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($c, CURLOPT_SSL_VERIFYHOST, 0);
    if($server->user != "") {
        curl_setopt($c, CURLOPT_USERPWD, $server->user.":".$server->pw);
    }

    $response = curl_exec($c);
    $httpCode = curl_getinfo($c, CURLINFO_HTTP_CODE);
    curl_close($c);
    if($httpCode == 200 && !$event) {
        $file = Alarms::alarmFile($errorCode);  // file to remember alarms to be able to clear them later
        if($clear) {
            if(file_exists($file)) {
                unlink($file);
            }
        }
        else {
            file_put_contents($file, "");
        }
    }
}

/**
 *
 * @param int $code
 * @param AlarmServer $alarmServer use this AlarmServer instead of the default configured alarm server
 */
function clear_alarm($code, AlarmServer &$alarmServer = null, $app = "") {
    $file = Alarms::alarmFile($code);
    if(file_exists($file)) {
        send_alarm("", "", $code, true, $alarmServer, "", false, $app);
    }
}

$mainSession = new MainSession(60*60*24);
if($mainSession->error) {
    $mainSession->destroy();
    home_die($mainSession->error);
}

// translations in "lang/"
$translator = new tl("en", dirname(__FILE__)."/lang", tl::encUtf8, tl::encUtf8);
// change language if necessary
$l = InputPageLanguageSelectorDropdownField::getDefaultedLanguage("en", "lang", "language", get_dhcp_address());
$translator->change($l);

require_once("postgresql.php");
$innovaphoneDb = null;
function get_innovaphone_db() {
    global $innovaphoneDb;
    if ($innovaphoneDb == null) {
        $innovaphoneDb = new DbPGSql("innovaphone", "innovaphone", "", "administration.php");
        if (!$innovaphoneDb->isOpen) {
            home_die(tl::tl("No postgresql db available."));
            exit;
        }
    }
    return $innovaphoneDb;
}

function close_innovaphone_db() {
    global $innovaphoneDb;
    if($innovaphoneDb) {
        $innovaphoneDb->close();
        $innovaphoneDb = null;
    }
}
?>